<?php

/*
 * Invision Power File Manager
 * Application launcher/wrapper
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.2
 * Last Modified: 16th February 2005
 *
 */

error_reporting(E_USER_WARNING | E_ERROR | E_PARSE);
set_magic_quotes_runtime(0);
set_time_limit(0);

require "./settings/settings.inc.php";

if ($settings["debug"] == true)
{
    error_reporting(E_ALL & ~E_NOTICE);
}

/* -----------------------------------------
               Program Execution
   ----------------------------------------- */

define("IPFM_RUNNING", true);   // Used for error checking

// Is the script directory setting valid?
if (file_exists($settings["script_dir"]) == false)
{
    // No! Then we have a problem, time to exit
    if (preg_match("/windows/i", php_uname()) == true)
    {
        die("The script directory setting is invalid");
    }
    else
    {
        die("The script directory setting is invalid.<br />Remember that on Unix based platforms directory names are case sensitive");
    }
}

require $settings["script_dir"] . "settings/templates.inc.php";
require $settings["script_dir"] . "settings/languages.inc.php";
require $settings["script_dir"] . "modules/common.lib.php";

require $settings["script_dir"] . "modules/db/" . $settings["dbtype"] . ".lib.php";

// And they're off
$debug["start"] = start_clock();
$link_id        = db_connect($settings["dbserver"], $settings["dbuser"], $settings["dbpass"], $settings["dbname"]);
$input          = clean_input();

// Enable HTTP no-cache headers if allowed
if ($settings["no_cache"] == true)
{
    header("HTTP/1.0 200 OK");
    header("HTTP/1.1 200 OK");
    header("Content-Type: text/html");                              // Output a content type header
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");               // Date in the past
    header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");  // Always modified
    header("Cache-Control: no-store, no-cache, must-revalidate");   // HTTP 1.1
    header("Cache-Control: post-check=0, pre-check=0", false);
    header("Pragma: no-cache");                                     // HTTP 1.0
}

// Get (and check) the users action and authorisation
if ($settings["require_auth"] == false && empty($link_id) == false)
{
    if (empty($_COOKIE[$settings["user_cookie"]]) == false)
    {
        $permissions = auth_user($_COOKIE[$settings["user_cookie"]], $_COOKIE[$settings["pass_cookie"]], true);
    }
    else if ($input["act"] != "login")
    {
        // Don't want to parse the queries on the login page otherwise we get errors

        // Use the guest login
        build_query($queries["get_guest_info"], array("UID" => $settings["default_userid"]));
        $data = db_query($queries["get_guest_info"], $link_id);

        if (db_num_rows($data, $link_id) > 0)
        {
            $tmp         = db_fetch_array($data, $link_id);
            $permissions = auth_user(trim($tmp["username"]), trim($tmp["password"]));
            if (empty($permissions) == false)
            {
                $found = true;
            }
       }

        // If the guest account isn't found show the login form
        if ($found == false)
        {
            $input["act"] = "login";
        }
    }
}
else
{
    $permissions = auth_user($_COOKIE[$settings["user_cookie"]], $_COOKIE[$settings["pass_cookie"]], true);
}

$scripts = array("reg" => "register", "pass" => "protect", "list" => "listing", "login" => "login", "source" => "source", "img" => "image", "card" => "popup", "notes" => "notes", "cp" => "usercp", "mv" => "move", "db" => "database", "ln" => "link", "edit" => "editor", "ext" => "extract", "err" => "error", "mp3" => "mp3");
$cards   = array("chmod" => "chmod", "fav" => "bookmark", "ren" => "rename", "del" => "delete", "leg" => "legend", "up" => "upload", "mup" => "multi-upload", "arc" => "archive", "new" => "mkfile", "mkdir" => "mkdir", "ver" => "verify", "err" => "error");

if (empty($link_id) == true)
{
    // Error connecting to the database;
    $input["act"] = "err";
}
else if (is_array($permissions) == false && $input["act"] != "reg")
{
    // Is the user logged in?
    $input["act"] = "login";
}
else if (isset($scripts[$input["act"]]) == false)
{
    // Is the users action valid?
    $input["act"] = "list";
}

if ($settings["offline"] == true && $permissions["offline"] != 1)
{
    // The page is offline and the user is not an admin, so show the login page
    $input["act"] = "login";
}

if ($input["act"] != "card")
{
    // Load the language pack if we aren't using a popup
    load_language($permissions["lang"], $scripts[$input["act"]]);
}
else
{
    if (isset($cards[$input["id"]]) == false)
    {
        // Is the card valid?
        $input["id"] = "err";
    }

    // Load the popup language pack
    load_language($permissions["lang"], $cards[$input["id"]]);
}

// Enable GZIP output compression if possible
$debug["gzip"] = $page["disabled"];
if (ini_get("zlib.output_compression") == true)
{
    $debug["gzip"] = $page["enabled"];
}
else if (phpversion() >= "4.0.4" && extension_loaded("zlib") && $settings["use_gzip"] == true)
{
    ob_end_clean();
    ob_start("ob_gzhandler");
    $debug["gzip"] = $page["enabled"];
}

// Quick and dirty hack to get rid of pages
if ($settings["perpage"] == 0)
{
    $settings["perpage"] = 1000000;
}

// Otherwise load the required module
require $settings["script_dir"] . "modules/" . $scripts[$input["act"]] . ".lib.php";

// Sort out the final page data
$page["tools"]     = tool_links();
$page["areas"]     = area_links();

$toolbar = "toolbar.tpl";
if (is_guest() == true)
{
    $toolbar = "guestToolbar.tpl";
}
$page["userlinks"] = static_template($settings["script_dir"] . "skin/" . $toolbar, $page);

$page["username"]  = $permissions["name"];
$page["groupname"] = $permissions["group"];

// If we are running on windows, remove all unix info
if (is_win() == true)
{
    $page["content"] = preg_replace("/<!-- BEGIN DYNAMIC BLOCK: unix -->(.+?)<!-- END DYNAMIC BLOCK: unix -->/si", "", $page["content"]);
}

// Sort out CSS, JS and wrapper files
$global_tpl        = isset($global_tpl) ? $global_tpl : "skin/global.tpl"; // If no global template is set, use the default
$page["js"]        = isset($page["js"]) ? $page["js"] : "";

echo static_template($settings["script_dir"] . $global_tpl, $page, 1);

db_close($link_id);

?>