<?php

/*
 * Invision Power File Manager
 * Language pack management
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.2
 * Last Modified: 14th February 2004
 *
 */

require $settings["script_dir"] . "settings/languages.inc.php";
require $settings["script_dir"] . "settings/sys/languages.inc.php";

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

$newStrings = 3; // Number of new string fields to show

/* -----------------------------------------
                Module Functions
   ----------------------------------------- */
// Opens a language files in limited scope
function open_language($lang, $file)
{
    global $settings, $lang_dir;

    if (file_exists($settings["script_dir"] . "languages/" . $lang_dir[$lang] . "/" . $file . ".lng.php") == true)
    {
        include $settings["script_dir"] . "languages/" . $lang_dir[$lang] . "/" . $file . ".lng.php";
    }

    if (isset($templates) == true)
    {
        return $templates["description"];
    }

    return $page;
}

// Removes a language directory
function remove_dir($dir)
{
    // Create arrays to store files and directories
    $_files      = array();
    $_subdirs    = array();

    if (($dp = opendir($dir)) == false)
    {
        // The directory could not be opened
        return false;
    }

    // Loop through all directory entries, and construct two temporary arrays containing files and sub directories
    while($entry = readdir($dp))
    {
        if (is_dir($dir . $entry) && $entry !=  ".." && $entry !=  ".")
        {
            $_subdirs[] = $entry;
        }
        else if ($entry !=  ".." && $entry !=  ".")
        {
            $_files[] = $entry;
        }
    }

    // Close directory
    closedir($dp);

    // Delete all files in the curent directory return false and halt if a file cannot be removed
    for($i=0; $i<count($_files); $i++)
    {
        @chmod($dir . $_files[$i], 0777);
        if (unlink($dir . $_files[$i]) == false)
        {
            return false;
        }
    }

    // Empty sub directories and then remove the directory
    for($i=0; $i<count($_subdirs); $i++)
    {
        @chmod($dir . $_subdirs[$i] . "/", 0777);

        if (remove_dir($dir . $_subdirs[$i] . "/") == false)
        {
            return false;
        }
    }
    @chmod($dir, 0777);
    return rmdir($dir);
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */
switch ($input["op"])
{
    case "list":
        /* ----------------------------
               List content of pack
           ---------------------------- */
        $page["section_title"]        = $page["section_title"] . $page["list_caption"];

        if (isset($lang_dir[$input["id"]]) == false || empty($lang_dir[$input["id"]]) == true)
        {
            $page["caption"] = $page["input_error"];
            $page["msg"]     = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {
            $page["created_by"] = preg_replace("/<%AUTHOR%>/i", $page["author_name"], $page["created_by"]);
            $page["created_by"] = preg_replace("/<%MAIL%>/i", $page["author_email"], $page["created_by"]);
            $page["header_description"] = preg_replace("/<%LANGNAME%>/i", $lang_name[$input["id"]], $input["new"] == 1 ? $page["new_by"] : $page["created_by"]);

            $page["id"]     = $input["id"];
            $page["name"]   = $lang_name[$input["id"]];

            $page["header"] = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);
            // Get the blocks from the template file and then parse them
            $blocks = get_blocks($settings["script_dir"] . "skin/admin/languages/list.tpl", array("sections"));

            $blocks["tmp"] = "";
            natcasesort($languages["sections"]);

            // Loops through entries
            foreach ($languages["sections"] as $key => $var)
            {
                if ($key > 0)
                {
                    $default = open_language($settings["default_lang"], $languages["files"][$key]);
                    $lang    = open_language($input["id"], $languages["files"][$key]);

                    $difference = count($default) - count($lang);
                    if ($difference > $languages["special"]["multi"])
                    {
                        $var = $var . " " . preg_replace("/<%MISSING%>/i", $difference, $page["missing_entries"]);
                    }

                    $data["entries"]      = count($lang);

                    $data["size"]         = "0 " . $page["bytes"];
                    if (file_exists($settings["script_dir"] . "languages/" . $lang_dir[$input["id"]] . "/" . $languages["files"][$key] . ".lng.php") == true)
                    {
                        $data["size"]     = format_bytes(filesize($settings["script_dir"] . "languages/" . $lang_dir[$input["id"]] . "/" . $languages["files"][$key] . ".lng.php"));
                    }
                }
                else
                {
                    $total           = 0;
                    $data["entries"] = 0;
                    foreach ($languages["multi"] as $file)
                    {
                        $file = $settings["script_dir"] . "languages/" . $lang_dir[$input["id"]] . "/" . $file . ".txt";

                        if (file_exists($file) == true)
                        {
                            $total += filesize($file);
                            $data["entries"]++;
                        }
                    }
                    $data["size"] = format_bytes($total);

                    $difference = count($languages["multi"]) - $data["entries"];
                    if ($difference > 0)
                    {
                        $var = $var . " " . preg_replace("/<%MISSING%>/i", $difference, $page["missing_entries"]);
                    }
                }
                $data["lid"]          = $input["id"];
                $data["id"]           = $key;
                $data["lang_section"] = $var;

                $blocks["tmp"] .= parse_html($data, $blocks["sections"]);
            }
            $blocks["sections"] = $blocks["tmp"];

            $config = open_language($input["id"], "config");
            $page["character_set"]  = $config["charset"];
            $page["encoding_value"] = $config["encoding"];
            $page["xml_lang_value"] = $config["lang"];
            $page["dir_dropdown"]   = dropdown("dir", array("ltr" => $page["left_to_right"], "rtl" => $page["right_to_left"]), $config["direction"]);

            $page["content"] = parse_html($page, dynamic_template($settings["script_dir"] . "skin/admin/languages/list.tpl", $blocks));
        }
    break;
    case "edit":
        $page["section_title"]        = $page["section_title"] . $page["edit_caption"];
        $page["header_description"]   =  str_replace("<%ID%>", $input["lid"], str_replace("<%LANG%>", $lang_name[$input["lid"]], $page["edit_description"]));

        /* ----------------------------
               Edit language file
           ---------------------------- */
        if (isset($lang_dir[$input["lid"]]) == false)
        {
            $page["caption"] = $page["input_error"];
            $page["msg"]     = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else if (isset($languages["files"][$input["id"]]) == false && $input["id"] != $languages["special"]["multi"])
        {
            $page["caption"] = $page["input_error"];
            $page["msg"]     = $page["invalid_langfile"] . "<br /><br />" . $page["go_back"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {
            $tpl = "edit.tpl";
            if ($input["lid"] == $settings["default_lang"])
            {
                $tpl = "editDefault.tpl";
            }

            $page["file_edit_caption"] = preg_replace("/<%FILE%>/i", $languages["sections"][$input["id"]], $page["file_edit_caption"]);

            $page["header"] = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

            $default = open_language($settings["default_lang"], $languages["files"][$input["id"]]);
            $lang    = open_language($input["lid"], $languages["files"][$input["id"]]);

            if ($input["id"] == $languages["special"]["templates"])
            {
                // Get the blocks from the template file and then parse them
                $blocks = get_blocks($settings["script_dir"] . "skin/admin/languages/" . $tpl, array("strings", "new", "newhead"));

                // User is editing file templates so rather than show the boring index keys show something of value
                require $settings["script_dir"] . "settings/templates.inc.php";

                $templates["extension"][0] = $settings["default_type"];

                $blocks["tmp"] = "";
                foreach ($default as $key => $var)
                {
                    $data["key"]         = $key;
                    if (isset($lang[$key]) == true)
                    {
                        $data["key_lbl"]  = $templates["extension"][$key];
                        $data["var"]      = make_safe($lang[$key]);
                        $data["checkbox"] = checkBox("inherit[" . $key . "]");
                    }
                    else
                    {
                        $data["key_lbl"]  = "<span class=\"default\">" . $templates["extension"][$key] . "</span>";
                        $data["var"]      = make_safe($var);
                        $data["checkbox"] = checkBox("inherit[" . $key . "]", true);
                    }

                    $blocks["tmp"] .= parse_html($data, $blocks["strings"]);
                }
                $blocks["strings"] = $blocks["tmp"];

                $blocks["new"]     = "";
                $blocks["newhead"] = "";

                $template = dynamic_template($settings["script_dir"] . "skin/admin/languages/" . $tpl, $blocks);
            }
            else if ($input["id"] == $languages["special"]["multi"])
            {
                $tpl = "multi.tpl";
                if ($input["lid"] == $settings["default_lang"])
                {
                    $tpl = "multiDefault.tpl";
                }

                // Get the blocks from the template file and then parse them
                $blocks = get_blocks($settings["script_dir"] . "skin/admin/languages/". $tpl, array("strings"));

                $blocks["tmp"] = "";
                foreach ($languages["multi"] as $key => $var)
                {
                    $data["key"]         = $key;
                    if (file_exists($settings["script_dir"] . "languages/" . $lang_dir[$input["lid"]] . "/" . $var . ".txt") == true)
                    {
                        $data["key_lbl"]  = $key;
                        $data["var"]      = make_safe(implode("", file($settings["script_dir"] . "languages/" . $lang_dir[$input["lid"]] . "/" . $var . ".txt")));
                        $data["checkbox"] = checkBox("inherit[" . $key . "]");
                    }
                    else
                    {
                        $data["key_lbl"]  = "<span class=\"default\">" . $key . "</span>";
                        $data["var"]      = make_safe(implode("", file($settings["script_dir"] . "languages/" . $lang_dir[$settings["default_lang"]] . "/" . $var . ".txt")));
                        $data["checkbox"] = checkBox("inherit[" . $key . "]", true);
                    }

                    $blocks["tmp"] .= parse_html($data, $blocks["strings"]);
                }
                $blocks["strings"] = $blocks["tmp"];

                $template = dynamic_template($settings["script_dir"] . "skin/admin/languages/". $tpl, $blocks);
            }
            else
            {
                // Get the blocks from the template file and then parse them
                $blocks = get_blocks($settings["script_dir"] . "skin/admin/languages/" . $tpl, array("strings", "new", "newhead"));

                // Loop through default file, if entry exists in user file print that one, otherwise print default in red
                $blocks["tmp"] = "";
                foreach ($default as $key => $var)
                {
                    $data["key"]         = $key;
                    if (isset($lang[$key]) == true)
                    {
                        $data["key_lbl"]  = $key;
                        $data["var"]      = make_safe($lang[$key]);
                        $data["checkbox"] = checkBox("inherit[" . $key . "]");
                    }
                    else
                    {
                        $data["key_lbl"]  = "<span class=\"default\">" . $key . "</span>";
                        $data["var"]      = make_safe($var);
                        $data["checkbox"] = checkBox("inherit[" . $key . "]", true);
                    }

                    $blocks["tmp"] .= parse_html($data, $blocks["strings"]);
                }
                $blocks["strings"] = $blocks["tmp"];

                if ($input["lid"] == $settings["default_lang"])
                {
                    // Populate the new fields
                    $blocks["tmp"] = "";
                    for ($i = 0; $i < $newStrings; $i++)
                    {
                        $page["i"]      = $i;
                        $blocks["tmp"] .= parse_html($page, $blocks["new"]);
                    }
                    $blocks["new"] = $blocks["tmp"];
                }
                else
                {
                    $blocks["new"]     = "";
                    $blocks["newhead"] = "";
                }

                $template = dynamic_template($settings["script_dir"] . "skin/admin/languages/" . $tpl, $blocks);
            }

            $page["id"]  = $input["id"];
            $page["lid"] = $input["lid"];

            $page["content"] = parse_html($page, $template);
        }
    break;
    case "save_edit":
        /* ----------------------------
               Save changes to file
           ---------------------------- */
        $page["section_title"]  = $page["section_title"] . $page["edit_caption"];
        $page["caption"]        = $page["input_error"];

        // Stop an error occuring later on when adding new entries
        if (is_array($input["newKey"]) == false)
        {
            $input["newKey"] = array();
        }

        if (isset($lang_dir[$input["lid"]]) == false)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else if (isset($languages["files"][$input["id"]]) == false && $input["id"] != $languages["special"]["multi"])
        {
            $page["msg"] = $page["invalid_langfile"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $page["header"] = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

            $error = false;

            if ($input["id"] == $languages["special"]["multi"])
            {
                // The user is editing the multi line entries
                foreach ($languages["multi"] as $key => $var)
                {
                    // Check that the variable isn't empty, it can be if the value is inherited and the language is not the default
                    if (empty($var) == true && ($input["lid"] == $settings["default_lang"] || ($input['inherit'][$key] != true && $input["lid"] == $settings["default_lang"])))
                    {
                        // One of the fields was empty
                        $error = true;
                        break;
                    }
                }

                if ($error == true)
                {
                    $page["msg"] = $page["missing_entry"] . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    foreach ($languages["multi"] as $key => $var)
                    {
                        $file = $settings["script_dir"] . "languages/" . $lang_dir[$input["lid"]] . "/" . $languages["multi"][$key] . ".txt";
                        if ($input["inherit"][$key] == false)
                        {
                            if (($fp = fopen($file, "w")) == false)
                            {
                                $error = $key;
                                break;
                            }
                            else
                            {
                                fwrite($fp, $input["lang"][$key]);
                                fclose($fp);
                            }
                        }
                        else
                        {
                            // The entry exists but we are inheriting it, so remove it
                            if (file_exists($file) == true && unlink($file) == false)
                            {
                                $error = $key;
                                break;
                            }
                        }
                    }

                    if ($error != false)
                    {
                        $page["caption"] = $page["server_error"];
                        $page["msg"]     = str_replace("<%FILE%>", $file, $page["error_editing"]) . "<br /><br />" . $page["go_back"];
                    }
                    else
                    {
                        $page["caption"] = $page["language_edited"];
                        $page["msg"]     = str_replace("<%ID%>", $input["lid"], str_replace("<%LANG%>", $lang_name[$input["lid"]], $page["language_saved"]));
                    }
                }
            }
            else
            {
                // The user is editing a "normal" language pack file
                $data  = "<?php\r\n\r\n";

                foreach ($input["lang"] as $key => $var)
                {
                    // Check that the variable isn't empty, it can be if the value is inherited and the language is not the default
                    if (empty($var) == true && ($input["lid"] == $settings["default_lang"] || ($input['inherit'][$key] != true && $input["lid"] == $settings["default_lang"])))
                    {
                        $error = true;
                        break;
                    }
                    else if ($input['inherit'][$key] == false)
                    {
                        $var = str_replace("\"", "&quot;", $var);
                        if ($input["id"] == $languages["special"]["templates"])
                        {
                            // This is a template description entry
                            $data .= "\$templates[\"description\"][" . $key  . "] = \"" . trim(make_unsafe($var, true)) . "\";\r\n";
                        }
                        else
                        {
                            // This is a normal language pack entry
                            $data .= "\$page[\"" . $key  . "\"] = \"" . trim(make_unsafe($var, true)) . "\";\r\n";
                        }
                    }
                }

                if ($error == true)
                {
                    $page["msg"] = $page["missing_entry"] . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    // Now we loop through the new entries, check for the key names
                    foreach ($input["newKey"] as $key => $var)
                    {
                        // Check that the variable isn't empty, it can be if the value is inherited and the language is not the default
                        if (empty($var) == false && (preg_match('/^[a-z0-9_]+$/i', $var) == false || isset($input["lang"][$var]) == true))
                        {
                            $error = true;
                            break;
                        }
                        else if (empty($input["newVar"][$key]) == false)
                        {
                            $input["newVar"][$key] = str_replace("\"", "&quot;", $input["newVar"][$key]);
                            $data .= "\$page[\"" . $var  . "\"] = \"" . trim(make_unsafe($input["newVar"][$key], true)) . "\";\r\n";
                        }
                    }

                    if ($error == true)
                    {
                        $page["msg"] = $page["language_key_error"] . "<br /><br />" . $page["go_back"];
                    }
                    else
                    {
                        $data .= "\r\n?>";

                        $file = $settings["script_dir"] . "languages/" . $lang_dir[$input["lid"]] . "/" . $languages["files"][$input["id"]] . ".lng.php";

                        if (($fp = fopen($file, "w")) == false)
                        {
                            $page["caption"] = $page["server_error"];
                            $page["msg"]     = str_replace("<%FILE%>", $file, $page["error_editing"]) . "<br /><br />" . $page["go_back"];
                        }
                        else
                        {
                            fwrite($fp, $data);
                            fclose($fp);

                            $page["caption"] = $page["language_edited"];
                            $page["msg"]     = str_replace("<%ID%>", $input["lid"], str_replace("<%LANG%>", $lang_name[$input["lid"]], $page["language_saved"]));
                        }
                    }
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "settings":
        /* ----------------------------
              Edit language settings
           ---------------------------- */
        $page["section_title"]  = $page["section_title"] . $page["edit_caption"];
        $page["caption"]        = $page["input_error"];
        if (isset($lang_dir[$input["id"]]) == false)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $default = open_language($input["id"], "config");

            if (empty($input["charset"]) == true)
            {
                $page["msg"] = $page["invalid_charset"] . "<br /><br />" . $page["go_back"];
            }
            else if (empty($input["encoding"]) == true)
            {
                $page["msg"] = $page["invalid_encoding"] . "<br /><br />" . $page["go_back"];
            }
            else if (empty($input["language"]) == true)
            {
                $page["msg"] = $page["invalid_xml"] . "<br /><br />" . $page["go_back"];
            }
            else if (empty($input["name"]) == true)
            {
                $page["msg"] = $page["invalid_lang_name"] . "<br /><br />" . $page["go_back"];
            }
            else if (in_array($input["name"], $lang_name) == true && $lang_name[$input["id"]] != $input["name"])
            {
                $page["msg"] = $page["lang_name_exists"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                if ($input["dir"] != "rtl")
                {
                    $input["dir"] = "ltr";
                }

                // Build the new language pack config
                $data  = "<?php\r\n\r\n";
                $data .= "\$page[\"author_name\"] = \"" . $page["author_name"] ."\";\r\n";
                $data .= "\$page[\"author_email\"] = \"" . $page["author_email"] . "\";\r\n";
                $data .= "\$page[\"charset\"] = \"" . make_unsafe($input["charset"], true) . "\";\r\n";
                $data .= "\$page[\"direction\"] = \"" . make_unsafe($input["dir"], true) . "\";\r\n";
                $data .= "\$page[\"encoding\"] = \"" . make_unsafe($input["encoding"], true) . "\";\r\n";
                $data .= "\$page[\"lang\"] = \"" . make_unsafe($input["language"], true) . "\";\r\n";
                $data .= "\r\n?>";

                // Rebuild the language configuration file
                $lang_name[$input["id"]] = $input["name"]; // Rename to the new name
                $conf = "<?php\r\n\r\n";
                foreach ($lang_dir as $key => $var)
                {
                    $conf .= "\$lang_dir[" . $key . "] = \"" . $var . "\";\r\n";
                    $conf .= "\$lang_name[" . $key . "] = \"" . $lang_name[$key] . "\";\r\n";
                }
                $conf .= "\r\n?>";

                $page["caption"] = $page["server_error"];
                // Write the changes
                $file = $settings["script_dir"] . "settings/languages.inc.php";
                if (($fp = fopen($file, "w")) == false)
                {
                    $page["msg"] = str_replace("<%FILE%>", $file, $page["error_editing"]) . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    // Write the language configuration file
                    fwrite($fp, $conf);
                    fclose($fp);

                    $file = $settings["script_dir"] . "languages/" . $lang_dir[$input["id"]] . "/config.lng.php";
                    if (($fp = fopen($file, "w")) == false)
                    {
                        $page["msg"] = str_replace("<%FILE%>", $file, $page["error_editing"]) . "<br /><br />" . $page["go_back"];
                    }
                    else
                    {
                        // Write the language properties file
                        fwrite($fp, $data);
                        fclose($fp);

                        $page["caption"] = $page["language_info_edited"];
                        $page["msg"]     = str_replace("<%ID%>", $input["id"], str_replace("<%LANG%>", $lang_name[$input["id"]], $page["language_info_saved"]));
                    }
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "del":
        /* ----------------------------
               Delete language pack
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["delete_caption"];
        $page["caption"]       = $page["input_error"];
        if (isset($lang_dir[$input["id"]]) == false)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else if ($input["id"] == $settings["default_lang"])
        {
            $page["msg"] = $page["no_delete_default"] . "<br /><br />" . $page["go_back"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {
            // Remove the language pack
            $page["header_description"] = str_replace("<%LANG%>", $lang_name[$input["id"]], $page["delete_description"]);
            $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

            // Populate the fields
            $page["id"]                 = $input["id"];

            $page["content"]            = static_template($settings["script_dir"] . "skin/admin/languages/delete.tpl", $page);
        }
    break;
    case "remove":
        /* ----------------------------
             Remove the language pack
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["delete_caption"];
        $page["caption"]       = $page["input_error"];
        if (isset($lang_dir[$input["id"]]) == false)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else if ($input["id"] == $settings["default_lang"])
        {
            $page["msg"] = $page["no_delete_default"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Now to rebuild the language config file
            $conf = "<?php\r\n\r\n";
            foreach ($lang_dir as $key => $var)
            {
                if ($key != $input["id"])
                {
                    $conf .= "\$lang_dir[" . $key . "] = \"" . $var . "\";\r\n";
                    $conf .= "\$lang_name[" . $key . "] = \"" . $lang_name[$key] . "\";\r\n";
                }
            }
            $conf .= "\r\n?>";

            $page["caption"] = $page["server_error"];
            // Write the changes
            $file = $settings["script_dir"] . "settings/languages.inc.php";
            if (($fp = fopen($file, "w")) == false)
            {
                $page["msg"] = str_replace("<%FILE%>", $file, $page["error_deleting"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                // Write the language configuration file
                fwrite($fp, $conf);
                fclose($fp);

                $page["caption"] = $page["language_deleted"];     
                $page["msg"] = $page["language_delete_saved"];
               
                if ($input["full"] != 1)
                {
                    require $settings["script_dir"] . "modules/admin/tar.php";

                    $tar = new export();
                    $tar->new_tar($settings["script_dir"] . "languages" , $lang_name[$input["id"]] . "(" . $lang_dir[$input["id"]] . ").tar");
                    $tar->add_directory($settings["script_dir"] . "languages/" . $lang_dir[$input["id"]]);               
                    $tar->write_tar();                                 
                }                                                
                
                if ($input["full"] == 1 || ($input["full"] != 1 && $tar->error == ""))
                {
                    // Remove field is set, so remove the files
                    if (remove_dir($settings["script_dir"] . "languages/" . $lang_dir[$input["id"]] . "/") == false)
                    {
                        if ($input["full"] == 1)
                        {
                            $page["msg"] = $page["language_delete_part"];
                        }
                        else
                        {
                            $page["msg"] = $page["language_delete_saved_part"];
                        }
                    }
                    else
                    {
                        if ($input["full"] == 1)
                        {
                            $page["msg"] = $page["language_delete_full"];
                        }
                    }
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "new":
        /* ----------------------------
                New language pack
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["new_caption"];
        $page["header_description"] = $page["new_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $default                = open_language($settings["default_lang"], "config");
        $page["author"]         = $permissions["name"];
        $page["email"]          = $permissions["email"];
        $page["character_set"]  = $default["charset"];
        $page["encoding_value"] = $default["encoding"];
        $page["xml_lang_value"] = $default["lang"];
        $page["dir_dropdown"]   = dropdown("dir", array("ltr" => $page["left_to_right"], "rtl" => $page["right_to_left"]), $default["direction"]);


        $page["content"]        = static_template($settings["script_dir"] . "skin/admin/languages/add.tpl", $page);
    break;
    case "add":
        /* ----------------------------
              Create the new language
           ---------------------------- */
        $page["section_title"]  = $page["section_title"] . $page["add_caption"];
        $page["caption"]        = $page["input_error"];
        if (in_array($input["target"], $lang_dir) == true || (file_exists($settings["script_dir"] . "languages/" . $input["target"] . "/") == true && is_dir($settings["script_dir"] . "languages/" . $input["target"] . "/") == true))
        {
            $page["msg"] = $page["directory_name_exists"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[a-z0-9_\-]+$/i", $input["target"]) == false)
        {
            $page["msg"] = $page["directory_name_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["name"]) == true)
        {
            $page["msg"] = $page["no_lang_name"] . "<br /><br />" . $page["go_back"];
        }
        else if (in_array($input["name"], $lang_name) == true)
        {
            $page["msg"] = $page["lang_name_exists"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["author"]) == true)
        {
            $page["msg"] = $page["no_author"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[^@ ]+@[^@ ]+[\.][^@ \.]+$/", $input["email"]) == false)
        {
            $page["msg"] = $page["invalid_email"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["charset"]) == true)
        {
            $page["msg"] = $page["invalid_charset"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["encoding"]) == true)
        {
            $page["msg"] = $page["invalid_encoding"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["language"]) == true)
        {
            $page["msg"] = $page["invalid_xml"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["name"]) == true)
        {
            $page["msg"] = $page["invalid_lang_name"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            if ($input["dir"] != "rtl")
            {
                $input["dir"] = "ltr";
            }

            $page["caption"] = $page["server_error"];
            if (mkdir($settings["script_dir"] . "languages/" . $input["target"] . "/", 0777) == false || mkdir($settings["script_dir"] . "languages/" . $input["target"] . "/admin/", 0777) == false)
            {
                $page["msg"] = $page["dir_not_made"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                // Build the new language pack config
                $data  = "<?php\r\n\r\n";
                $data .= "\$page[\"author_name\"] = \"" . make_unsafe($page["author"], true) ."\";\r\n";
                $data .= "\$page[\"author_email\"] = \"" . make_unsafe($page["email"], true) . "\";\r\n";
                $data .= "\$page[\"charset\"] = \"" . make_unsafe($input["charset"], true) . "\";\r\n";
                $data .= "\$page[\"direction\"] = \"" . make_unsafe($input["dir"], true) . "\";\r\n";
                $data .= "\$page[\"encoding\"] = \"" . make_unsafe($input["encoding"], true) . "\";\r\n";
                $data .= "\$page[\"lang\"] = \"" . make_unsafe($input["language"], true) . "\";\r\n";
                $data .= "\r\n?>";

                // Rebuild the language configuration file
                // Add the new entries
                $lang_dir[]  = $input["target"];
                $lang_name[] = $input["name"];
                $conf = "<?php\r\n\r\n";
                foreach ($lang_dir as $key => $var)
                {
                    $conf .= "\$lang_dir[" . $key . "] = \"" . $var . "\";\r\n";
                    $conf .= "\$lang_name[" . $key . "] = \"" . $lang_name[$key] . "\";\r\n";
                }
                $conf .= "\r\n?>";

                // Write the changes
                $file = $settings["script_dir"] . "settings/languages.inc.php";
                if (($fp = fopen($file, "w")) == false)
                {
                    $page["msg"] = str_replace("<%FILE%>", $file, $page["error_creating"]) . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    // Write the language configuration file
                    fwrite($fp, $conf);
                    fclose($fp);

                    $file = $settings["script_dir"] . "languages/" . $input["target"] . "/config.lng.php";
                    if (($fp = fopen($file, "w")) == false)
                    {
                       $page["msg"] = str_replace("<%FILE%>", $file, $page["error_creating"]) . "<br /><br />" . $page["go_back"];
                    }
                    else
                    {
                        // Write the language properties file
                        fwrite($fp, $data);
                        fclose($fp);
                        go_to($settings["script_url"] . "admin.php?act=lng&op=list&new=1&id=" . $key);
                    }
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "export":
        /* ----------------------------
                Export a language
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["export_caption"];
        $page["caption"]       = $page["input_error"];
        if (isset($lang_dir[$input["id"]]) == false)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $page["caption"] = $page["server_error"];
            require $settings["script_dir"] . "modules/admin/tar.php";

            $tar = new export();
            $tar->new_tar($settings["script_dir"] . "languages" , $lang_name[$input["id"]] . "(" . $lang_dir[$input["id"]] . ").tar");

            if ($tar->error != "")
            {
                $page["msg"] = $tar->error . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $tar->add_directory($settings["script_dir"] . "languages/" . $lang_dir[$input["id"]]);
                if ($tar->error != "")
                {
                    $page["msg"] = $tar->error . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    $tar->download_archive();
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "import":
        /* ----------------------------
              Import languages form
           ---------------------------- */
        $found = array();

        $page["section_title"] = $page["section_title"] . $page["import_caption"];
        $page["caption"]       = $page["server_error"];

        if (($dp = opendir($settings["script_dir"] . "languages/")) == true)
        {
            while ($entry = readdir($dp))
            {
                if (preg_match("/\.tar$/i", $entry) == true)
                {
                    $found[] = $entry;
                }
            }

            closedir($dp);

            if (count($found) == 0)
            {
                $page["caption"] = $page["import_caption"];
                $page["msg"]     = $page["none_to_import"];
                $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
            }
            else
            {
                $page["header_description"]   = $page["import_description"];
                $page["header"] = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

                // Get the blocks from the template file and then parse them
                $blocks = get_blocks($settings["script_dir"] . "skin/admin/languages/import.tpl", array("archive"));

                foreach ($found as $var)
                {
                    $name = substr($var, 0, strlen($var) - 4);
                    $name = str_replace(")", "", $name); // Do this instead of taking 5 characters off because not all
                                                         // files will have a ) so we don't want to cut part of the name off
                    $name = explode("(", $name);

                    $action = "im";

                    $data["filename"] = $var;
                    $data["langname"] = $name[0];
                    $data["langdir"]  = empty($name[1]) == true ? $name[0] : $name[1];

                    if (in_array($data["langname"], $lang_name) == true)
                    {
                        $action = "ig";
                        $data["langname"] .= "\" style=\"color: red;\"";
                    }

                    if (in_array($data["langdir"], $lang_dir) == true || (file_exists($settings["script_dir"] . "languages/" . $data["langdir"] . "/") == true && is_dir($settings["script_dir"] . "languages/" . $data["langdir"] . "/") == true))
                    {
                        $action = "ig";
                        $data["langdir"] .= "\" style=\"color: red;\"";
                    }

                    $data["actions"]  = dropdown("action[]", array("im" => "Import", "del" => "Delete", "ig" => "Ignore"), $action);

                    $blocks["tmp"] .= parse_html($data, $blocks["archive"]);
                }
                $blocks["archive"] = $blocks["tmp"];

                $page["content"] = parse_html($page, dynamic_template($settings["script_dir"] . "skin/admin/languages/import.tpl", $blocks));
            }
        }
        else
        {
            $page["msg"]     = $page["language_dir_open"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
    break;
    case "do_import":
        /* ----------------------------
               Import the languages
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["import_caption"];
        $page["caption"]       = $page["server_error"];
           
        require $settings["script_dir"] . "modules/admin/tar.php";
        if (is_array($input["archive"]) == false || count($input["archive"]) == false)
        {
            $page["msg"] = $page["import_no_data"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $count = count($input["archive"]);
            if (count($input["name"]) != $count || count($input["dir"]) != $count || count($input["action"]) != $count)
            {
                $page["msg"] = $page["invalid_data"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                // Loop through the input and check for errors
                foreach ($input["archive"] as $key => $var)
                {
                    if (empty($input["name"][$key]) == true && $input["action"][$key] == "im")
                    {
                        $page["msg"] = $page["no_lang_name"] . "<br /><br />" . $page["go_back"];
                        break;
                    }
                    else if (in_array($input["name"][$key], $lang_name) == true && $input["action"][$key] == "im")
                    {
                        $page["msg"] = $page["lang_name_exists"] . "<br /><br />" . $page["go_back"];
                        break;
                    }
                    else if (file_exists($settings["script_dir"] . "languages/" . $input["dir"][$key] . "/") == true && is_dir($settings["script_dir"] . "languages/" . $input["dir"][$key] . "/") == true && $input["action"][$key] == "im")
                    {
                        $page["msg"] = $page["directory_name_exists"] . "<br /><br />" . $page["go_back"];
                        break;
                    }
                    else if (preg_match("/^[a-z0-9_\-]+$/i", $input["dir"][$key]) == false && $input["action"][$key] == "im")
                    {
                        $page["msg"] = $page["directory_name_invalid"] . "<br /><br />" . $page["go_back"];
                        break;
                    }
                }

                if (isset($page["msg"]) == false)
                {
                    foreach ($input["archive"] as $key => $var)
                    {
                        if ($input["action"][$key] == "del")
                        {
                            // Delete archive
                            if (unlink($settings["script_dir"] . "languages/" . $var) == false)
                            {
                                $page["msg"] = str_replace("<%FILE%>", $var, $page["error_deleting"]) . "<br /><br />" . $page["go_back"];
                                break;
                            }
                        }
                        else if ($input["action"][$key] == "im")
                        {
                            if (mkdir($settings["script_dir"] . "languages/" . $input["dir"][$key] . "/", 0777) == false || mkdir($settings["script_dir"] . "languages/" . $input["dir"][$key] . "/admin/", 0777) == false)
                            {
                                // Make the directory for the archive
                                $page["msg"] = $page["dir_not_made"] . "<br /><br />" . $page["go_back"];
                                break;
                            }
                            else
                            {
                                // Extract the archive
                                $tar = new tar();
                                $tar->new_tar($settings["script_dir"] . "languages", $var);
                                $tar->extract_files($settings["script_dir"] . "languages/" . $input["dir"][$key]);
                                
                                if ($tar->error != "")
                                {
                                    $page["msg"] = $tar->error . "<br /><br />" . $page["go_back"];
                                    break;
                                }

                                unset($tar);    
                                unlink($settings["script_dir"] . "languages/" . $var);
                                
                                // Add the new entries
                                $lang_dir[]  = $input["dir"][$key];
                                $lang_name[] = $input["name"][$key];                                                                                                    
                            }
                        }
                    }                                

                    // Rebuild the language configuration file
  
                    $conf = "<?php\r\n\r\n";
                    foreach ($lang_dir as $key => $var)
                    {
                        $conf .= "\$lang_dir[" . $key . "] = \"" . $var . "\";\r\n";
                        $conf .= "\$lang_name[" . $key . "] = \"" . $lang_name[$key] . "\";\r\n";
                    }
                    $conf .= "\r\n?>";

                    // Write the changes
                    $file = $settings["script_dir"] . "settings/languages.inc.php";
                    if (($fp = fopen($file, "w")) == false)
                    {
                        $page["msg"] = str_replace("<%FILE%>", $file, $page["error_editing"]) . "<br /><br />" . $page["go_back"];
                    }
                    else
                    {
                        // Write the language configuration file
                        fwrite($fp, $conf);
                        fclose($fp);
                        
                        if (isset($page["msg"]) == false)
                        {
                        
                            $page["caption"] = $page["import_caption"];
                            $page["msg"]     = $page["language_imported"];
                        }
                    }
                }
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    default:
        /* ----------------------------
                Manage languages
           ---------------------------- */
        $page["section_title"]        = $page["section_title"] . $page["manage_caption"];
        $page["header_description"]   = $page["manage_description"];
        $page["header"]               = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Get the blocks from the template file and then parse them
        $blocks = get_blocks($settings["script_dir"] . "skin/admin/languages/manage.tpl", array("languages"));

        $blocks["tmp"] = "";
        foreach ($lang_name as $key => $var)
        {
            $data["langname"]      = $var;
            $data["id"]            = $key;
            $data["del"]           = "<a href=\"admin.php?act=lng&amp;op=del&amp;id=" . $key . "\">" . $page["delete"] . "</a>";
            if ($key == $settings["default_lang"])
            {
                $data["del"]       = $page["na"];
                $data["langname"] .= " " . $page["default_lang"];
            }

            $blocks["tmp"] .= parse_html($data, $blocks["languages"]);
        }
        $blocks["languages"] = $blocks["tmp"];

        // Populate the fields
        $page["lang_dropdown"] = dropdown("id", $lang_name, $settings["default_lang"]);

        $page["content"] = parse_html($page, dynamic_template($settings["script_dir"] . "skin/admin/languages/manage.tpl", $blocks));
}

$page["js"] = "adlang";

?>