<?php

/*
 * Invision Power File Manager
 * Script settings screen
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.2
 * Last Modified: 26th January 2005
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

$newTypes = 3; // Number of new filetype fields to show

/* -----------------------------------------
               Module Functions
   ----------------------------------------- */

// Updates the settings file
function update_settings($updated)
{
    // Import the current values
    global $settings;

    // Generate the list of setting arrays which depend upon their keys
    $useKeys = array("icons", "filters");

    $toWrite = "<?php\r\n\r\n";

    foreach ($settings as $key => $var)
    {
        if (isset($updated[$key]) == true)
        {
            // A new value exists
            $data = $updated[$key];
        }
        else
        {
            // Use the old value
            $data = $var;
        }

        // Silly hack, damn slashes
        if ($key == "format")
        {
            $data = str_replace("<%\A\T%>", "<%\\A\\\T%>", $data);
        }

        // If the data is a string or array tidy it up
        if (is_array($data) == true)
        {
            if (in_array($key, $useKeys) == true)
            {
                $data = array_to_string($data, true);
            }
            else
            {
                $data = array_to_string($data);
            }
        }
        else if ($data === true)
        {
            // Check for value and type of true
            $data = 'true';
        }
        else if ($data === false)
        {
            // Check for value and type of false
            $data = 'false';
        }
        else
        {
            if (preg_match("/^[.0-9]+$/", $data) == false)
            {
                $data = '"' . str_replace('"', '\"', $data) . '"';
            }
        }

        $toWrite .= "\$settings[\"" . $key . "\"] = " . $data . ";\r\n";
    }

    $toWrite .= "\r\n?>";

    if (($fp = fopen($settings["script_dir"] . "settings/settings.inc.php", "w")) == false)
    {
        return false;
    }
    else
    {
        file_lock($fp);
        fwrite($fp, $toWrite);
        file_lock($fp, LOCK_UN);
        fclose($fp);

        return true;
    }
}

// Converts an array into a string to be written to the settings file
function array_to_string($info, $useKeys = false)
{
    $data = "array(";

    foreach ($info as $key => $var)
    {
        if ($useKeys == true)
        {
            $data .= "\"" . $key . "\" => ";
        }

        $var = str_replace('"', '\"', $var);

        $data .= "\"" . $var . "\", ";
    }

    $data .= ")";

    // Clean up the extra ,
    return preg_replace("/, \)$/", ")", $data);
}

// A function to check is a string value is a valid RGB value
function is_rgb($array)
{
    foreach ($array as $var)
    {
        $var = (real) $var; // Set the value to an integer

        if (is_real($var) == false)
        {
            // The value is not a whole number, it is either a string or decimal
            return false;
        }
        else if ($var < 0 || $var > 255)
        {
            // The value is too high or too low
            return false;
        }
    }

    // All items have been checked so the values are valid
    return true;
}

// A pair of functions to convert an RGB value to a hex string
function RGBtoHex($red, $green, $blue)
{
    $a = HexValue(floor($red / 16));
    $b = HexValue($red % 16);
    $c = HexValue(floor($green / 16));
    $d = HexValue($green % 16);
    $e = HexValue(floor($blue / 16));
    $f = HexValue($blue % 16);

    return $a . $b . $c . $d . $e . $f;
}

function HexValue($dec)
{
    $hex = array("10" => "A", "11" => "B", "12" => "C", "13" => "D", "14" => "E", "15" => "F");

    foreach($hex as $key => $var)
    {
        if ($key == $dec)
        {
            return $var;
        }
    }
    return $dec;
}

// Simple function to split an RGB array into a string
function RGBtoCSS($colours)
{
    return $colours[0] . "," . $colours[1] . "," . $colours[2];
}

// Creates a HTML drop down list from an array
function imageDropdown($name, $items, $id, $selected = 0)
{
    $list = "<select name=\"" . $name . "\" onchange=\"javascript:updateIcon(" . $id . ", this);\" onblur=\"javascript:updateIcon(" . $id . ");\">";

    foreach ($items as $key => $var)
    {
        if ($key == $selected)
        {
            $list .= "<option value=\"" . $key . "\" selected=\"selected\">" . $var . "</option>";
        }
        else
        {
            $list .= "<option value=\"" . $key . "\">" . $var . "</option>";
        }
    }

    $list .= "</select>";

    return $list;
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */
$path = $settings["script_dir"] . "settings/settings.inc.php";
if (is_win() == true)
{
    $path = preg_replace("/[\/]/", "\\", $path);
}

switch ($input["op"])
{
    case "cookie":
        /* ----------------------------
                Cookie settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["cookies_caption"];
        $page["header_description"] = $page["cookies_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the settings
        $page["user_cookie"]        = $settings["user_cookie"];
        $page["pass_cookie"]        = $settings["pass_cookie"];
        $page["filter_cookie"]      = $settings["filter_cookie"];
        $page["admin_user_cookie"]  = $settings["admin_user_cookie"];
        $page["admin_pass_cookie"]  = $settings["admin_pass_cookie"];
        $page["cookie_time"]        = empty($settings["cookie_time"]) == true ? "" : $settings["cookie_time"];

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/cookies.tpl", $page);
    break;
    case "save_cookie":
        /* ----------------------------
                Save cookie settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["cookies_caption"];
        $page["caption"]            = $page["input_error"];

        // Ensure cookies start with a letter or underscore character and the entire name contains only A-Z, 0-9 or _
        if (preg_match("/^[a-z_][a-z0-9_]+$/i", $input["user_cookie"]) == false)
        {
            $page["msg"] = $page["invalid_user_cookie"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[a-z_][a-z0-9_]+$/i", $input["pass_cookie"]) == false)
        {
            $page["msg"] = $page["invalid_pass_cookie"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[a-z_][a-z0-9_]+$/i", $input["filter_cookie"]) == false)
        {
            $page["msg"] = $page["invalid_filter_cookie"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[a-z_][a-z0-9_]+$/i", $input["admin_user_cookie"]) == false)
        {
            $page["msg"] = $page["invalid_admin_user_cookie"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[a-z_][a-z0-9_]+$/i", $input["admin_pass_cookie"]) == false)
        {
            $page["msg"] = $page["invalid_admin_pass_cookie"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["cookie_time"]) == false && preg_match("/^[0-9]+$/", $input["cookie_time"]) == false)
        {
            // Check that if the cookie time is set that it contains only valid characters (0-9)
            $page["msg"] = $page["invalid_cookie_timeout"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("user_cookie"       => $input["user_cookie"],
                             "pass_cookie"       => $input["pass_cookie"],
                             "filter_cookie"     => $input["filter_cookie"],
                             "admin_user_cookie" => $input["admin_user_cookie"],
                             "admin_pass_cookie" => $input["admin_pass_cookie"],
                             "admin_menu_cookie" => $input["admin_menu_cookie"],
                             "cookie_time"       => empty($input["cookie_time"]) == true ? 0 : (int) $input["cookie_time"]
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["cookie_settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "charts":
        /* ----------------------------
                Pie chart settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["charts_caption"];
        $page["header_description"] = $page["charts_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the chart lists
        $charts = array("free", "used", "folder", "img", "mm", "db", "bin", "off", "arc", "txt", "misc");
        foreach ($charts as $chart)
        {
            $page[$chart . "_r"]    = $settings[$chart . "_chart"][0];
            $page[$chart . "_g"]    = $settings[$chart . "_chart"][1];
            $page[$chart . "_b"]    = $settings[$chart . "_chart"][2];
            $page[$chart . "_rgb"]  = RGBtoCSS($settings[$chart . "_chart"]);
            $page[$chart . "_hex"]  = RGBtoHex($settings[$chart . "_chart"][0], $settings[$chart . "_chart"][1], $settings[$chart . "_chart"][2]);
        }

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/charts.tpl", $page);
    break;
    case "save_charts":
        /* ----------------------------
             Save pie chart settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["charts_caption"];
        $page["caption"]            = $page["input_error"];

        // Loop over the colour fields to make sure the are between 0 and 255
        $error = false;
        foreach ($input as $key => $var)
        {
            if (preg_match("/_(red|green|blue)$/", $key) == true)
            {
                // OK so the field is part of "the grid" so check it
                if (preg_match("/[0-9]{1,3}$/", $var) == false || $var < 0 || $var > 255)
                {
                    // Error found, no point in checking the rest
                    $error = true;
                    break;
                }
            }
        }

        if ($error == true)
        {
            $page["msg"] = $page["rgb_error"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("free_chart"   => array($input["free_red"], $input["free_green"], $input["free_blue"]),
                             "used_chart"   => array($input["used_red"], $input["used_green"], $input["used_blue"]),
                             "folder_chart" => array($input["folder_red"], $input["folder_green"], $input["folder_blue"]),
                             "img_chart"    => array($input["img_red"], $input["img_green"], $input["img_blue"]),
                             "mm_chart"     => array($input["mm_red"], $input["mm_green"], $input["mm_blue"]),
                             "db_chart"     => array($input["db_red"], $input["db_green"], $input["db_blue"]),
                             "bin_chart"    => array($input["bin_red"], $input["bin_green"], $input["bin_blue"]),
                             "off_chart"    => array($input["off_red"], $input["off_green"], $input["off_blue"]),
                             "arc_chart"    => array($input["arc_red"], $input["arc_green"], $input["arc_blue"]),
                             "txt_chart"    => array($input["txt_red"], $input["txt_green"], $input["txt_blue"]),
                             "misc_chart"   => array($input["misc_red"], $input["misc_green"], $input["misc_blue"])
                       );
            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "email":
        /* ----------------------------
                 Email settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["email_caption"];
        $page["header_description"] = $page["email_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["mail_method"]        = dropdown("mail_type", array("smtp" => "Simple mail transfer protocol (SMTP)", "php" => "PHP mail()"), $settings["mail_type"]);
        $page["admin_out_email"]    = $settings["admin_out_email"];
        $page["admin_in_email"]     = $settings["admin_in_email"];
        $page["admin_name"]         = $settings["admin_name"];
        $page["smtp_server"]        = $settings["smtp_server"];
        $page["smtp_port"]          = $settings["smtp_port"];
        $page["smtp_username"]      = $settings["smtp_username"];
        $page["smtp_password"]      = $settings["smtp_password"];

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/email.tpl", $page);
    break;
    case "save_email":
        /* ----------------------------
                Save email settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["email_caption"];
        $page["caption"]            = $page["input_error"];

        if (strlen(trim($input["admin_name"])) == 0)
        {
            // Can't use empty(trim($var))
            $page["msg"] = $page["no_admin_name"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["admin_in_email"])) < 6 || preg_match("/^[^@ ]+@[^@ ]+[\.][^@ \.]+$/", $input["admin_in_email"]) == false)
        {
            $page["msg"] = $page["invalid_admin_in_email"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["admin_out_email"])) < 6 || preg_match("/^[^@ ]+@[^@ ]+[\.][^@ \.]+$/", $input["admin_out_email"]) == false)
        {
            $page["msg"] = $page["invalid_admin_out_email"] . "<br /><br />" . $page["go_back"];
        }
        else if ($input["mail_type"] == "smtp" && strlen(trim($input["smtp_server"])) == 0)
       {
            $page["msg"] = $page["no_smtp_server"] . "<br /><br />" . $page["go_back"];
        }
        else if ($input["mail_type"] == "smtp" && (strlen(trim($input["smtp_port"])) == 0 || preg_match("/^[0-9]+$/", $input["smtp_port"]) == false))
        {
            $page["msg"] = $page["invalid_smtp_port"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("mail_type"        => $input["mail_type"] == "smtp" ? "smtp" : "php",
                             "admin_out_email"  => $input["admin_out_email"],
                             "admin_in_email"   => $input["admin_in_email"],
                             "admin_name"       => $input["admin_name"],
                             "smtp_server"      => $input["smtp_server"],
                             "smtp_port"        => (int) $input["smtp_port"],
                             "smtp_username"    => $input["smtp_username"],
                             "smtp_password"    => $input["smtp_password"]
                       );


            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "reg":
        /* ----------------------------
              Registration settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["reg_caption"];
        $page["header_description"] = $page["reg_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["reg_input"]          = dualRadio("allow_reg", $settings["allow_reg"]);
        $page["bot_input"]          = dualRadio("security_check", $settings["security_check"]);
        $page["image_input"]        = dropdown("security_type", array("gd" => $page["dynamic_img"], "gif" => $page["static_img"]), $settings["security_type"]);
        $page["gd_input"]           = dropdown("gd_version", array(1 => "1", 2 => "2"), $settings["gd_version"]);
        $page["security_timeout"]   = $settings["security_timeout"];
        $page["email_timeout"]      = $settings["email_timeout"];
        $page["new_path"]           = $settings["new_path"];
        $page["new_url"]            = $settings["new_url"];
        $page["validate_input"]     = dropdown("auth_user_type", array(0 => $page["no_verify"], 1 => $page["email_verify"], 2 => $page["admin_verify"], 3 => $page["both_verify"]), $settings["auth_user_type"]);
        $page["login_input"]        = dualRadio("login_on_reg", $settings["login_on_reg"]);
        $page["min_user_length"]    = $settings["min_user_length"];
        $page["min_pass_length"]    = $settings["min_pass_length"];

        // Get the group list
        $result = db_query($queries["group_list"], $link_id);

        $groups = array();
        while ($query_data = db_fetch_array($result, $link_id))
        {
            $groups[$query_data["gid"]] = $query_data["groupname"];
        }

        $page["group_input"]        = dropdown("default_group", $groups, $settings["default_group"]);

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/registration.tpl", $page);
    break;
    case "save_reg":
        /* ----------------------------
            Save registration settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["reg_caption"];
        $page["caption"]            = $page["input_error"];

        if (strlen(trim($input["security_timeout"])) == 0 || preg_match("/^[0-9]+$/", $input["security_timeout"]) == false || $input["security_timeout"] < 1)
        {
            $page["msg"] = $page["invalid_security_timeout"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["email_timeout"])) == 0 || preg_match("/^[0-9]+$/", $input["email_timeout"]) == false || $input["email_timeout"] < 1)
        {
            $page["msg"] = $page["invalid_email_timeout"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["min_user_length"])) == 0 || preg_match("/^[0-9]+$/", $input["min_user_length"]) == false || $input["min_user_length"] < 1)
        {
            $page["msg"] = $page["invalid_username_length"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["min_pass_length"])) == 0 || preg_match("/^[0-9]+$/", $input["min_pass_length"]) == false || $input["min_user_length"] < 1)
        {
            $page["msg"] = $page["invalid_password_length"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["new_path"])) == 0 && $input["allow_reg"] == 1)
        {
            $page["msg"] = $page["invalid_new_path"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["new_url"])) == 0 && $input["allow_reg"] == 1)
        {
            $page["msg"] = $page["invalid_new_url"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("allow_reg"        => $input["allow_reg"] == 1 ? true : false,
                             "security_check"   => $input["security_check"] == 1 ? true : false,
                             "security_type"    => $input["security_type"] == "gd" ? "gd" : "gif",
                             "gd_version"       => $input["gd_version"] == 1 ? 1 : 2,
                             "security_timeout" => (int) $input["security_timeout"],
                             "email_timeout"    => (int) $input["email_timeout"],
                             "min_user_length"  => $input["min_user_length"],
                             "min_pass_length"  => $input["min_pass_length"],
                             "new_path"         => $input["new_path"],
                             "new_url"          => $input["new_url"],
                             "auth_user_type"   => $input["auth_user_type"] <= 3 ? $input["auth_user_type"] : 1,
                             "default_group"    => (int) $input["default_group"],
                             "login_on_reg"     => $input["login_on_reg"] == 1 ? true : false
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "type":
        /* ----------------------------
              File type assoications
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["types_caption"];
        $page["header_description"] = $page["types_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Build the list of images
        $dp     = opendir($settings["script_dir"] . "images/icons/");
        $system = array("back.gif", "folder.gif", "linkedfolder.gif", "linkedopenfolder.gif", "lockedfolder.gif", "lockedlinkedfolder.gif", "lockedopenfolder.gif", "openfolder.gif", "unknown.gif");
        $images = array();
        while($entry = readdir($dp))
        {
            if (is_dir($entry) == false && $entry != "." && $entry != ".." && in_array($entry, $system) == false)
            {
                // Only display images
                if (preg_match('/\.gif$|\.png$/i', $entry) == true)
                {
                    $images[$entry] = $entry;
                }
            }
        }
        closedir($dp);
        natcasesort($images);

        // Get the blocks from the template file and then parse them
        $blocks = get_blocks($settings["script_dir"] . "skin/admin/settings/association.tpl", array("icons", "new"));

        // Populate the existing field
        $i = 0;
        $blocks["tmp"] = "";
        foreach ($settings["icons"] as $key => $var)
        {
            // Sort the extension list
            $list = explode(" ", $var);
            natcasesort($list);

            $var = "";
            foreach ($list as $exts)
            {
                $var .= $exts . " ";
            }
            $page["extensions"] = trim($var);

            $page["image_list"] = imageDropdown("images[" . $i . "]", $images, $i, $key);
            $page["image"]      = $key;
            $page["id"]         = $i;

            $blocks["tmp"] .= parse_html($page, $blocks["icons"]);

            $i++;
        }

        $blocks["icons"]            = $blocks["tmp"];

        // Populate the new fields
        $blocks["tmp"] = "";
        $images["unknown.gif"] = "unknown.gif";
        for ($x = $i; $x < $i+$newTypes; $x++)
        {
            $page["id"]         = $x;
            $page["image_list"] = imageDropdown("images[" . $x . "]", $images, $x, "unknown.gif");

            $blocks["tmp"] .= parse_html($page, $blocks["new"]);

        }
        $blocks["new"]              = $blocks["tmp"];

        $page["content"]            = parse_html($page, dynamic_template($settings["script_dir"] . "skin/admin/settings/association.tpl", $blocks));

        $page["icondir"]            = $settings["script_dir"] . "images/icons/";

        if (is_win() == true)
        {
            $page["icondir"] = preg_replace("/[\/]/", "\\", $page["icondir"]);
        }
    break;
    case "save_type":
        /* ----------------------------
           Save file type assoications
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["types_caption"];
        $page["caption"]            = $page["input_error"];

        foreach ($input["images"] as $key => $var)
        {
            $exts = trim($input["extensions"][$key]);
            if (empty($exts) == false && empty($input["delete"][$key]) == true)
            {
                // Fix to stop people moaning
                // "I selected a file icon which already existed and it overwrite my current setting"
                if (isset($updated["icons"][$var]) == true)
                {
                    $updated["icons"][$var] = $updated["icons"][$var] . " " . $exts;
                }
                else
                {
                    $updated["icons"][$var] = $exts;
                }
            }
        }

        if (update_settings($updated) == false)
        {
            $page["caption"] = $page["server_error"];
            $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $page["caption"] = $page["done"];
            $page["msg"]     = $page["settings_saved"];
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "edit":
        /* ----------------------------
                  File editing
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["edit_caption"];
        $page["header_description"] = $page["edit_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["locktimeout"]        = $settings["locktimeout"];
        $page["default_type"]       = $settings["default_type"];
        $page["ascii"]              = trim(implode($settings["ascii"], " "));
        $page["user_lock"]          = dualRadio("edit_lock", $settings["edit_lock"]);
        $page["process_lock"]       = dualRadio("flock", $settings["flock"]);

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/editing.tpl", $page);
    break;
    case "save_edit":
        /* ----------------------------
                 Save file editing
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["edit_caption"];
        $page["caption"]            = $page["input_error"];

        if (strlen(trim($input["default_type"])) == 0 || preg_match("/^[a-z0-9]+$/i", $input["default_type"]) == false)
        {
            $page["msg"] = $page["invalid_default_type"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-9]+$/", $input["locktimeout"]) == false || empty($input["locktimeout"]) == true)
        {
            $page["msg"] = $page["invalid_locktimeout"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("default_type" => $input["default_type"],
                             "edit_lock"    => $input["edit_lock"] == 1 ? true : false,
                             "locktimeout"  => (int) $input["locktimeout"],
                             "ascii"        => explode(" ", $input["ascii"]),
                             "flock"        => $input["flock"] == 1 ? true : false
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "filter":
        /* ----------------------------
                   File filters
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["filter_caption"];
        $page["header_description"] = $page["filter_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["img_filter"]         = $settings["filters"]["img"];
        $page["mm_filter"]          = $settings["filters"]["mm"];
        $page["arc_filter"]         = $settings["filters"]["arc"];
        $page["bin_filter"]         = $settings["filters"]["bin"];
        $page["db_filter"]          = $settings["filters"]["db"];
        $page["off_filter"]         = $settings["filters"]["off"];

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/filters.tpl", $page);
    break;
    case "save_filter":
        /* ----------------------------
                 Save file filters
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["filter_caption"];
        $page["caption"]            = $page["input_error"];

        if (strlen(trim($input["filters"]["img"])) == 0 || preg_match("/^[a-z0-9 ]+$/i", $input["filters"]["img"]) == false)
        {
            $page["msg"] = $page["invalid_filter_img"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["filters"]["mm"])) == 0 || preg_match("/^[a-z0-9 ]+$/i", $input["filters"]["mm"]) == false)
        {
            $page["msg"] = $page["invalid_filter_mm"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["filters"]["arc"])) == 0 || preg_match("/^[a-z0-9 ]+$/i", $input["filters"]["arc"]) == false)
        {
            $page["msg"] = $page["invalid_filter_arc"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["filters"]["bin"])) == 0 || preg_match("/^[a-z0-9 ]+$/i", $input["filters"]["bin"]) == false)
        {
            $page["msg"] = $page["invalid_filter_bin"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["filters"]["db"])) == 0 || preg_match("/^[a-z0-9 ]+$/i", $input["filters"]["db"]) == false)
        {
            $page["msg"] = $page["invalid_filter_db"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["filters"]["off"])) == 0 || preg_match("/^[a-z0-9 ]+$/i", $input["filters"]["off"]) == false)
        {
            $page["msg"] = $page["invalid_filter_off"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("filters" => $input["filters"]
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "db":
        /* ----------------------------
                 Database setup
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["db_caption"];
        $page["header_description"] = $page["db_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["dbserver"]           = $settings["dbserver"];
        $page["dbuser"]             = $settings["dbuser"];
        $page["dbpass"]             = $settings["dbpass"];
        $page["dbname"]             = $settings["dbname"];
        $page["dbprefix"]           = $settings["dbprefix"];

        // Build the list of drivers
        $dp     = opendir($settings["script_dir"] . "modules/db/");
        $drivers = array();
        while($entry = readdir($dp))
        {
            if (is_dir($entry) == false && $entry != "." && $entry != "..")
            {
                // Only display images
                if (preg_match("/\.lib.php$/i", $entry) == true && preg_match("/_queries/i", $entry) == false)
                {
                    $entry = explode(".", $entry);
                    $drivers[$entry[0]] = $entry[0];
                }
            }
        }
        closedir($dp);
        natcasesort($drivers);

        $page["drivers"]            = dropdown("dbtype", $drivers, $settings["dbtype"]);
        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/database.tpl", $page);
    break;
    case "save_db":
        /* ----------------------------
               Save database setup
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["db_caption"];
        $page["caption"]            = $page["input_error"];

        // Check that the driver exists
        $dp     = opendir($settings["script_dir"] . "modules/db/");
        $found = false;
        while($entry = readdir($dp))
        {
            if (is_dir($entry) == false && $entry != "." && $entry != "..")
            {
                // Only display drivers
                if (preg_match("/\.lib.php$/i", $entry) == true && preg_match("/_queries/i", $entry) == false)
                {
                    $entry = explode(".", $entry);
                    if ($entry[0] == $input["dbtype"])
                    {
                        $found = true;
                        break;
                    }
                }
            }
        }
        closedir($dp);

        // Driver not found, use mySQL
        if ($found == false)
        {
            $input["dbtype"] = "mySQL";
        }

        if (strlen(trim($input["dbserver"])) == 0)
        {
            $page["msg"] = $page["no_dbserver"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["dbuser"])) == 0)
        {
            $page["msg"] = $page["no_dbuser"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen(trim($input["dbname"])) == 0)
        {
            $page["msg"] = $page["no_dbname"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["dbprefix"]) == false && preg_match("/^[a-z0-9_]+$/i", $input["dbprefix"]) == false)
        {
            $page["msg"] = $page["invalid_dbprefix"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("dbtype"   => $input["dbtype"],
                             "dbserver" => $input["dbserver"] ,
                             "dbuser"   => $input["dbuser"],
                             "dbpass"   => $input["dbpass"],
                             "dbname"   => $input["dbname"],
                             "dbprefix" => empty($input["dbprefix"]) == true ? "ipfm" : $input["dbprefix"]
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "off":
        /* ----------------------------
                 Turn on/off
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["offline_caption"];
        $page["header_description"] = $page["offline_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["offline_reason"]     = htmlspecialchars($settings["offline_reason"]);
        $page["offline"]            = dualRadio("offline", $settings["offline"]);

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/offline.tpl", $page);
    break;
    case "save_off":
        /* ----------------------------
               Save offline status
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["offline_caption"];
        $page["caption"]            = $page["input_error"];

        if ($input["offline"] == 1 && strlen(trim($input["offline_reason"])) == 0)
        {
            $page["msg"] = $page["no_offline_reason"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Strip new lines for safety
            $input["offline_reason"] = str_replace("\n", "", $input["offline_reason"]);
            $input["offline_reason"] = str_replace("\r", "", $input["offline_reason"]);

            // Build the array
            $updated = array("offline"        => $input["offline"] == 0 ? false : true,
                             "offline_reason" => $input["offline_reason"]
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "cpu":
        /* ----------------------------
               Resource management
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["resources_caption"];
        $page["header_description"] = $page["resources_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["redirect_list"]      = dropdown("redirect_header", array(0 => $page["use_location"], 1 => $page["use_refresh"], 2 => $page["use_meta"]), $settings["redirect_header"]);
        $page["cache_headers"]      = dualRadio("no_cache", $settings["no_cache"]);
        $page["use_gzip"]           = dualRadio("use_gzip", $settings["use_gzip"]);
        $page["cpu_load"]           = dualRadio("load", $settings["load"]);
        $page["query_count"]        = dualRadio("query_count", $settings["queries"]);

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/resources.tpl", $page);
    break;
    case "save_cpu":
        /* ----------------------------
             Save resource management
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["resources_caption"];
        $page["caption"]            = $page["input_error"];

        // Build the array
        $updated = array("redirect_header" => $input["redirect_header"] < 3 ? $input["redirect_header"] : 0,
                         "no_cache"        => $input["no_cache"] == 1 ? 1 : 0,
                         "use_gzip"        => $input["use_gzip"] == 1 ? 1 : 0,
                         "load"            => $input["load"] == 1 ? 1 : 0,
                         "queries"         => $input["query_count"] == 1 ? 1 : 0
                   );

        if (update_settings($updated) == false)
        {
            $page["caption"] = $page["server_error"];
            $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $page["caption"] = $page["done"];
            $page["msg"]     = $page["settings_saved"];
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "arc":
        /* ----------------------------
                 File archiving
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["archiving_caption"];
        $page["header_description"] = $page["archiving_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $ratios = array(0 => "1 " . $page["lowest_ratio"], 2, 3, 4 => "4 " . $page["default_ratio"], 5, 6, 7, 8, 9 => "9 " . $page["highest_ratio"]);

        $page["gzip_ratio_list"]    = dropdown("gzip_rate", $ratios, $settings["gzip_rate"]);
        $page["bzip2_ratio_list"]   = dropdown("bzip2_rate", $ratios, $settings["bzip2_rate"]);
        $page["zip_input"]          = dualRadio("types[zip]", in_array("zip", $settings["types"]));
        $page["bzip_input"]         = dualRadio("types[bz2]", in_array("bz2", $settings["types"]));
        $page["gzip_input"]         = dualRadio("types[tgz]", in_array("tgz", $settings["types"]));

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/archives.tpl", $page);
    break;
    case "save_arc":
        /* ----------------------------
               Save file archiving
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["archiving_caption"];
        $page["caption"]            = $page["input_error"];

        if ($input["gzip_rate"] < 1 || $input["gzip_rate"] > 9 || preg_match("/^[1-9]+$/", $input["gzip_rate"]) == false)
        {
            $page["msg"] = $page["invalid_gzip_rate"] . "<br /><br />" . $page["go_back"];
        }
        else if ($input["bzip2_rate"] < 1 || $input["bzip2_rate"] > 9 || preg_match("/^[1-9]+$/", $input["bzip2_rate"]) == false)
        {
            $page["msg"] = $page["invalid_bzip2_rate"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("gzip_rate"    => (int) $input["gzip_rate"],
                             "bzip2_rate"   => (int) $input["bzip2_rate"]
                       );

            foreach ($input["types"] as $key => $var)
            {
                $updated["types"][] = $key;
            }

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "time":
        /* ----------------------------
                 Time settings
           ---------------------------- */
        require $settings["script_dir"] . "settings/sys/zones.inc.php";

        $page["section_title"]      = $page["section_title"] . $page["time_caption"];
        $page["header_description"] = $page["time_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["default_time_dropdown"] = dropdown("default_offset", $zones, $settings["default_offset"]/3600);
        $page["format"]                = preg_replace("/<%\\\A\\\T%>/i", "{AT}", $settings["format"]);

        $page["content"]               = static_template($settings["script_dir"] . "skin/admin/settings/time.tpl", $page);
    break;
    case "save_time":
        /* ----------------------------
                Save time settings
           ---------------------------- */
        require $settings["script_dir"] . "settings/sys/zones.inc.php";

        $page["section_title"]      = $page["section_title"] . $page["time_caption"];
        $page["caption"]            = $page["input_error"];

        $valid_zones                = array_keys($zones);

        // If the user tries to enter an invalid timezone use the default zone
        if (in_array($input["default_offset"], $valid_zones) == false)
        {
            $input["default_offset"] = 0;
        }

        $input["format"] = str_replace("{AT}", "<%\\A\\\T%>", $input["format"]);

        // Build the array
        $updated = array("format"           => $input["format"],
                         "default_offset"   => $input["default_offset"] * 3600
                   );

        if (update_settings($updated) == false)
        {
            $page["caption"] = $page["server_error"];
            $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $page["caption"] = $page["done"];
            $page["msg"]     = $page["settings_saved"];
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "sys":
        /* ----------------------------
                  System files
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["sys_caption"];
        $page["header_description"] = $page["sys_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["msg"]                = $settings["msg"];
        $page["bookmarks"]          = $settings["bookmarks"];
        $page["history"]            = $settings["history"];
        $page["notes"]              = $settings["notes"];
        $page["editing"]            = $settings["editing"];
        $page["htaccess"]           = $settings["htaccess"];
        $page["htpasswd"]           = $settings["htpasswd"];

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/system.tpl", $page);
    break;
    case "save_sys":
        /* ----------------------------
                Save system files
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["sys_caption"];
        $page["caption"]            = $page["input_error"];

        if (preg_match("/^[\.a-z0-9_]+$/i", $input["msg"]) == false)
        {
            $page["msg"] = $page["invalid_msg"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[\.a-z0-9_]+$/i", $input["bookmarks"]) == false)
        {
            $page["msg"] = $page["invalid_bookmarks"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[\.a-z0-9_]+$/i", $input["history"]) == false)
        {
            $page["msg"] = $page["invalid_history"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[\.a-z0-9_]+$/i", $input["notes"]) == false)
        {
            $page["msg"] = $page["invalid_notes"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[\.a-z0-9_]+$/i", $input["editing"]) == false)
        {
            $page["msg"] = $page["invalid_editing"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[\.a-z0-9_]+$/i", $input["htaccess"]) == false)
        {
            $page["msg"] = $page["invalid_htaccess"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[\.a-z0-9_]+$/i", $input["htpasswd"]) == false)
        {
            $page["msg"] = $page["invalid_htpasswd"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("msg"          => $input["msg"],
                             "bookmarks"    => $input["bookmarks"],
                             "history"      => $input["history"],
                             "notes"        => $input["notes"],
                             "editing"      => $input["editing"],
                             "htaccess"     => $input["htaccess"],
                             "htpasswd"     => $input["htpasswd"]
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "misc":
        /* ----------------------------
                  Other settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["misc_caption"];
        $page["header_description"] = $page["misc_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Populate the fields
        $page["filext_input"]       = dualRadio("filext.com", $settings["filext.com"]);
        $page["full_chmod_input"]   = dualRadio("full_chmod", $settings["full_chmod"]);
        $page["hide_dot_input"]     = dualRadio("hide_dot", $settings["hide_dot"]);
        $page["toolbar_input"]      = dualRadio("show_second_toolbar", $settings["show_second_toolbar"]);
        $page["guest_input"]        = dualRadio("require_auth", !$settings["require_auth"]);
        $page["history_input"]      = dualRadio("keep_history", $settings["keep_history"]);
        $page["debug_input"]        = dualRadio("debugging", $settings["debug"]);
        $page["default_userid"]     = $settings["default_userid"];
        $page["default_chmod"]      = $settings["default_chmod"];
        $page["perpage"]            = $settings["perpage"];
        $page["script_dir"]         = $settings["script_dir"];
        $page["script_url"]         = $settings["script_url"];
        $page["msgtimeout"]         = $settings["msgtimeout"];
        $page["max_uploads"]        = $settings["max_uploads"];
        $page["lowspace"]           = $settings["lowspace"];

        $page["language_input"]     = dropdown("default_lang", $lang_name, $settings["default_lang"]);
        $page["folder_icon_input"]  = dropdown("folder_link", array(0 => $page["no_link"], 1 => $page["dir_url"], 2 => $page["browse_dir"]), $settings["folder_link"]);
        $page["file_icon_input"]    = dropdown("file_link", array(0 => $page["no_link"], 1 => $page["file_url"], 2 => $page["download_file"]), $settings["file_link"]);


        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/settings/misc.tpl", $page);
    break;
    case "save_misc":
        /* ----------------------------
               Save other settings
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["misc_caption"];
        $page["caption"]            = $page["input_error"];

        // Check User ID
        build_query($queries["check_id"], array("uid" => $input["default_userid"]));
        $result = db_query($queries["check_id"], $link_id);

        if (isset($lang_name[$input["default_lang"]]) == false)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-7]{3}$/", $input["default_chmod"]) == false)
        {
            $page["msg"] = $page["invalid_chmod"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-9]+$/", $input["default_userid"]) == false || db_num_rows($result, $link_id) == 0)
        {
            $page["msg"] = $page["invalid_uid"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-9]+$/", $input["perpage"]) == false || $input["perpage"] < 1)
        {
            $page["msg"] = $page["invalid_perpage"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-9]+$/", $input["max_uploads"]) == false || $input["max_uploads"] < 1)
        {
            $page["msg"] = $page["invalid_max_uploads"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-9]+$/", $input["msgtimeout"]) == false)
        {
            $page["msg"] = $page["invalid_msgtimeout"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[0-9]+$/", $input["lowspace"]) == false)
        {
            $page["msg"] = $page["invalid_msgtimeout"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Build the array
            $updated = array("default_lang"         => (int) $input["default_lang"],
                             "default_chmod"        => (int) $input["default_chmod"],
                             "default_userid"       => (int) $input["default_userid"],
                             "perpage"              => (int) $input["perpage"],
                             "max_uploads"          => (int) $input["max_uploads"],
                             "msgtimeout"           => (int) $input["msgtimeout"],
                             "lowspace"             => (int) $input["lowspace"],
                             "script_dir"           => $input["script_dir"],
                             "script_url"           => $input["script_url"],
                             "folder_link"          => $input["folder_link"] < 3 ? $input["folder_link"] : 0,
                             "file_link"            => $input["file_link"] < 3 ? $input["file_link"] : 0,
                             "require_auth"         => !$input["require_auth"] == 1 ? true : false,
                             "keep_history"         => $input["keep_history"] == 1 ? true : false,
                             "debug"                => $input["debugging"] == 1 ? true : false,
                             "filext_com"           => $input["filext_com"] == 1 ? true : false,
                             "hide_dot"             => $input["hide_dot"] == 1 ? true : false,
                             "full_chmod"           => $input["full_chmod"] == 1 ? true : false,
                             "show_second_toolbar"  => $input["show_second_toolbar"] == 1 ? true : false
                       );

            if (update_settings($updated) == false)
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];
            }
            else
            {
                $page["caption"] = $page["done"];
                $page["msg"]     = $page["settings_saved"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "notes":
        /* ----------------------------
              Update the admin notes
           ---------------------------- */
        // Couldn't get the regex working so do it the dirty way
        $input["notes"] = trim($input["notes"]);
        $input["notes"] = str_replace("\r", "\\r", $input["notes"]); // Replace carriage return
        $input["notes"] = str_replace("\f", "\\f", $input["notes"]); // Replace form feed
        $input["notes"] = str_replace("\n", "\\n", $input["notes"]); // Replace line feed
        $input["notes"] = str_replace("\t", "\\t", $input["notes"]); // Replace horizontal tab

        if ($input["notes"] == $page["note_default"])
        {
            $input["notes"] = "";
        }

        $updated = array("notepad" => $input["notes"]
                   );

        if (update_settings($updated) == false)
        {
            $page["section_title"] = $page["section_title"] . $page["admin_caption"];
            $page["caption"]       = $page["server_error"];
            $page["msg"]           = str_replace("<%FILE%>", $path, $page["setting_file_error"]) . "<br /><br />" . $page["go_back"];

            $page["content"]       = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {
            // Lets go home
            go_to($settings["script_url"] . "admin.php?act=idx&op=main");
        }
    break;
    default:
        // Lets go home
        go_to($settings["script_url"] . "admin.php?act=idx&op=main");
}

$page["js"] = "adset";

?>