<?php

/*
 * Invision Power File Manager
 * User management
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.2
 * Last Modified: 26th February 2005
 *
 */

require $settings["script_dir"] . "settings/sys/zones.inc.php";

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

/* -----------------------------------------
               Module Functions
   ----------------------------------------- */

// Copies the old root directory to the new
function transfer_dir($source, $destination)
{
    // Create arrays to store files and directories
    $_files      = array();
    $_subdirs    = array();

    if (($dp = opendir($source)) == false)
    {
        // The directory could not be opened
        return false;
    }

    // Loop through all directory entries, and construct two temporary arrays containing files and sub directories
    while($entry = readdir($dp))
    {
        if (is_dir($source . $entry) && $entry !=  ".." && $entry !=  ".")
        {
            $_subdirs[] = $entry;
        }
        else if ($entry !=  ".." && $entry !=  ".")
        {
            $_files[] = $entry;
        }
    }

    // Close directory
    closedir($dp);

    // Next we need to create the destination directory
    if (file_exists($destination) == false)
    {
        if (mkdir($destination, 0777) == false)
        {
            return false;
        }
    }

    // Transfer all files within the current directory
    for($i=0; $i<count($_files); $i++)
    {
        if (copy($source . $_files[$i], $destination . $_files[$i]) == false)
        {
            return false;
        }
    }

    // Empty sub directories and then remove the directory
    for($i=0; $i<count($_subdirs); $i++)
    {
        if (transfer_dir($source . $_subdirs[$i] . "/", $destination . $_subdirs[$i] . "/") == false)
        {
            return false;
        }
    }

    return true;
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */
switch ($input["op"])
{
    case "add":
        /* ----------------------------
               Add a new user form
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["add_caption"];
        $page["header_description"] = $page["add_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Get the group list
        $result = db_query($queries["group_list"], $link_id);

        $groups = array();
        while ($query_data = db_fetch_array($result, $link_id))
        {
            $groups[$query_data["gid"]] = $query_data["groupname"];
        }

        $lang_name[0] = $page["default_lang"] . " - " . $lang_name[$settings["default_lang"]];

        // Populate the fields
        $page["mgroup_dropdown"]    = dropdown("mgroup", $groups, $settings["default_group"]);
        $page["timezone_dropdown"]  = dropdown("timezone", $zones, $settings["default_offset"]/3600);
        $page["language_dropdown"]  = dropdown("lang", $lang_name, 0);

        $page["is_bst_input"]       = dualRadio("bst", 0);
        $page["dbman_access"]       = dualRadio("dbman", 0);

        // Sort out the root path
        if (preg_match("/\/$/", $_SERVER["DOCUMENT_ROOT"]) == false && preg_match("/\\$/", $_SERVER["DOCUMENT_ROOT"]) == false)
        {
            $_SERVER["DOCUMENT_ROOT"] = $_SERVER["DOCUMENT_ROOT"] . "/";
        }
        $page["root_dir_value"]     = is_win() == true ? str_replace("/", "\\", $_SERVER["DOCUMENT_ROOT"]) : $_SERVER["DOCUMENT_ROOT"];
        $page["root_url_value"]     = "http://" . $_SERVER["HTTP_HOST"] . "/";

        $page["password_info"] = str_replace("<%MIN_PASS_LEN%>", $settings["min_pass_length"], $page["password_info"]);

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/users/add.tpl", $page);
    break;
    case "save_add":
        /* ----------------------------
                 Adds the new user
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["add_caption"];
        $page["caption"]            = $page["input_error"];

        if (strlen($input["name"]) > 32)
        {
            $page["msg"] = $page["username_length_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen($input["password"]) < $settings["min_pass_length"] || strlen($input["password"]) > 32)
        {
            $page["msg"] = $page["password_length_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen($input["email"]) > 75)
        {
            $page["msg"] = $page["email_length_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[^@ ]+@[^@ ]+[\.][^@ \.]+$/", $input["email"]) == false || strlen($input["email"]) < 6)
        {
            $page["msg"] = $page["invalid_email"] . "<br /><br />" . $page["go_back"];
        }
        else if (isset($lang_name[$input["lang"]]) == false && $input["lang"] > 0)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["space"]) == false && preg_match("/^[0-9]+$/", $input["space"]) == false)
        {
            $page["msg"] = $page["invalid_space"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $input["dbs"]  = preg_replace("/\s/", "", $input["dbs"]);
            $input["name"] = preg_replace("/\s{2,}/", " ", $input["name"]);

            // Now to check if the username is unique
            $email    = strtolower($input["email"]);
            $username = strtolower($input["name"]);

            build_query($queries["check_unique"], array("EMAIL" => $email, "USERNAME" => $username));
            $result = db_query($queries["check_unique"], $link_id);

            // Check for unique username and email address
            $error = "";
            if (db_num_rows($result, $link_id) > 0)
            {
                while ($query_data = db_fetch_array($result, $link_id))
                {
                    if ($query_data["email"] == $email)
                    {
                        $error = $page["email_used"];
                        break;
                    }
                    else if ($query_data["username"] == $username)
                    {
                        $error = $page["username_used"];
                        break;
                    }
                }
            }

            if (empty($error) == false)
            {
                $page["msg"] = $error . "<br /><br />" . $page["go_back"];
            }
            else if ($input["dbman"] == 1 && strlen(trim($input["dbserver"])) == 0)
            {
                $page["msg"] = $page["no_dbserver"] . "<br /><br />" . $page["go_back"];
            }
            else if ($input["dbman"] == 1 && strlen(trim($input["dbuser"])) == 0)
            {
                $page["msg"] = $page["no_dbuser"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                // Sort out the root path
                $input["dir"] = str_replace("\\", "/", $input["dir"]);
                if (preg_match("/\/$/", $input["dir"]) == false)
                {
                    $input["dir"] = $input["dir"] . "/";
                }

                // Sort out the root URL
                if (preg_match("/\/$/", $input["url"]) == false && empty($input["url"]) == false)
                {
                    $input["url"] = $input["url"] . "/";
                }

                $valid_zones = array_keys($zones);

                // If the user tries to enter an invalid timezone use the default zone
                if (in_array($input["timezone"], $valid_zones) == false)
                {
                    $input["timezone"] = $settings["default_offset"];
                }

                // Sort out query values
                if (empty($input["space"]) == true)
                {
                    $input["space"] = 'NULL';
                }

                $input["dbman"]    = $input["dbman"] == 0 ? 0 : 1;
                $input["dbserver"] = empty($input["dbserver"]) == true ? 'NULL' : "'" . $input["dbserver"] . "'";
                $input["dbuser"]   = empty($input["dbuser"]) == true ? 'NULL' : "'" . $input["dbuser"] . "'";
                $input["dbpass"]   = empty($input["dbpass"]) == true ? 'NULL' : "'" . $input["dbpass"] . "'";
                $input["dbs"]      = empty($input["dbs"]) == true ? 'NULL' : "'" . $input["dbs"] . "'";

                build_query($queries["add_user"], array("USERNAME" => $input["name"], "PASSWORD" => md5($input["password"]), "EMAIL" => $input["email"], "LANG" => $input["lang"], "TIME" => $input["timezone"] * 3600, "BST" => $input["bst"], "GID" => $input["mgroup"], "SQL" => $input["dbman"], "DBHOST" => $input["dbserver"], "DBUSER" => $input["dbuser"], "DBPASS" => $input["dbpass"], "DBNAME" => $input["dbs"], "URL" => $input["url"], "ROOT" => $input["dir"], "SPACE" => $input["space"], "LANG" => $input["lang"]));
                $result = db_query($queries["add_user"], $link_id);

                if (empty($result) == true)
                {
                    $page["caption"] = $page["server_error"];
                    $page["msg"]     = $page["add_error"] . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    $page["caption"]           = $page["account_created"];

                    $settings["old_chmod"]     = $settings["default_chmod"];
                    $settings["default_chmod"] = "777";
                    if (create_directory_structure($input["dir"]) == false)
                    {
                        $input["msg"]          = $page["error_creating_root"];
                    }
                    else
                    {
                        chmod($input["dir"], 0777);

                        $page["msg"]           = $page["account_added"];
                    }
                    $settings["default_chmod"] = $settings["old_chmod"];
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "find":
        /* ----------------------------
                  Find a member
           ---------------------------- */
        if ($input["next"] == "del")
        {
            $page["section_title"] = $page["section_title"] . $page["delete_caption"];
            $page["next"]          = "delete";
        }
        else
        {
            $page["section_title"] = $page["section_title"] . $page["find_edit_caption"];
            $page["next"]          = "search";
        }

        $page["header_description"] = $page["find_description"];

        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        $page["content"]            = static_template($settings["script_dir"] . "skin/admin/users/find.tpl", $page);
    break;
    case "search":
        /* ----------------------------
              Results of edit search
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["edit_caption"];
        if (empty($input["username"]) == true && empty($input["gid"]) == true)
        {
            $page["caption"] = $page["input_error"];
            $page["msg"]     = $page["no_query"] . "<br /><br />" . $page["go_back"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {

            if (empty($input["gid"]) == true)
            {
                $sql = $queries["find_user"];
            }
            else
            {
                $sql = $queries["find_user_by_gid"];
            }

            build_query($sql, array("USERNAME" => $input["username"], "GID" => (int) $input["gid"]));
            $result = db_query($sql, $link_id);

            if (db_num_rows($result, $link_id) == 0)
            {
                $page["caption"] = $page["results"];
                $page["msg"]     = $page["no_matches"] . "<br /><br />" . $page["go_back"];
                $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
            }
            else
            {
                $page["header_description"] = $page["edit_results_description"];

                // Get the blocks from the template file and then parse them
                $blocks = get_blocks($settings["script_dir"] . "skin/admin/users/results.tpl", array("users"));

                $page["header"]  = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

                $blocks["tmp"]   = "";
                while ($query_data = db_fetch_array($result, $link_id))
                {
                    // Highlight the match
                    $query_data["username"] = preg_replace("/(" . str_replace("/", "\\/", $input["username"]) . ")/i", "<span class=\"nameMatch\">\\1</span>", $query_data["username"]);

                    $query_data["del"] = "<a href=\"admin.php?act=mem&amp;op=delete&amp;id=" . $query_data["uid"] . "\">" . $page["delete_user"] . "</a>";
                    if ($query_data["uid"] < 3)
                    {
                        $query_data["del"] = $page["na"];
                    }

                    $blocks["tmp"] .= parse_html($query_data, $blocks["users"]);
                }
                $blocks["users"] = $blocks["tmp"];

                $page["content"] = parse_html($page, dynamic_template($settings["script_dir"] . "skin/admin/users/results.tpl", $blocks));
            }
        }
    break;
    case "delete":
        /* ----------------------------
             Results of delete search
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["delete_caption"];
        if (empty($input["username"]) == true && empty($input["id"]) == true)
        {
            $page["caption"] = $page["input_error"];
            $page["msg"]     = $page["no_query"] . "<br /><br />" . $page["go_back"];;
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {
            $sql = $queries["get_guest_info"];
            if (empty($input["id"]) == true)
            {
                $sql = $queries["find_user"];
            }

            build_query($sql, array("USERNAME" => $input["username"], "UID" => (int) $input["id"]));
            $result = db_query($sql, $link_id);

            if (db_num_rows($result, $link_id) == 0)
            {
                $page["caption"] = $page["results"];
                $page["msg"]     = $page["no_matches"] . "<br /><br />" . $page["go_back"];
                $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
            }
            else
            {
                $page["header_description"] = $page["delete_results_description"];

                while ($query_data = db_fetch_array($result, $link_id))
                {
                    if ($query_data["uid"] < 3)
                    {
                        $query_data["username"] = $query_data["username"] . " " . $page["user_no_delete"];
                    }

                    $users[$query_data["uid"]] = $query_data["username"];
                }

                $page["header"]          = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

                // Populate the fields
                $page["members"]         = dropdown("uid", $users);
                $page["delete_root_dir"] = dualRadio("delroot", 0);

                $page["content"]         = static_template($settings["script_dir"] . "skin/admin/users/delete.tpl", $page);
            }
        }
    break;
    case "do_delete":
        /* ----------------------------
                 Delete the user
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["delete_caption"];

        if ($input["uid"] < 3)
        {
            $page["caption"] = $page["input_error"];
            $page["content"] = $page["cancel_delete"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            build_query($queries["delete_user"], array("UID" => (int) $input["uid"]));
            $result = db_query($queries["delete_user"], $link_id);

            if (empty($result) == true)
            {
                $page["caption"] = $page["server_error"];
                $page["content"] = $page["delete_error"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                if ($input["delroot"] == 1)
                {
                    // Delete root is set, so we need to get the users root directory path
                    build_query($queries["get_guest_info"], array("UID" => (int) $input["uid"]));
                    $result = db_query($queries["get_guest_info"], $link_id);

                    while ($query_data = db_fetch_array($result, $link_id))
                    {
                        $root = $query_data["root_path"];
                        break;
                    }

                    if (empty($root) == true || remove_root($root) == false)
                    {
                        $page["caption"] = $page["account_deleted"];
                        $page["msg"]     = $page["delete_no_root"];
                    }
                    else
                    {
                        $page["caption"] = $page["account_deleted"];
                        $page["msg"]     = $page["deleted_root"];
                    }
                }
                else
                {
                    $page["caption"] = $page["account_deleted"];
                    $page["msg"]     = $page["deleted_root_left"];
                }

                // Delete any validation requests for the user
                build_query($queries["delete_uid_validate"], array("UID" => (int) $input["uid"]));
                db_query($queries["delete_uid_validate"], $link_id);
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "do_edit";
        /* ----------------------------
            Display the edit user form
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["edit_caption"];
        $page["header_description"] = $page["edit_description"];
        $page["header"]             = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Get the group list
        $result = db_query($queries["group_list"], $link_id);

        $groups = array();
        while ($query_data = db_fetch_array($result, $link_id))
        {
            $groups[$query_data["gid"]] = $query_data["groupname"];
        }

        $lang_name[0] = $page["default_lang"] . " - " . $lang_name[$settings["default_lang"]];

        build_query($queries["get_guest_info"], array("UID" => $input["id"]));
        $result = db_query($queries["get_guest_info"], $link_id);

        while ($query_data = db_fetch_array($result, $link_id))
        {
            // Populate the fields
            $page["mgroup_dropdown"]    = dropdown("mgroup", $groups, $query_data["gid"]);
            $page["timezone_dropdown"]  = dropdown("timezone", $zones, $query_data["offset"]/3600);
            $page["language_dropdown"]  = dropdown("lang", $lang_name, $query_data["language"]);

            $page["is_bst_input"]       = dualRadio("bst", $input["dst"]);
            $page["dbman_access"]       = dualRadio("dbman", $query_data["sql"]);
            $page["copy_dir"]           = dualRadio("copy", 0);
            $page["remove_dir"]         = dualRadio("rmdir", 0);

            $page["uid"]                = $query_data["uid"];
            $page["username_value"]     = $query_data["username"];
            $page["email_value"]        = $query_data["email"];
            $page["dbhost"]             = $query_data["dbhost"];
            $page["dbuser"]             = $query_data["dbuser"];
            $page["dbpass"]             = $query_data["dbpass"];
            $page["dbs"]                = $query_data["dbname"];
            $page["root_url_value"]     = $query_data["root_url"];
            $page["root_dir_value"]     = is_win() == true ? str_replace("/", "\\", $query_data["root_path"]) : $query_data["root_path"];
            $page["user_space"]         = $query_data["user_space"] == 0 ? "" : $query_data["user_space"];
            break;
        }

        $page["password_info"] = str_replace("<%MIN_PASS_LEN%>", $settings["min_pass_length"], $page["password_info"]);

        $page["content"]         = static_template($settings["script_dir"] . "skin/admin/users/edit.tpl", $page);
    break;
    case "update":
        /* ----------------------------
                Update the account
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["edit_caption"];
        $page["caption"]       = $page["input_error"];

        if (strlen($input["name"]) > 32)
        {
            $page["msg"] = $page["username_length_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["password"]) == false && (strlen($input["password"]) < $settings["min_pass_length"] || strlen($input["password"]) > 32))
        {
            $page["msg"] = $page["password_length_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (strlen($input["email"]) > 75)
        {
            $page["msg"] = $page["email_length_invalid"] . "<br /><br />" . $page["go_back"];
        }
        else if (preg_match("/^[^@ ]+@[^@ ]+[\.][^@ \.]+$/", $input["email"]) == false || strlen($input["email"]) < 6)
        {
            $page["msg"] = $page["invalid_email"] . "<br /><br />" . $page["go_back"];
        }
        else if (isset($lang_name[$input["lang"]]) == false && $input["lang"] > 0)
        {
            $page["msg"] = $page["invalid_lang"] . "<br /><br />" . $page["go_back"];
        }
        else if (empty($input["space"]) == false && preg_match("/^[0-9]+$/", $input["space"]) == false)
        {
            $page["msg"] = $page["invalid_space"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            $input["dbs"]  = preg_replace("/\s/", "", $input["dbs"]);
            $input["name"] = preg_replace("/\s{2,}/", " ", $input["name"]);

            // Now to check if the username is unique
            $email    = strtolower($input["email"]);
            $username = strtolower($input["name"]);

            build_query($queries["check_unique"], array("EMAIL" => $email, "USERNAME" => $username));
            $result = db_query($queries["check_unique"], $link_id);

            // Check for unique username and email address
            $error = "";
            if (db_num_rows($result, $link_id) > 0)
            {
                while ($query_data = db_fetch_array($result, $link_id))
                {
                    if (strtolower($query_data["email"]) == $email && $query_data["uid"] != (int) $input["uid"])
                    {
                        $error = $page["email_used"];
                        break;
                    }
                    else if (strtolower($query_data["username"]) == $username && $query_data["uid"] != (int) $input["uid"])
                    {
                        $error = $page["username_used"];
                        break;
                    }
                }
            }

            if (empty($error) == false)
            {
                $page["msg"] = $error . "<br /><br />" . $page["go_back"];
            }
            else if ($input["dbman"] == 1 && strlen(trim($input["dbserver"])) == 0)
            {
                $page["msg"] = $page["no_dbserver"] . "<br /><br />" . $page["go_back"];
            }
            else if ($input["dbman"] == 1 && strlen(trim($input["dbuser"])) == 0)
            {
                $page["msg"] = $page["no_dbuser"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                // Sort out the root path
                $input["dir"] = str_replace("\\", "/", $input["dir"]);
                if (preg_match("/\/$/", $input["dir"]) == false)
                {
                    $input["dir"] = $input["dir"] . "/";
                }

                // Sort out the root URL
                if (preg_match("/\/$/", $input["url"]) == false && empty($input["url"]) == false)
                {
                    $input["url"] = $input["url"] . "/";
                }

                $valid_zones = array_keys($zones);

                // If the user tries to enter an invalid timezone use the default zone
                if (in_array($input["timezone"], $valid_zones) == false)
                {
                    $input["timezone"] = $settings["default_offset"];
                }

                build_query($queries["get_guest_info"], array("UID" => (int) $input["uid"]));
                $result = db_query($queries["get_guest_info"], $link_id);

                while ($query_data = db_fetch_array($result, $link_id))
                {
                    $oldRoot = $query_data["root_path"];
                    break;
                }


                // Sort out query values
                if (empty($input["space"]) == true)
                {
                    $input["space"] = 'NULL';
                }

                $input["dbman"]    = $input["dbman"] == 0 ? 0 : 1;
                $input["dbserver"] = empty($input["dbserver"]) == true ? 'NULL' : "'" . $input["dbserver"] . "'";
                $input["dbuser"]   = empty($input["dbuser"]) == true ? 'NULL' : "'" . $input["dbuser"] . "'";
                $input["dbpass"]   = empty($input["dbpass"]) == true ? 'NULL' : "'" . $input["dbpass"] . "'";
                $input["dbs"]      = empty($input["dbs"]) == true ? 'NULL' : "'" .  $input["dbs"] . "'";
                $input["password"] = empty($input["password"]) == true ? 'password' : "'" . md5($input["password"]) . "'";

                build_query($queries["update_user"], array("UID" => (int) $input["uid"], "USERNAME" => $input["name"], "PASSWORD" => $input["password"], "EMAIL" => $input["email"], "LANG" => $input["lang"], "TIME" => $input["timezone"] * 3600, "BST" => $input["bst"], "GID" => $input["mgroup"], "SQL" => $input["dbman"], "DBHOST" => $input["dbserver"], "DBUSER" => $input["dbuser"], "DBPASS" => $input["dbpass"], "DBNAME" => $input["dbs"], "URL" => $input["url"], "ROOT" => $input["dir"], "SPACE" => $input["space"], "LANG" => $input["lang"]));
                $result = db_query($queries["update_user"], $link_id);

                if (empty($result) == true)
                {
                    $page["caption"] = $page["server_error"];
                    $page["msg"]     = $page["update_error"] . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    $page["caption"]           = $page["account_updated"];

                    $settings["old_chmod"]     = $settings["default_chmod"];
                    $settings["default_chmod"] = "777";
                    if (create_directory_structure($input["dir"]) == false)
                    {
                        $input["msg"]          = $page["error_creating_root"];
                    }
                    else
                    {
                        chmod($input["dir"], 0777);

                        // Finally transfer the files
                        if ($oldRoot != $input["dir"] && $input["copy"] == 1 && transfer_dir($oldRoot, $input["dir"]) == false)
                        {
                            $page["msg"]       = $page["updated_no_root"];
                        }
                        else if ($oldRoot != $input["dir"] && $input["rmdir"] == 1 && remove_root($oldRoot) == false)
                        {
                            $page["msg"]       = $page["updated_no_delete"];
                        }
                        else
                        {
                            $page["msg"]       = $page["account_update_complete"];
                        }
                    }
                    $settings["default_chmod"] = $settings["old_chmod"];
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "msg":
        /* ----------------------------
               Send message form
           ---------------------------- */
        $page["section_title"]      = $page["section_title"] . $page["msg_caption"];
        $page["header_description"] = $page["msg_description"];

        $page["header"]  = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

        // Get the group list
        $result = db_query($queries["group_list"], $link_id);

        $groups = array();
        while ($query_data = db_fetch_array($result, $link_id))
        {
            $groups[$query_data["gid"]] = $query_data["groupname"];
        }

        // Populate the fields
        $page["mgroup_dropdown"]    = dropdown("mgroup", $groups, $settings["default_group"]);

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/users/send.tpl", $page);
    break;
    case "send":
        /* ----------------------------
              Send the actual message
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["msg_caption"];

        if (empty($input["msg"]) == true)
        {
            $page["caption"] = $page["input_error"];
            $page["msg"]     = $page["no_message"] . "<br /><br />" . $page["go_back"];
        }
        else
        {
            // Sort out the users
            if (empty($input["users"]) == false)
            {
                $list = explode(",", $input["users"]);

                $in = "";
                foreach ($list as $var)
                {
                    $var = trim($var);
                    if (empty($var) == false)
                    {
                        $in .= "'" . $var . "', ";
                    }
                }

                $in = preg_replace("/, $/", "", $in);

                $sql = $queries["users_by_name"];
            }
            else
            {
                $sql = $queries["users_by_group"];
            }

            build_query($sql, array("IN" => $in, "GID" => (int) $input["mgroup"]));
            $result = db_query($sql, $link_id);

            if (empty($result) == true || db_num_rows($result, $link_id) == 0)
            {
                $page["caption"] = $page["msg_caption"];
                $page["msg"]     = $page["no_msg_matches"] . "<br /><br />" . $page["go_back"];
            }
            else
            {
                // Build a list of messages to write
                $toDo = array();
                while ($query_data = db_fetch_array($result, $link_id))
                {
                    $path = $query_data["root_path"] . $settings["msg"];
                    if (is_win() == true)
                    {
                        $path = strtolower($path);
                    }

                    if (in_array($path, $toDo) == false)
                    {
                        $toDo[$query_data["username"]] = $path;
                    }
                }

                // Write the messages
                $errors = array();
                foreach ($toDo as $key => $var)
                {
                    $write = $input["msg"];
                    if (file_exists($var) == true && filesize($var) > 0)
                    {
                        $write = "\r\n\r\n" . $write;
                    }

                    // Write the message
                    if (($fp = fopen($var, "a")) == true)
                    {
                        file_lock($fp);
                        fwrite($fp, $write);
                        file_lock($fp, LOCK_UN);
                        fclose($fp);
                    }
                    else
                    {
                        $errors[] = $key;
                    }
                }

                if (count($errors) == 0)
                {
                    $page["caption"] = $page["msg_done"];
                    $page["msg"]     = $page["msg_sent"];
                }
                else if (count($errors) == count($toDo))
                {
                    $page["caption"] = $page["server_error"];
                    $page["msg"]     = $page["none_sent"] . "<br /><br />" . $page["go_back"];
                }
                else
                {
                    $users = count($errors) == 1 ? $errors[0] : build_list($errors, true, $page["or"]);

                    $page["caption"] = $page["msg_done"];
                    $page["msg"]     = str_replace("<%USERS%>", $users, $page["msg_errors"]);
                }
            }
        }
        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    case "val":
        /* ----------------------------
                Account validation
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["validate_caption"];

        // Get the list of members waiting for validation
        $result = db_query($queries["validate_list"], $link_id);

        if (db_num_rows($result, $link_id) == 0)
        {
            // None waiting for validation
            $page["caption"] = $page["account_validation"];
            $page["msg"]     = $page["no_validate"];
            $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
        }
        else
        {
            $page["header_description"] = $page["validate_description"];

            $page["header"]  = static_template($settings["script_dir"] . "skin/admin/header.tpl", $page);

            // Get the blocks from the template file and then parse them
            $blocks = get_blocks($settings["script_dir"] . "skin/admin/users/validation.tpl", array("users"));

            $blocks["tmp"]   = "";
            while ($query_data = db_fetch_array($result, $link_id))
            {
                if ($query_data["type"] == AUTH_BOTH)
                {
                    $query_data["admin_validate"] = checkBox("admin[" . $query_data["uid"] . "]");
                    $query_data["email_validate"] = checkBox("email[" . $query_data["uid"] . "]");
                }
                else if ($query_data["type"] == AUTH_ADMIN)
                {
                    $query_data["admin_validate"] = checkBox("admin[" . $query_data["uid"] . "]");
                    $query_data["email_validate"] = checkBox("email[" . $query_data["uid"] . "]", true);
                }
                else if ($query_data["type"] == AUTH_EMAIL)
                {
                    $query_data["admin_validate"] = checkBox("admin[" . $query_data["uid"] . "]", true);
                    $query_data["email_validate"] = checkBox("email[" . $query_data["uid"] . "]");
                }

                $query_data["vtime"] = date("j F Y", $query_data["vtime"]);

                $blocks["tmp"] .= parse_html($query_data, $blocks["users"]);
            }
            $blocks["users"] = $blocks["tmp"];

            $page["content"] = parse_html($page, dynamic_template($settings["script_dir"] . "skin/admin/users/validation.tpl", $blocks));
        }
    break;
    case "validate":
        /* ----------------------------
              Validate the account
           ---------------------------- */
        $page["section_title"] = $page["section_title"] . $page["validate_caption"];
        $page["caption"]       = $page["account_validation"];

        // Get current validation status
        $result = db_query($queries["validate_list"], $link_id);

        if (db_num_rows($result, $link_id) == 0)
        {
            // None waiting for validation
            $page["msg"] = $page["no_validate"];
        }
        else
        {
            $current = array();
            while ($query_data = db_fetch_array($result, $link_id))
            {
                $current[$query_data["uid"]] = $query_data["type"];
            }

            // Now to loop round and set all the new values
            // Very messy way to do it, but my brain has gone to mush d'oh!
            $new = array();
            foreach ($input["users"] as $var)
            {
                if ($input["admin"][$var] == 1)
                {
                    if ($input["email"][$var] == 1)
                    {
                        // Approved completely
                        $new[$var] = AUTH_NONE;
                    }
                    else
                    {
                        if ($current[$var] == AUTH_EMAIL || $current[$var] == AUTH_BOTH)
                        {
                            // Admin approval
                            $new[$var] = AUTH_EMAIL;
                        }
                        else
                        {
                            // Even though the user said to not approve email, it has already been done so we're completely approved
                            $new[$var] = AUTH_NONE;
                        }
                    }
                }
                else if ($input["email"][$var] == 1)
                {
                    if ($current[$var] == AUTH_ADMIN || $current[$var] == AUTH_BOTH)
                    {
                        // Just need admin approval now
                        $new[$var] = AUTH_ADMIN;
                    }
                    else
                    {
                        // Approved completely
                        $new[$var] = AUTH_NONE;
                    }
                }
                else
                {
                    // Nothings changed
                    $new[$var] = $current[$var];
                }
            }

            // Now we run over the queries!
            $errors = false;
            foreach ($new as $key => $var)
            {
                $sql = $queries["update_validate"];
                if ($var == AUTH_NONE)
                {
                    $sql = $queries["delete_uid_validate"];
                }

                build_query($sql, array("TYPE" => $var, "UID" => $key));
                $result = db_query($sql, $link_id);

                if (empty($result) == true)
                {
                    $errors = true;
                }
            }

            if ($errors == false)
            {
                $page["msg"]     = $page["validation_done"];
            }
            else
            {
                $page["caption"] = $page["server_error"];
                $page["msg"]     = $page["validation_error"];
            }
        }

        $page["content"] = static_template($settings["script_dir"] . "skin/admin/msg.tpl", $page);
    break;
    default:
        // Lets go home
        go_to($settings["script_url"] . "admin.php?act=idx&op=main");
}

$page["js"] = "adusers";

?>