<?php

/*
 * Invision Power File Manager
 * Tar Archiving
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Matthew Mecham, Stephen Ball
 * Version: 1.0.0
 * Last Modified: 9th August 2004
 * -------------------------------------
 *
 * Modification of the TAR module from Invision Power Board
 *
 * Official TAR file format: http://www.dwam.net/docs/rktools/tar/tar_11.html#SEC120
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

class archive
{
    var $tar_header_length = "512";
    var $tar_unpack_header = "a100filename/a8mode/a8uid/a8gid/a12size/a12mtime/a8chksum/a1typeflag/a100linkname/a6magic/a2version/a32uname/a32gname/a8devmajor/a8devminor/a155/prefix";
    var $tar_pack_header   = "A100 A8 A8 A8 A12 A12 A8 A1 A100 A6 A2 A32 A32 A8 A8 A155";
    var $work_dir          = array();
    var $data_in_mem       = array();
    var $file_name         = "";
    var $file_path         = "";
    var $file_path_name    = "";
    var $old_file_name     = "";
    var $data              = "";
    var $type              = "tar";

    // Error data
    var $errors            = 0;
    var $nodir             = array();
    var $no_dir_open       = array();
    var $sysdir            = array();
    var $openfile          = array();
    var $name_length       = array();
    var $path_length       = array();
    var $nospace           = array();
    var $file_exists       = array();
    var $no_overwrite_file = array();
    var $no_save           = false;
    var $noext             = false;
    var $open_error        = false;
    var $total_files       = 0;

    // Constructor
    function archive($tarpath, $tarname)
    {
        $this->file_name = $tarname;
        $this->file_path = $tarpath;

        $this->file_path = preg_replace("#[/\\\]$#", "", $this->file_path);
        $this->file_path_name = $this->file_path . "/" . $this->file_name;
    }

    // Adds a directory to an archive
    function add_directory($dir)
    {
        $this->workfiles = array();

        if (is_dir($dir) == false)
        {
            $this->nodir[] = basename($dir);
            $this->errors++;
            return false;
        }

        // Reset the file list to prevent files from being added many times
        $this->get_dir_contents($dir);
        $this->add_files($this->workfiles);

        return true;
    }

    // Get the directories content
    function get_dir_contents($dir)
    {
        $dir = preg_replace("#/$#", "", $dir);

        if (file_exists($dir) == true)
        {
            if (is_dir($dir) == true)
            {
                $handle = opendir($dir);

                while (($filename = readdir($handle)) !== false)
                {
                    if ($filename != "." && $filename != "..")
                    {
                        if (is_dir($dir . "/" . $filename . "/") == true && check_link($dir . "/" . $filename . "/") == false)
                        {
                            if (is_sys_file($filename, true) == true)
                            {
                                $this->sysdir[] = $filename;
                                $this->errors++;
                            }
                            else
                            {
                                $this->get_dir_contents($dir . "/" . $filename . "/");
                            }
                        }
                        else
                        {
                            $this->workfiles[] = $dir . "/" . $filename;
                        }
                    }
                }

                closedir($handle);
            }
            else
            {
                $this->error = "$dir is not a directory";
                return FALSE;
            }
        }
        else
        {
            $this->no_dir_open[] = $dir;
            return;
        }
    }

    // Adds an array of files to the archive
    function add_files($files)
    {
        global $settings, $permissions;

        if (count($files) < 1)
        {
            return;
        }

        foreach ($files as $file)
        {
            // Skip any system files
            if (preg_match("/\.ds_store/i", $file) == false && is_sys_file($file) == false && file_permission($file) == true)
            {
                $typeflag = 0;
                $data     = "";
                $linkname = "";

                $stat = stat($file);

                $mode  = fileperms($file);
                $uid   = $stat[4];
                $gid   = $stat[5];
                $rdev  = $stat[6];
                $size  = filesize($file);
                $mtime = filemtime($file);

                if (is_file($file) == true)
                {
                    $typeflag = 0;

                    if ($fp = fopen($file, "rb"))
                    {
                        $data = fread($fp, filesize($file));
                        fclose($fp);
                    }
                    else
                    {
                        $this->openfile[] = $file;
                        $this->errors++;
                        continue;
                    }
                }
                else if (is_link($file) == true)
                {
                    $typeflag = 1;
                    $linkname = @readlink($file);
                }
                else if (is_dir($file) == true)
                {
                    $typeflag = 5;
                }
                else
                {
                    // Sockets, Pipes and char/block specials are not
                    // supported, so - lets use a silly value to keep the
                    // tar ball legitimate.
                    $typeflag = 9;
                }

                // Add this data to our in memory tar file
                $this->data_in_mem[] = array ("name"    => str_replace($permissions["root"], "", $file),
                                            "mode"     => $mode,
                                            "uid"      => $uid,
                                            "gid"      => $gid,
                                            "size"     => strlen($data),
                                            "mtime"    => $mtime,
                                            "chksum"   => "      ",
                                            "typeflag" => $typeflag,
                                            "linkname" => $linkname,
                                            "magic"    => "ustar\0",
                                            "version"  => "00",
                                            "uname"    => "unknown",
                                            "gname"    => "unknown",
                                            "devmajor" => "",
                                            "devminor" => "",
                                            "prefix"   => "",
                                            "data"     => $data
                                            );
                // Clear the stat cache
                clearstatcache();
            }
            else
            {
                $this->errors++;
                $this->noext = true;
            }
        }
        return;
    }

    // Creates and stores the archive in memory
    function create_archive()
    {
        $tardata = "";

        foreach ($this->data_in_mem as $file)
        {
            $prefix = "";
            $tmp    = "";
            $last   = "";

            // make sure the filename isn't longer than 99 characters.
            if (strlen($file["name"]) > 99)
            {
                $pos = strrpos($file["name"], "/");

                if (is_string($pos) && empty($pos) == true)
                {
                    // Filename alone is longer than 99 characters!
                    $this->name_length[] = basename($file["name"]);
                    $this->errors++;
                    continue;
                }

                $prefix = substr($file["name"], 0, $pos);  // Move the path to the prefix
                $file["name"] = substr($file["name"], $pos + 1);
                if (strlen($prefix) > 154)
                {
                    $this->path_length[] = basename($file["name"]);
                    $this->errors++;
                    continue;
                }
            }

            // BEGIN FORMATTING (a8a1a100)
            $mode  = sprintf("%6s ", decoct($file["mode"]));
            $uid   = sprintf("%6s ", decoct($file["uid"]));
            $gid   = sprintf("%6s ", decoct($file["gid"]));
            $size  = sprintf("%11s ", decoct($file["size"]));
            $mtime = sprintf("%11s ", decoct($file["mtime"]));

            $tmp   = pack("a100a8a8a8a12a12", $file["name"], $mode, $uid, $gid, $size, $mtime);

            $last  = pack("a1"   , $file["typeflag"]);
            $last .= pack("a100" , $file["linkname"]);

            $last .= pack("a6", "ustar");           // Magic
            $last .= pack("a2", "" );               // Version
            $last .= pack("a32", $file["uname"]);
            $last .= pack("a32", $file["gname"]);
            $last .= pack("a8", "");                // Devmajor
            $last .= pack("a8", "");                // Devminor
            $last .= pack("a155", $prefix);
            // $last .= pack("a12", "");

            $test_len = $tmp . $last . "12345678";

            $last .= $this->internal_build_string("\0", ($this->tar_header_length - strlen($test_len)));

            // Here comes the science bit, handling the checksum.
            $checksum = 0;

            for ($i = 0; $i < 148; $i++)
            {
                $checksum += ord(substr($tmp, $i, 1));
            }

            for ($i = 148; $i < 156; $i++)
            {
                $checksum += ord(" ");
            }

            for ($i = 156, $j = 0; $i < 512; $i++, $j++)
            {
                $checksum += ord(substr($last, $j, 1));
            }

            $checksum = sprintf("%6s ", decoct($checksum));

            $tmp .= pack("a8", $checksum);
            $tmp .= $last;
            $tmp .= $file["data"];

            // Tidy up this chunk to the power of 512
            if ($file["size"] > 0)
            {
                if ($file["size"] % 512 != 0)
                {
                    $homer = $this->internal_build_string("\0", (512 - ($file["size"] % 512)));
                    $tmp .= $homer;
                }
            }
            $tardata .= $tmp;
        }

        // Add the footer
        $tardata .= pack("a512", "");

        $this->data = $tardata;
    }

    // Saves the archive to disk
    function save_archive()
    {
        global $settings;

        if (($fp = fopen($this->file_path_name, "wb")) == true)
        {
            fputs($fp, $this->data, strlen($this->data));
            fclose($fp);
            chmod($this->file_path_name, octdec($settings["default_chmod"]));

            return true;
        }
        else
        {
            $this->no_save = true;
        }
    }

    // Downloads the archive
    function download_archive()
    {
        $browser = web_browser();
        $type    = ($browser == "ie" || $browser == "opera") ? "application/octetstream" : "application/octet-stream";

        // Set the required HTTP headers
        header("Content-Type: " . $type, true);
        header("Content-Transfer-Encoding: binary");
        header("Content-Length: " . strlen($this->data));
        header("Content-Disposition: attachment; filename=" . str_replace(" ", "_", $this->file_name)); // Doesn't like spaces in file names

        echo $this->data;
        exit;
    }

    // Compresses the file using GZip compression
    function gzip_compress($type)
    {
        global $settings;
        $this->data = gzencode($this->data, $settings["gzip_rate"]);
    }

    // Compresses the file using BZip2 compression
    function bzip2_compress()
    {
        global $settings;
        $this->data = bzcompress($this->data, $settings["bzip2_rate"]);
    }

    // Extracts the files from the archive
    function extract_files($path)
    {
        // Need to add space and filetype checking as well as clean up errors to match IPFM

	    global $settings, $permissions, $input;

        // Get the current directory size
        $size = directory_size($permissions["root"]);

        if(file_exists($path) == false)
        {
            if (mkdir($path, octdec((int) $settings["default_chmod"])) == false)
            {
                // Not much point in continuing if we can't make the directory
                $this->no_dir_open[] = $path;
                $this->errors++;

                return false;
            }
            else
            {
                chmod($path, octdec((int) $settings["default_chmod"]));
            }
        }

		// Change into the directory chosen by the user.
		chdir($path);
		$cur_dir = getcwd();

		// Get the file info from the tar
		$in_files = $this->read_tar();

		if ($this->open_error == true)
        {
			return false;
		}

		foreach ($in_files as $k => $file)
        {
			chdir($cur_dir);

			// GNU TAR format dictates that all paths *must* be in the *nix
			// format - if this is not the case, blame the tar vendor, not me!
			if (preg_match("/\//", $file["name"]) == true)
			{
				$path_info = explode("/", $file["name"]);
				$file_name = array_pop($path_info);
			}
            else
			{
				$path_info = array();
				$file_name = $file["name"];
			}

			// If we have a path, then we must build the directory tree
			if (count($path_info) > 0)
			{
				foreach($path_info as $dir_component)
				{
					if ($dir_component == "")
					{
						continue;
					}

					if (file_exists($dir_component) == true && is_dir($dir_component) == false)
					{
						continue;
					}

					if (is_dir($dir_component) == false)
					{
						if (mkdir($dir_component, octdec((int) $settings["default_chmod"])) == false)
                        {
                            // Not much point in continuing if we can't make the directory
                            $this->no_dir_open[] = $dir_component;
                            $this->errors++;

                            return false;
                        }
						chmod($dir_component, octdec((int) $settings["default_chmod"]));
					}

					chdir($dir_component);
				}
			}

			// check the typeflags, and work accordingly
			if ($file["typeflag"] == 0 || empty($file["typeflag"]) == true || isset($file["typeflag"]) == false)
			{
                $fsize = strlen($file["data"]);

                if (file_permission($file_name) == false)
                {
                    $this->noext = true;
                    $this->errors++;
                }
                else if (file_exists($file_name) == true && $permissions["overwrite"] == 0)
                {
                    $this->file_exists[] = basename($file_name);
                    $this->errors++;
                }
                else if (file_exists($file_name) == true && $input["overwrite"] != 1)
                {
                    $this->no_overwrite_file[] = basename($file_name);
                    $this->errors++;
                }
                else if ($size + $fsize > $permissions["space"])
                {
                    // Continue on to the next file as it may be smaller
                    $this->nospace[] = basename($file_name);
                    $this->errors++;
                }
                else
                {
    	    	    if (($fp = fopen($file_name, "wb")) == true)
	    	    	{
				        fputs($fp, $file["data"], $fsize);
        				fclose($fp);

                        $size = $size + $fsize;

                        $this->total_files++;
                    }
        			else
		        	{
                        $this->errors++;
                        $this->openfile[] = $file_name;
        			}
                }
            }
			else if ($file["typeflag"] == 5)
			{
				if (file_exists($file_name) == true && is_dir($file_name) == false)
				{
					continue;
				}

				if (is_dir($file_name) == false)
				{
					mkdir($file_name, octdec((int) $settings["default_chmod"]));
                    chmod($file_name, octdec((int) $settings["default_chmod"]));
				}
			}
			else
			{
                // This triggers if we have a symlink, block device, named pipe or some unknown type
                // Do we need to print a warning? As we don't have this problem with zip so we would require
                // additional debugging
                continue;
            }

			if (chmod($file_name, $file["mode"]) == false)
			{
                // Can't CHMOD to predefined mode, so use the default mode
				chmod($file_name, octdec((int) $settings["default_chmod"]));
			}

            // Set the mtime
            if (file_exists($file_name) == true)
            {
			    touch($file_name, $file["mtime"]);
            }
		}

		// Return to the "real" directory the scripts are in
		chdir($this->current_dir);

        // Old file is set, which means we created a temp file, so we better delete it
        if (empty($this->old_file_name) == false)
        {
            unlink($this->file_path_name);

            $this->file_name      = $this->old_file_name;
            $this->file_path_name = $this->file_path . "/" . $this->file_name;
        }

        return;
	}

	// Read the tarball - builds an associative array
	function read_tar()
    {
		$tar_info   = array();

        $type       = ext($this->file_name);
        $type       = $type == "tgz" ? "gz" : $type;
        $this->type = $type;

        $this->decompress();

        if ($this->open_error == true)
        {
			return false;
        }

		// Open up the tar file and start the loop
		if (($fp = fopen($this->file_path_name, "rb")) == false)
        {
			$this->open_error = true;
			return false;
		}

		// Grrr, perl allows spaces, PHP doesn't. Pack strings are hard to read without
		// them, so to save my sanity, I'll create them with spaces and remove them here
		$this->tar_unpack_header = preg_replace("/\s/", "", $this->tar_unpack_header);

		while (feof($fp) == false)
        {
			$buffer = fread($fp , $this->tar_header_length);

			// check the block
			$checksum = 0;

			for ($i = 0; $i < 148; $i++)
            {
				$checksum += ord(substr($buffer, $i, 1));
			}

			for ($i = 148; $i < 156; $i++)
            {
				$checksum += ord(" ");
			}

			for ($i = 156; $i < 512; $i++)
            {
				$checksum += ord(substr($buffer, $i, 1));
			}

			$fa       = unpack($this->tar_unpack_header, $buffer);

			$name     = trim($fa["filename"]);
			$mode     = octdec(trim($fa["mode"]));
			$uid      = octdec(trim($fa["uid"]));
			$gid      = octdec(trim($fa["gid"]));
			$size     = octdec(trim($fa["size"]));
			$mtime    = octdec(trim($fa["mtime"]));
			$chksum   = octdec(trim($fa["chksum"]));
			$typeflag = trim($fa["typeflag"]);
			$linkname = trim($fa["linkname"]);
			$magic    = trim($fa["magic"]);
			$version  = trim($fa["version"]);
			$uname    = trim($fa["uname"]);
			$gname    = trim($fa["gname"]);
			$devmajor = octdec(trim($fa["devmajor"]));
			$devminor = octdec(trim($fa["devminor"]));
			$prefix   = trim($fa["prefix"]);

			if ($checksum == 256 && $chksum == 0)
            {
				// EOF!
				break;
			}

			if (isset($prefix) == true)
            {
				$name = $prefix . "/" . $name;
			}

			// Some broken tars don't set the type flag
			// correctly for directories, so we assume that
			// if it ends in / it's a directory...
			if (preg_match("#/$#", $name) == true && (!$name))
            {
				$typeflag = 5;
			}

			// If it's the end of the tarball...
			$test = $this->internal_build_string("\0", 512);
			if ($buffer == $test)
            {
				break;
			}

			// Read the next chunk
			$data = fread($fp, $size);

			if (strlen($data) != $size)
            {
                // Read error
				fclose($fp);
				return false;
			}

			$diff = $size % 512;
			if ($diff != 0)
            {
				// Padding, throw away
				$crap = fread($fp, (512 - $diff));
			}

			// Protect against tarfiles with garbage at the end
			if ($name == "")
            {
				break;
			}

			$tar_info[] = array (
								  "name"     => $name,
								  "mode"     => $mode,
								  "uid"      => $uid,
								  "gid"      => $gid,
								  "size"     => $size,
								  "mtime"    => $mtime,
								  "chksum"   => $chksum,
								  "typeflag" => $typeflag,
								  "linkname" => $linkname,
								  "magic"    => $magic,
								  "version"  => $version,
								  "uname"    => $uname,
								  "gname"    => $gname,
								  "devmajor" => $devmajor,
								  "devminor" => $devminor,
								  "prefix"   => $prefix,
								  "data"     => $data
                                 );
		}

		fclose($fp);

        return $tar_info;
	}

    // Decompresses a file
    function decompress()
    {
        global $settings;

        if ($this->type == "tar")
        {
            return;
        }

        $data = "";
        if ($this->type == "bz2")
        {
            if (($fp = bzopen($this->file_path_name, "rb")) == false)
            {
                $this->open_error = true;
	    		return false;
            }
            else
            {
                while (feof($fp) == false)
                {
                    $data .= bzread($fp, 2048);
                }

                bzclose($fp);
            }
        }
        else
        {
            if (($fp = gzopen($this->file_path_name, "rb")) == false)
            {
                $this->open_error = true;
	    		return false;
            }
            else
            {
                while (gzeof($fp) == false)
                {
                    $data .= gzread($fp, 2048);
                }

                gzclose($fp);
            }
        }

        $tmp = tempnam($settings["script_dir"] . "tmp", "");
        $tmp = str_replace("\\", "/", $tmp); // Small windows fix

        $this->old_file_name  = $this->file_name;
        $this->file_name      = basename($tmp);
        $this->file_path_name = $tmp;

        if (($fp = fopen($tmp, "w")) == false)
        {
            $this->open_error = true;
	    	return false;
        }
        else
        {
            fwrite($fp, $data);
            fclose($fp);
        }

        return true;
    }

    // Builds a repititive string
    function internal_build_string($string = "", $times = 0)
    {
        $return = "";

        for ($i=0; $i < $times; ++$i)
        {
            $return .= $string;
        }

        return $return;
    }
}

?>