<?php

/*
 * Invision Power File Manager
 * Zip Archiving
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.0
 * Last Modified: 9th August 2004
 * -------------------------------------
 *
 * Modification of the Zip compression class from Zend
 * Based on :
 *
 * http://www.zend.com/zend/spotlight/creating-zip-files1.php
 * By John Coggeshall
 *
 * Official ZIP file format: http://www.pkware.com/appnote.txt
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

class archive
{
    var $data_in_mem       = array(); // Array to store compressed data
    var $ctrl_dir          = array(); // Central directory
    var $eof_ctrl_dir      = "\x50\x4b\x05\x06\x00\x00\x00\x00";
    var $old_offset        = 0;

    var $file_name         = "";
    var $file_path         = "";
    var $file_path_name    = "";

    var $workfiles         = array();

    // Error data
    var $errors            = 0;
    var $nodir             = array();
    var $no_dir_open       = array();
    var $sysdir            = array();
    var $openfile          = array();
    var $name_length       = array();
    var $path_length       = array();
    var $nospace           = array();
    var $file_exists       = array();
    var $no_overwrite_file = array();
    var $no_save           = false;
    var $noext             = false;
    var $open_error        = false;
    var $total_files       = 0;

    // Constructor
    function archive($path, $name)
    {
        $this->file_name = $name;
        $this->file_path = $path;

        $this->file_path = preg_replace("#[/\\\]$#", "", $this->file_path);
        $this->file_path_name = $this->file_path . "/" . $this->file_name;
    }

    function add_directory($dir)
    {
        $this->workfiles = array();

        if (is_dir($dir) == false)
        {
            $this->nodir[] = basename($dir);
            $this->errors++;
            return false;
        }

        // Reset the file list to prevent files from being added many times
        $this->get_dir_contents($dir);
        $this->add_files($this->workfiles);

        return true;
    }

    // Get the directories content
    function get_dir_contents($dir)
    {
        $dir = preg_replace("#/$#", "", $dir);

        if (file_exists($dir) == true)
        {
            if (is_dir($dir) == true)
            {
                $handle = opendir($dir);

                while (($filename = readdir($handle)) !== false)
                {
                    if ($filename != "." && $filename != "..")
                    {
                        if (is_dir($dir . "/" . $filename . "/") == true && check_link($dir . "/" . $filename . "/") == false)
                        {
                            if (is_sys_file($filename, true) == true)
                            {
                                $this->sysdir[] = $filename;
                                $this->errors++;
                            }
                            else
                            {
                                $this->get_dir_contents($dir . "/" . $filename . "/");
                            }
                        }
                        else
                        {
                            $this->workfiles[] = $dir . "/" . $filename;
                        }
                    }
                }

                closedir($handle);
            }
            else
            {
                $this->error = "$dir is not a directory";
                return FALSE;
            }
        }
        else
        {
            $this->no_dir_open[] = $dir;
            return;
        }
    }

    // Adds an array of files to the archive
    function add_files($files)
    {
        global $settings, $permissions;

        if (count($files) < 1)
        {
            return;
        }

        foreach ($files as $file)
        {
            // Skip any system files
            if (preg_match("/\.ds_store/i", $file) == false && is_sys_file($file) == false && file_permission($file) == true)
            {
                if ($fp = fopen($file, "rb"))
                {
                    $data = fread($fp, filesize($file));
                    fclose($fp);

                    $this->add_data($data, $file);
                }
                else
                {
                    $this->openfile[] = $file;
                    $this->errors++;
                    continue;
                }
            }
            else
            {
                $this->noext = true;
                $this->errors++;
            }
        }
        return;
    }

    // Creates the file $name and then add the content of $data to it
    function add_data($data, $name)
    {
        global $permissions;

        $name = str_replace($permissions["root"], "", $name);

        $fr   = "\x50\x4b\x03\x04";
        $fr   .= "\x14\x00";            // Version needed to extract
        $fr   .= "\x00\x00";            // General purpose bit flag
        $fr   .= "\x08\x00";            // Compression method
        $fr   .= "\x00\x00\x00\x00";    // Last modified time and date

        // "Local file header" segment
        $unc_len = strlen($data);
        $crc     = crc32($data);
        $zdata   = gzcompress($data);
        $zdata   = substr(substr($zdata, 0, strlen($zdata) - 4), 2); // Fix CRC bug
        $c_len   = strlen($zdata);
        $fr      .= pack('V', $crc);             // CRC32
        $fr      .= pack('V', $c_len);           // Compressed filesize
        $fr      .= pack('V', $unc_len);         // Uncompressed filesize
        $fr      .= pack('v', strlen($name));    // Length of filename
        $fr      .= pack('v', 0);                // Extra field length
        $fr      .= $name;

        // "File data" segment
        $fr .= $zdata;

        // "Data descriptor" segment (optional but necessary if archive is not served as file)
        $fr .= pack('V', $crc);                 // CRC32
        $fr .= pack('V', $c_len);               // Compressed filesize
        $fr .= pack('V', $unc_len);             // Uncompressed filesize

        // Add this entry to array
        $this->data_in_mem[] = $fr;

        // Now add to central directory record
        #$new_offset      = strlen(implode("", $this->data_in_mem));
        $new_offset       = $this->old_offset + strlen($fr);

        $cdrec = "\x50\x4b\x01\x02";
        $cdrec .= "\x00\x00";                // Version made by
        $cdrec .= "\x14\x00";                // Version needed to extract
        $cdrec .= "\x00\x00";                // General purpose bit flag
        $cdrec .= "\x08\x00";                // Compression method
        $cdrec .= "\x00\x00\x00\x00";        // Last modified time and date
        $cdrec .= pack('V', $crc);           // CRC32
        $cdrec .= pack('V', $c_len);         // Compressed filesize
        $cdrec .= pack('V', $unc_len);       // Uncompressed filesize
        $cdrec .= pack('v', strlen($name));  // Kength of filename
        $cdrec .= pack('v', 0);              // Extra field length
        $cdrec .= pack('v', 0);              // File comment length
        $cdrec .= pack('v', 0);              // Disk number start
        $cdrec .= pack('v', 0);              // Internal file attributes
        $cdrec .= pack('V', 32 );            // External file attributes - 'archive' bit set

        $cdrec .= pack('V', $this->old_offset); // Relative offset of local header
        $this->old_offset = $new_offset;

        $cdrec .= $name;

        // Optional extra field, file comment goes here save to central directory
        $this->ctrl_dir[] = $cdrec;
    }

    // Creates and stores the archive in memory
    function create_archive()
    {
        $data    = implode("", $this->data_in_mem);
        $ctrldir = implode("", $this->ctrl_dir);

        $this->data = $data . $ctrldir . $this->eof_ctrl_dir .
            pack('v', sizeof($this->ctrl_dir)) .  // Total # of entries "on this disk"
            pack('v', sizeof($this->ctrl_dir)) .  // Total # of entries overall
            pack('V', strlen($ctrldir)) .         // Size of central dir
            pack('V', strlen($data)) .            // Offset to start of central dir
            "\x00\x00";                           // .zip file comment length

        return;
    }

    // Saves the archive to disk
    function save_archive()
    {
        global $settings;

        if (($fp = fopen($this->file_path_name, "wb")) == true)
        {
            fputs($fp, $this->data, strlen($this->data));
            fclose($fp);
            chmod($this->file_path_name, octdec($settings["default_chmod"]));

            return true;
        }
        else
        {
            $this->no_save = true;
        }
    }

    // Downloads the archive
    function download_archive()
    {
        $browser = web_browser();
        $type    = ($browser == "ie" || $browser == "opera") ? "application/octetstream" : "application/octet-stream";

        // Set the required HTTP headers
        header("Content-Type: " . $type, true);
        header("Content-Transfer-Encoding: binary");
        header("Content-Length: " . strlen($this->data));
        header("Content-Disposition: attachment; filename=" . str_replace(" ", "_", $this->file_name)); // Doesn't like spaces in file names

        echo $this->data;
        exit;
    }

    // Extracts the archive
    function extract_files($path)
    {
        global $settings, $input, $permissions;

        // Get the current directory size
        $size = directory_size($permissions["root"]);

        if (($fp = zip_open($this->file_path_name)) == true)
        {
            while ($entry = zip_read($fp))
            {
                if (zip_entry_filesize($entry) > 0)
                {
                    $this->total_files++;

                    $file  = zip_entry_name($entry);
                    $fsize = zip_entry_filesize($entry);

                    $complete_name = $path . $file;
                    $complete_path = $path . dirname($file) . "/";

                    // If the path ends in ./ strip it off
                    if (preg_match("/\.\/$/", $complete_path) == true)
                    {
                        $complete_path = substr($complete_path, 0, strlen($complete_path) - 2);
                    }

                    if(file_exists($complete_path) == false)
                    {
                        if (create_directory_structure($complete_path) == false)
                        {
                            // Not much point in continuing if we can't make the directory
                            $this->no_dir_open[] = $tmp;
                            $this->errors++;

                            return false;
                        }
                    }

                    if (file_permission($file) == false)
                    {
                        $this->noext = true;
                        $this->errors++;
                    }
                    else if (file_exists($complete_name) == true && $permissions["overwrite"] == 0)
                    {
                        $this->file_exists[] = basename($complete_name);
                        $this->errors++;
                    }
                    else if (file_exists($complete_name) == true && $input["overwrite"] != 1)
                    {
                        $this->no_overwrite_file[] = basename($complete_name);
                        $this->errors++;
                    }
                    else if ($size + $fsize > $permissions["space"])
                    {
                        // Continue on to the next file as it may be smaller
                        $this->nospace[] = basename($complete_name);
                        $this->errors++;
                    }
                    else
                    {
                        if (zip_entry_open($fp, $entry, "rb"))
                        {
                            if (($fd = fopen($complete_name, "wb")) == false)
                            {
                                $this->errors++;
                                $this->openfile[] = basename($complete_name);
                            }
                            else
                            {
                                fwrite($fd, zip_entry_read($entry, $size));
                                fclose($fd);

                                chmod($complete_name, octdec((int) $settings["default_chmod"]));

                                // File was created so increase the size
                                $size = $size + $fsize;
                            }
                            zip_entry_close($entry);
                        }
                    }
                }
            }
            zip_close($fp);
        }
        else
        {
            $this->open_error = true;
            return false;
        }
    }
}

?>