<?php

/*
 * Invision Power File Manager
 * File Editor
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.1
 * Last Modified: 24th February 2005
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

/* -----------------------------------------
               Module Functions
   ----------------------------------------- */

// Shows the editor
function show_editor_form($msg = "")
{
    global $permissions, $input, $system, $page, $templates, $settings;

    if (isset($input["isnew"]) == false || isset($input["op"]) == true)
    {
        $fullpath = $permissions["root"] . $input["target"];

        $dirpath = "/" .  dirname($fullpath) . "/";
        $page["file"] = make_navigation($dirpath);
        $page["file"] .= $permissions["download"] == 1 ? "<a href=\"index.php?act=list&amp;op=get&amp;target=" . $input["target"] . "\">" . basename($input["target"]) . "</a>" : basename($input["target"]);

        // 'isnew' isn't set, so first check for edit permission
        if ($permissions["edit"] == 0)
        {
            go_to("index.php?op=list&dir=" . $dir . "&msg=no_edit_permission");
        }
        else if (file_exists($fullpath) == false)
        {
            go_to("index.php?op=list&dir=" . $dir . "&msg=file_not_exist");
        }
        else if (file_permission($fullpath) == false || is_ascii($fullpath) == false)
        {
            go_to("index.php?op=list&dir=" . $dir . "&msg=no_edit_file_permission");
        }
        else
        {
            if (file_exists($fullpath . $settings["editing"]) == true && $settings["edit_lock"] == true)
            {
                // If the amount of time left is less than or equal to 0 unlock the file
                if (filemtime($fullpath . $settings["editing"]) - (time() - (int) $settings["locktimeout"]) <= 0)
                {
                    unlink($fullpath . $settings["editing"]);
                }
            }

            if (file_exists($fullpath . $settings["editing"]) == true && $settings["edit_lock"] == true)
            {
                if ($input["op"] == "save")
                {
                    continue;
                }

                go_to("index.php?op=list&dir=" . substr($input["target"], 0, strrpos($input["target"], "/")) . "&msg=file_locked");
            }
            else
            {
                if ($input["op"] == "save")
                {
                    $page["file_data"] = $input["body"];
                }
                else
                {
                    if (($fp = fopen($fullpath, "r")) == true)
                    {
                        file_lock($fp, LOCK_SH);
                        $page["file_data"] = trim(fread($fp, filesize($fullpath)));
                        file_lock($fp, LOCK_UN);
                        fclose($fp);

                        if ($settings["edit_lock"] == 1)
                        {
                            touch($fullpath . $settings["editing"]);
                            chmod($fullpath . $settings["editing"], 0777);
                        }

                        $page["file_data"] = make_safe($page["file_data"]);

                        // Get the template ID
                        $ext = ext($page["filename"]);
                        $tmp = array_keys($templates["extension"], $ext);
                    }
                    else
                    {
                        go_to("index.php?op=list&dir=" . $dir . "&msg=error_reading_file");
                    }
                }

                $page["filename"] = isset($input["filename"]) == true ? $input["filename"] : basename($fullpath);
                $page["isnew"]    = 0;

                $blocks = get_blocks($settings["script_dir"] . "skin/editor.tpl", array("documentation", "error"));
                if (empty($templates["help"][$tmp[0]]) == true)
                {
                    $blocks["documentation"] = "";
                }

                if (empty($msg) == false)
                {
                    $page["msg"] = $msg;
                }
                else
                {
                    $blocks["error"] = "";
                }

                $icon = get_icon($fullpath);
                if (check_link($fullpath) == true)
                {
                    // Check if the file is a symlink
                    $icon = get_icon($fullpath, LINK);
                }

                $page["link"]      = "<a href=\"" . $templates["help"][$tmp[0]] . "\" target=\"_blank\">" . $templates["help"][$tmp[0]] . "</a>";
                $page["icon"]      = icon_link($page["filename"], preg_replace("/^[\/]/", "", $dirpath), $icon);
                $page["content"]   = parse_html($page, dynamic_template($settings["script_dir"] . "skin/editor.tpl", $blocks));
            }
        }
    }
    else
    {
        if ($permissions["newfile"] == 0)
        {
            go_to("index.php?op=list&dir=" . $dir . "&msg=no_newfile_permission");
        }
        else
        {
            $page["file_data"] = "";

            $blocks = get_blocks($settings["script_dir"] . "skin/editor.tpl", array("documentation", "error"));
            if (empty($templates["help"][$input["template"]]) == true)
            {
                $blocks["documentation"] = "";
            }

            $blocks["error"] = "";

            if ($input["template"] != 0)
            {
                $fp = fopen($settings["script_dir"] . "templates/" . $templates["filename"][$input["template"]] . ".tpl", "r");
                $page["file_data"] = fread($fp, filesize($settings["script_dir"] . "templates/" . $templates["filename"][$input["template"]] . ".tpl"));
                fclose($fp);
            }
            else
            {
                $templates["extension"][$input["template"]] = $settings["default_type"];
            }

            $page["filename"] = $page["untitled"] . "." . $templates["extension"][$input["template"]];
            $page["file"]     = make_navigation($input["target"]) . $page["filename"];

            $input["target"]  = $input["target"] . "/" . $page["filename"];

            $page["icon"]     = get_icon($page["filename"]);
            $page["link"]     = "<a href=\"" . $templates["help"][$input["template"]] . "\" target=\"_blank\">" . $templates["help"][$input["template"]] . "</a>";
            $page["isnew"]    = 1;
            $page["content"]  = parse_html($page, dynamic_template($settings["script_dir"] . "skin/editor.tpl", $blocks));
        }
    }
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */

$input["target"] = clean_path($input["target"]);
$dir             = substr($input["target"], 0, strrpos($input["target"], "/"));

if ($permissions["edit"] == 0 && $permissions["newfile"] == 0)
{
    go_to("index.php?op=list&dir=" . $dir . "&msg=no_editor_permission");
}
else
{
    switch ($input["op"])
    {
        case "save":
            /* ----------------------------
                     Save the changes
               ---------------------------- */
            $input["filename"] = clean_path($input["filename"]);

            $oldfile = $permissions["root"] . $input["target"];
            if (empty($dir) == true)
            {
                $fullpath = $permissions["root"] . $input["filename"];
            }
            else
            {
                $fullpath = $permissions["root"] . $dir . "/" . $input["filename"];
            }

            // Delete the edit lock if it exists
            if (file_exists($permissions["root"] . $input["target"] . $settings["editing"]) == true && $settings["edit_lock"] == true)
            {
                unlink($permissions["root"] . $input["target"] . $settings["editing"]);
            }

            $page["body"] = make_unsafe($page["body"]);

            // Get the space
            $used = directory_size($permissions["root"]) + strlen($input["body"]);

            $exists = false;
			if (file_exists($fullpath) == true)
            {
                $used = $used - filesize($fullpath);
                if ($input["isnew"] == 1 && $permission["edit"] == 0)
                {
                    show_editor_form($page["no_edit_permission"]);
                }
                else if ($permissions["newfile"] == 0)
                {
                    show_editor_form($page["no_newfile_permission"]);
                }

                $exists = true;
            }

            if (is_invalid_name($input["filename"]) == true || empty($input["filename"]) == true)
            {
                show_editor_form($page["name_invalid"]);
            }
            else if (file_permission($fullpath) == false)
            {
                show_editor_form($page["no_edit_file_permission"]);
            }
            else if (is_ascii($fullpath) == false)
            {
                show_editor_form($page["not_ascii_file"]);
            }
            else if ($used > $permissions["space"])
            {
                show_editor_form($page["low_space"]);
            }
            else
            {
                if (($fp = fopen($fullpath, "w")) == false)
                {
                    show_editor_form($page["file_save_error"]);
                }
                else
                {
                    file_lock($fp);
                    fwrite($fp, $input["body"]);
                    file_lock($fp, LOCK_UN);
                    fclose($fp);

                    if ($exists == false)
                    {
                        chmod($fullpath, octdec($settings["default_chmod"]));
                    }

                    go_to("index.php?op=list&dir=" . $dir . "&msg=file_saved");
                }
            }
        break;
        case "cancel":
            if (file_exists($permissions["root"] . $input["target"] . $settings["editing"]) == true)
            {
                unlink($permissions["root"] . $input["target"] . $settings["editing"]);
            }

            go_to("index.php?op=list&dir=" . $dir);
        break;
        case "preview":
            /* ----------------------------
                   Preview the changes
               ---------------------------- */
            $page["body"] = make_unsafe($page["body"]);

            // Change into the scripts directory so that PHP works
            if (empty($dir) == true)
            {
                chdir($permissions["root"]);
                $url = $permissions["root_url"];
            }
            else
            {
                chdir($permissions["root"] . $dir . "/");
                $url = $permissions["root_url"] . relative_path("", $dir, 1) . "/";
            }

            // Check that the root url is set
            if (empty($permissions["root_url"]) == false && $permissions["show_url"] == 1)
            {
                // See if the base tag already exists
                if (preg_match("<base href=(.*)>", $input["body"]) == false)
                {
                    // Add the base tag so links etc work
                    $input["body"] = str_replace("</head>", "<base href=\"" . $url . "\" /></head>", $input["body"]);
                }
            }

            if (in_array(ext($input["filename"]), array("php", "php4", "php3", "phtml")) == true && file_permission($input["filename"]) == true)
            {
                ob_start();
                eval("?>" . $input["body"]);
                $input["body"] = ob_get_contents();
                ob_end_clean();
            }

            echo $input["body"];
            exit;
        break;
        default:
            /* ----------------------------
                        File editor
               ---------------------------- */
            show_editor_form();
    }
}

$page["js"]     = "edit";
$page["target"] = $input["target"];

?>