<?php

/*
 * Invision Power File Manager
 * Archive extractor dialog
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.1
 * Last Modified: 23rd September 2004
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

 /* -----------------------------------------
               Module Functions
   ----------------------------------------- */

// Traverses a directory and returns an array of it's content
function traverse($dir)
{
    global $settings, $permissions, $page, $input;

    // Create the function variables
    $dirs   = array();
    $total  = 0;
    $count  = 0;
    $up     = "";
    $msg    = "";
    $_dirs  = array();

    // Reverse any user inputted \ slashes
    $dir = str_replace("\\", "/", $dir);

    // Does the directory exist?
    if (file_exists($dir) == false)
    {
        if ($dir == $permissions["root"])
        {
            // Fatal error! The root directory could not be opened!
            return array("folders" => "", "msg" => $page["no_root_directory"], "fatal" => 1, "up" => "", "nav" => "", "dir" => "", "count" => $count, "back_img" => "");
        }
        else
        {
            // The directory could not be opened, but was not root so start again at root
            $returned = traverse($permissions["root"]);

            return array("folders" => $returned["folders"], "msg" => $page["no_browse_directory"], "up" => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"]);
        }
    }
    else if (is_sys_file($dir) == true)
    {
        // The directory is hidden, so start again at the root
        $returned = traverse($permissions["root"]);

        return array("folders" => $returned["folders"], "msg" => $page["hidden_directory"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "count" => $returned["count"], "back_img" => $returned["back_img"]);
    }
    else if (check_link($dir) == true)
    {
        if (preg_match("/^" . str_replace("/", "\/", $permissions["root"]) . "/", realpath($dir) . "/") == false)
        {
            // The target of this link is outside of the users root folder, so go back to root
            $returned = traverse($permissions["root"]);

            return array("folders" => $returned["folders"], "this" => $returned["this"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["dir_is_a_disallowed_link"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"], "pages" => $returned["pages"]);
        }
        else
        {
            // The directory is a link, so resolve it
            $returned = traverse(realpath($dir) . "/");

            return array("folders" => $returned["folders"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["dir_is_a_link"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"]);
        }
    }


    // Open the directory and then check for errors
    chdir($dir);
    if (($dp = opendir($dir)) == false)
    {
        // Directory could not be opened
        if ($dir == $permissions["root"])
        {
            // Fatal error! The root directory could not be opened!
            return array("folders" => "", "msg" => $page["root_not_open"], "fatal" => 1, "up" => "", "nav" => "", "dir" => "", "count" => 0, "back_img" => "");
        }
        else
        {
            // The directory could not be opened, but was not root so start again at root
            $returned = traverse($permissions["root"]);

            return array("folders" => $returned["folders"], "msg" => $page["directory_not_open"], "up" => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"]);
        }
    }

    // Construct an array of files and directories
    while (($entry = readdir($dp)) != false)
    {
        if (is_dir($dir . $entry) == true && $entry != ".")
        {
            // Determine if the directory should be hidden
            if ($entry == "..")
            {
                // Make sure that the back link isn't hidden
                $_dirs[] = $entry;
            }
            else if ($settings["hide_dot"] == false || ($settings["hide_dot"] == true && preg_match("/^\./", $entry) == false))
            {
                if (check_link($dir . $entry) == true && preg_match("/^" . str_replace("/", "\/", $permissions["root"]) . "/", realpath($dir . $entry) . "/") == false)
                {
                    continue;
                }
                $_dirs[] = $entry;
            }
        }
    }
    closedir($dp);

    // Sort the arrays
    natcasesort($_dirs);

    $thisdir = preg_replace("/^\//",  "", relative_path($dir, "", 1));
    $thisdir = preg_replace("/\/$/",  "", $thisdir);

    // Proceed through the directory list getting any required information about the entry
    foreach($_dirs as $entry)
    {
        if ($entry == "..")
        {
            if ($dir != $permissions["root"])
            {
                $prev = str_replace($permissions["root"],  "", $dir);
                $prev = preg_replace("/[\/]$/", "", $prev);
                $prev = preg_replace("/" . basename($dir) . "$/", "", $prev);
                $prev = preg_replace("/[\/]$/", "", $prev);

                $backimg = "<a href=\"index.php?act=ext&amp;dir=" . url_safe($prev) . "&amp;target=" . url_safe($input["target"]) . "&amp;archive=" . url_safe($input["archive"]) . "\"><img src=\"images/icons/back.gif\" border=\"0\" alt=\"" . $page["back"] . "\" title=\"" . $page["back"] . "\" /></a>";
                $up      = "<a href=\"index.php?act=ext&amp;dir=" . url_safe($prev) . "&amp;target=" . url_safe($input["target"]) . "&amp;archive=" . url_safe($input["archive"]) . "\">" . $page["back"] . "</a>";
            }
        }
        else
        {
            $count++;
            $unix = unix_info($dir . $entry . "/");

            // If the folder is locked then show a locked icon
            $icon = "folder";

            if (preg_match("/apache/i", $_SERVER["SERVER_SOFTWARE"]) && file_exists($dir . $entry . "/" . $settings["htaccess"]) == true && file_exists($dir . $entry . "/" . $settings["htpasswd"]) == true && $permissions["show_url"] == 1)
            {
                $icon = "lockedfolder";
            }
            else if (check_link($dir . $entry) == true)
            {
                // is_link doesn't work with directories on some servers, so we get the realpath and then check our existing path
                $icon = "linkedfolder";
            }

            $icon = icon_link($entry, $dir, $icon, 1, "ext&amp;dir=" . relative_path($entry, $dir) . "&amp;target=" . url_safe($input["target"]));

            $dirs[$entry] = array(
                            "name"      => $entry,
                            "name_link" => "<a href=\"index.php?act=ext&amp;dir=" . relative_path($entry, $dir) . "&amp;target=" . url_safe($input["target"]) . "&amp;archive=" . url_safe($input["archive"]) . "\">" . $entry . "</a>",
                            "open"      => empty($permissions["root_url"]) == false && $permissions["show_url"] == 1 ? "(<a href=\"" . $permissions["root_url"] . relative_path($entry, $dir, 1) . "/\" target=\"_blank\">" . $page["open_url"] . "</a>)" : "",
                            "icon"      => $icon,
                            );
            $count++;
        }
    }

    // Create the navigation link
    $nav      = str_replace($permissions["root"], "", $dir);
    $exploded = explode("/", $nav);

    $nav = "/";
    $next = "";
    foreach ($exploded as $var)
    {
        if ($var != "")
        {
            $next .= $var;
            $nav .= "<a href=\"index.php?act=ext&amp;dir=" . url_safe($next) . "&amp;target=" . url_safe($input["target"]) . "&amp;archive=" . url_safe($input["archive"]) . "\">" . $var . "</a>" . "/";
            $next .= "/";
        }
    }

    $dir = "/" . relative_path($dir, "", 1);

    // If we are running on Windows we need to reverse the slashes
    if (is_win() == true)
    {
        // Replace the slashes, dirty but it does the job
        $nav = preg_replace("/^[\/]/", "\\", $nav);
        $nav = preg_replace("/<\/a>[\/]/", "</a>\\", $nav);

        $dir = preg_replace("/[\/]/", "\\", $dir);
    }

    return array("dir" => $dir, "this" => rawurlencode($thisdir), "nav" => $nav, "folders" => $dirs, "up" => $up, "msg" => "", "count" => $count, "back_img" => $backimg);
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */

// Set some parameters
$global_tpl = "skin/popup.tpl";

$dir        = clean_path($input["dir"]);

if (empty($dir) == true)
{
    $dir = $permissions["root"];
}
else
{
    $dir = $permissions["root"] . $dir . "/";
}

$info      = traverse($dir);
$bookmarks = bookmark_form();

// Show the border areas if needed
$info["upperall"]  = $info["count"] > 1 ? 1 : 0;
$info["lowerall"]  = $info["count"] > 1 ? 1 : 0;
$info["bookmarks"] = $bookmarks["count"] > 0 ? 1 : 0;

$info["msg"]       = $info["count"] < 1 ? $page["empty_directory"] : $info["msg"];

$blocks = array("msg", "up", "folders", "bookmarks", "overwrite", "delete");
$blocks = get_blocks($settings["script_dir"] . "skin/cards/extract.tpl", $blocks);

if ($permissions["delete"] == 0)
{
    $blocks["delete"] = "";
}
if ($permissions["overwrite"] == 0)
{
    $blocks["overwrite"] = "";
}

$blocks = parse_blocks($info, $blocks);

// There is a fatal error!
if (isset($info["fatal"]))
{
    // Finish up by outputting the completed page and then exit
    $page["content"] = $blocks["msg"];
    $page["title"]   = $page["fatal_error"];
    exit(static_template($settings["script_dir"] . "skin/popup.tpl", $page, 1));
}

// Construct the page
$page["content"]    = parse_html($page, dynamic_template($settings["script_dir"] . "skin/cards/extract.tpl", $blocks));
$page["js"]         = "move";
$page["this"]       = $info["this"];
$page["nav"]        = $info["nav"];
$page["back_img"]   = $info["back_img"];
$page["dir"]        = str_replace("%23", "#", $info["dir"]);
$page["booklist"]   = $bookmarks["list"];

$page["target"]     = str_replace("#", "%23", $input["target"]);
$page["archive"]    = str_replace("#", "%23", $input["archive"]);
$page["dir"]        = str_replace("#", "%23", $input["dir"]);

$page["this_folder_icon"] = "openfolder";
if (file_exists($dir . $settings["htaccess"]) == true && file_exists($dir . $settings["htpasswd"]) == true && preg_match("/apache/i", $_SERVER["SERVER_SOFTWARE"]) == true)
{
    $page["this_folder_icon"] = "lockedopenfolder";
}

?>