<?php

/*
 * Invision Power File Manager
 * File lister
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.1
 * Last Modified: 20th October 2004
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

/* -----------------------------------------
                Define Constants
   ----------------------------------------- */

// Use our custom function to define constants automatically defined in later PHP versions
// These below were defined in 4.3.0 but the values they represent were introduced in 4.2.0
define_constant("UPLOAD_ERR_OK", 0);
define_constant("UPLOAD_ERR_INI_SIZE", 1);
define_constant("UPLOAD_ERR_FORM_SIZE", 2);
define_constant("UPLOAD_ERR_PARTIAL", 3);
define_constant("UPLOAD_ERR_NO_FILE", 4);

/* -----------------------------------------
               Module Functions
   ----------------------------------------- */

// Traverses a directory and returns an array of it's content
function traverse($dir, $first = "")
{
    global $settings, $permissions, $input, $page;

    // Create the function variables
    $files  = array();
    $dirs   = array();
    $total  = 0;
    $count  = 0;
    $up     = "";
    $msg    = "";
    $_files = array();
    $_dirs  = array();
    $unix   = array();

    // Reverse any user inputted \ slashes
    $dir = str_replace("\\", "/", $dir);

    // Does the directory exist?
    if (file_exists($dir) == false)
    {
        if ($dir == $permissions["root"])
        {
            // Fatal error! The root directory could not be opened!
            return array("folders" => "", "this" => "", "files" => "", "total_size" => "", "msg" => $page["no_root_directory"], "fatal" => 1, "up" => "", "nav" => "", "dir" => "", "count" => "", "back_img" => "", "pages" => 1);
        }
        else
        {
            // The directory could not be opened, but was not root so start again at root
            $returned = traverse($permissions["root"]);

            return array("folders" => $returned["folders"], "this" => $returned["this"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["no_browse_directory"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"], "pages" => $returned["pages"]);
        }
    }
    else if (is_sys_file($dir) == true)
    {
        // The directory is hidden, so start again at the root
        $returned = traverse($permissions["root"]);

        return array("folders" => $returned["folders"], "this" => $returned["this"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["hidden_directory"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"], "pages" => $returned["pages"]);
    }
    else if (check_link($dir) == true)
    {
        if (preg_match("/^" . str_replace("/", "\/", $permissions["root"]) . "/", realpath($dir) . "/") == false)
        {
            // The target of this link is outside of the users root folder, so go back to root
            $returned = traverse($permissions["root"]);

            return array("folders" => $returned["folders"], "this" => $returned["this"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["dir_is_a_disallowed_link"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"], "pages" => $returned["pages"]);
        }
        else
        {
            // The directory is a link, so resolve it
            $returned = traverse(realpath($dir) . "/", $first);

            return array("folders" => $returned["folders"], "this" => $returned["this"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["dir_is_a_link"], "up"  => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"], "pages" => $returned["pages"]);
        }
    }


    // Open the directory and then check for errors
    chdir($dir);
    if (($dp = opendir($dir)) == false)
    {
        // Directory could not be opened
        if ($dir == $permissions["root"])
        {
            // Fatal error! The root directory could not be opened!
            return array("folders" => "", "this" => $returned["this"], "files" => "", "total_size" => "&nbsp;", "msg" => $page["root_not_open"], "fatal" => 1, "up" => "", "nav" => "", "dir" => "", "count" => "", $back => "", "pages" => 1);
        }
        else
        {
            // The directory could not be opened, but was not root so start again at root
            $returned = traverse($permissions["root"], $first);

            return array("folders" => $returned["folders"], "this" => $returned["this"], "files" => $returned["files"], "total_size" => $returned["total_size"], "msg" => $page["directory_not_open"], "up" => $returned["up"], "nav" => $returned["nav"], "dir" => $returned["dir"], "count" => $returned["count"], "back_img" => $returned["back_img"], "pages" => $returned["pages"]);
        }
    }

    // Construct an array of files and directories
    while (($entry = readdir($dp)) != false)
    {
        if (is_dir($dir . $entry) == true && $entry != ".")
        {
            // Determine if the directory should be hidden
            if ($entry == "..")
            {
                // Make sure that the back link isn't hidden
                $_dirs[] = $entry;
            }
            else if ($settings["hide_dot"] == false || ($settings["hide_dot"] == true && preg_match("/^\./", $entry) == false))
            {
                if (check_link($dir . $entry) == true && preg_match("/^" . str_replace("/", "\/", $permissions["root"]) . "/", realpath($dir . $entry) . "/") == false)
                {
                    continue;
                }

                $_dirs[] = $entry;
            }
        }
        else if (is_file($dir . $entry) == true)
        {
            // Check that it is a file as we don't want sockets, pipes or block special
            if (substr($entry, strlen($entry) - strlen($settings["editing"]), strlen($entry)) == $settings["editing"])
            {
                // Check the the "lock" file should still exist, if not delete it
                if (filemtime($dir . $entry) - (time() - (int) $settings["locktimeout"]) <= 0)
                {
                    unlink($dir . $entry);
                }
            }
            else if ($settings["hide_dot"] == false || ($settings["hide_dot"] == true && preg_match("/^\./", $entry) == false))
            {
                if ($permissions["showall"] == 1 || ($permissions["showall"] == 0 && file_permission($entry) == true))
                {
                    $_files[] = $entry;
                }
            }
        }
    }
    closedir($dp);

    // Sort the arrays
    natcasesort($_files);
    natcasesort($_dirs);

    $thisdir = preg_replace("/^\//",  "", relative_path($dir, "", 1));
    $thisdir = preg_replace("/\/$/",  "", $thisdir);

    $i = 0;

    // Proceed through the directory list getting any required information about the entry
    foreach($_dirs as $entry)
    {
        if ($entry == "..")
        {
            if ($dir != $permissions["root"])
            {
                $prev = str_replace($permissions["root"],  "", $dir);
                $prev = preg_replace("/[\/]$/", "", $prev);
                $prev = preg_replace("/" . basename($dir) . "$/", "", $prev);
                $prev = preg_replace("/[\/]$/", "", $prev);

                $backimg = "<a href=\"index.php?act=list&amp;dir=" . rawurlencode($prev) . "\"><img src=\"images/icons/back.gif\" border=\"0\" alt=\"" . $page["back"] . "\" title=\"" . $page["back"] . "\" class=\"icon\" /></a>";
                $up      = "<a href=\"index.php?act=list&amp;dir=" . rawurlencode($prev) . "\">" . $page["back"] . "</a>";
            }
        }
        else
        {
            $count++;

            if ($i >= $first && $i < $first + $settings["perpage"] || $input["pg"] == "all")
            {
                $unix = unix_info($dir . $entry . "/");

                // If the folder is locked then show a locked icon
                $icon = "folder";

                $locked = false;
                if (file_exists($dir . $entry . "/" . $settings["htaccess"] ) == true && file_exists($dir . $entry . "/" . $settings["htpasswd"]) == true && preg_match("/apache/i", $_SERVER["SERVER_SOFTWARE"]) == true && $permissions["show_url"] == 1)
                {
                    $locked = true;
                }

                if (check_link($dir . $entry) == true)
                {
                    // is_link doesn't work with directories on some servers, so we get the realpath and then check our existing path
                    if ($locked == true)
                    {
                        $icon = "lockedlinkedfolder";
                    }
                    else
                    {
                        $icon = "linkedfolder";
                    }
                }
                else if ($locked == true)
                {
                    $icon = "lockedfolder";
                }

                // Build the tools & icon links
                $tools = file_tool_link($entry, $dir, 1, $thisdir);
                $icon  = icon_link($entry, $dir, $icon, 1);

                $dirs[$entry] = array(
                                "name"      => rawurlencode($entry),
                                "name_link" => "<a href=\"index.php?act=list&amp;dir=" . relative_path($entry, $dir) . "\">" . $entry . "</a>",
                                "open"      => $tools,
                                "icon"      => $icon,
                                "chmod_val" => $unix["chmod"],
                                "owner"     => $unix["owner"],
                                "group"     => $unix["group"],
                                "mdate"     => gmdate($settings["format"], timestamp(filemtime($dir . $entry)))
                                );
            }
            $i++;
        }
    }

    // Build the list of file extension for the filters
    if ($input["custom"] != "")
    {
        $show = explode(" ", $input["custom"]);
    }
    else if ($input["predef"] == "misc")
    {
        foreach($settings["filters"] as $var)
        {
            $tmp = explode(" ", $var);

            foreach ($tmp as $ext)
            {
                $show[] = $ext;
            }
        }
        $show = array_merge($show, $settings["ascii"]);
    }

    // Proceed through the file list getting any required information about the entry
    foreach($_files as $entry)
    {
        $fsize = filesize($dir . $entry);
        $total = $total + $fsize;

        if ($input["predef"] != "fold")
        {
            // Get the file information
            $unix = unix_info($dir . $entry);

            $locked = false;
            if (file_exists($dir . $entry . $settings["editing"]) == true && $settings["edit_lock"] == true)
            {
                $locked = true;
            }

            if (check_link($dir . $entry) == true)
            {
                if ($locked == true)
                {
                    $icon = get_icon($entry, BOTH);
                }
                else
                {
                    // Check if the file is a symlink
                    $icon = get_icon($entry, LINK);
                }
            }
            else if ($locked == true)
            {
                $icon = get_icon($entry, LOCK);
            }
            else
            {
                $icon = get_icon($entry);
            }

            // Build the icon link
            $tools = file_tool_link($entry, $dir);
            $icon  = icon_link($entry, $dir, $icon);

            $count++;

            if (is_array($show) == true)
            {
                $ext = ext($entry);

                if (($input["predef"] == "misc" && empty($ext) == true) || ($input["predef"] == "misc" && in_array($ext, $show) == false) || ($input["predef"] != "misc" && in_array($ext, $show) == true))
                {
                    if ($i >= $first && $i < $first + $settings["perpage"] || $input["pg"] == "all")
                    {
                        $files[$entry] = array(
                                         "name"      => rawurlencode($entry),
                                         "name_link" => file_permission($entry) == true && $permissions["download"] == 1 ? "<a href=\"index.php?act=list&amp;op=get&amp;target=" . relative_path($entry, $dir) . "\">" . $entry . "</a>" : $entry,
                                         "open"      => $tools,
                                         "icon"      => $icon,
                                         "fsize"     => format_bytes($fsize),
                                         "mdate"     => gmdate($settings["format"], timestamp(filemtime($dir . $entry))),
                                         "chmod_val" => $unix["chmod"],
                                         "owner"     => $unix["owner"],
                                         "group"     => $unix["group"]
                                         );
                    }
                    $i++;
                }
            }
            else
            {
                if ($i >= $first && $i < $first + $settings["perpage"] || $input["pg"] == "all")
                {
                    $files[$entry] = array(
                                     "name"      => rawurlencode($entry),
                                     "name_link" => file_permission($entry) == true && $permissions["download"] == 1 ? "<a href=\"index.php?act=list&amp;op=get&amp;target=" . relative_path($entry, $dir) . "\">" . $entry . "</a>" : $entry,
                                     "open"      => $tools,
                                     "icon"      => $icon,
                                     "fsize"     => format_bytes($fsize),
                                     "mdate"     => gmdate($settings["format"], timestamp(filemtime($dir . $entry))),
                                     "chmod_val" => $unix["chmod"],
                                     "owner"     => $unix["owner"],
                                     "group"     => $unix["group"]
                                     );
                }
                $i++;
            }
        }
    }

    $nav = make_navigation($dir);

    $dir = "/" . relative_path($dir, "", 1);
    // If we are running on Windows we need to reverse the slashes
    if (is_win() == true)
    {
        $dir = preg_replace("/[\/]/", "\\", $dir);
    }

    // Check if there is anything in the directory
    if (count($files) + count($dirs) == 0)
    {
        // Count the entries in the lists, if they are 0 and the total in the
        // directory is more than 0 then although no files are shown (due to filter)
        // there are some files in the directory
        if (($input["custom"] != "" || $input["predef"] != "all") && $count > 0)
        {
            // A filter is in use
            $msg = $page["no_filtered_files"];
        }
        else
        {
            // None so display a message
            $msg = $page["empty_directory"];
        }
    }
    else
    {
        // There are some, so format the size
        $total    = $total == 0 ? 0 : format_bytes($total);
    }

    return array("dir" => $dir, "this" => rawurlencode($thisdir), "nav" => $nav, "folders" => $dirs, "files" => $files, "total_size" => $total, "msg" => $msg, "up" => $up, "count" => $count, "back_img" => $backimg, "pages" => ceil($i/$settings["perpage"]));
}

// Creates the toolbar
function generate_toolbar()
{
    global $permissions, $page, $settings;

    $html = "";

    $html .= $permissions["upload"]    == 1 ? "<a href=\"javascript:open_popup('upload');\"><img src=\"images/upload.gif\" border=\"0\" alt=\"" . $page["upload"] . "\" title=\"" . $page["upload"] . "\" /> " . $page["upload"] . "</a>&nbsp;" : "";
    $html .= $permissions["newfile"]   == 1 ? "<a href=\"javascript:open_popup('mkfile');\"><img src=\"images/new.gif\" border=\"0\" alt=\"" . $page["new_file"] . "\" title=\"" . $page["new_file"] . "\" /> " . $page["new_file"] . "</a>&nbsp;" : "";
    $html .= $permissions["newfolder"] == 1 ? "<a href=\"javascript:open_popup('mkdir');\"><img src=\"images/newdir.gif\" border=\"0\" alt=\"" . $page["new_directory"] . "\" title=\"" . $page["new_directory"] . "\" /> " . $page["new_directory"] . "</a>&nbsp;" : "";
    $html .= $permissions["delete"]    == 1 ? "<a href=\"javascript:del();\"><img src=\"images/delete.gif\" border=\"0\" alt=\"" . $page["delete"] . "\" title=\"" . $page["delete"] . "\" /> " . $page["delete"] . "</a>&nbsp;" : "";
    $html .= $permissions["rename"]    == 1 ? "<a href=\"javascript:rename();\"><img src=\"images/rename.gif\" border=\"0\" alt=\"" . $page["rename"] . "\" title=\"" . $page["rename"] . "\" /> " . $page["rename"] . "</a>&nbsp;" : "";

    // Need a bit of a more complex check here, need to check for the permission and the extension
    //  && (extension_loaded("zlib") == true || extension_loaded("zip") == true) ??? Do we need to? as do we need it for tar
    if ($permissions["archive"] == 1)
    {
        $html .= "<a href=\"javascript:archive()\"><img src=\"images/archive.gif\" border=\"0\" alt=\"" . $page["archive"] . "\" title=\"" . $page["archive"] . "\" /> " . $page["archive"] . "</a>&nbsp;";
    }

    $html .= $permissions["move"]    == 1 ? "<a href=\"javascript:move();\"><img src=\"images/move.gif\" border=\"0\" alt=\"" . $page["move"] . "\" title=\"" . $page["move"] . "\" /> " . $page["move"] . "</a>&nbsp;" : "";
    $html .= $permissions["copy"]    == 1 ? "<a href=\"javascript:copy();\"><img src=\"images/copy.gif\" border=\"0\" alt=\"" . $page["copy"] . "\" title=\"" . $page["copy"] . "\" /> " . $page["copy"] . "</a>&nbsp;" : "";

    // Needs a bit more complex check here, not just permssion but also OS
    if (is_win() == false)
    {
        $html .= $permissions["chmod"] == 1 ? "<a href=\"javascript:chmod();\"><img src=\"images/chmod.gif\" border=\"0\" alt=\"" . $page["chmod"] . "\" title=\"" . $page["chmod"] . "\" /> " . $page["chmod"] . "</a>&nbsp;" : "";
        $html .= $permissions["symlink"] == 1 ? "<a href=\"javascript:link();\"><img src=\"images/link.gif\" border=\"0\" alt=\"" . $page["link"] . "\" title=\"" . $page["link"] . "\" /> " . $page["link"] . "</a>&nbsp;" : "";
    }
    $data["filetools"] = $html;

    if (empty($html) == true)
    {
        return "";
    }

    return static_template($settings["script_dir"] . "skin/filetools.tpl", $data);
}

// Remove an entire directory
function remove_dir($dir)
{
    global $debug;

    $debug["noperm"] = false;

    // Create arrays to store files and directories
    $_files      = array();
    $_subdirs    = array();

    $myerr = false; // This one needs to be reset every time the function is called

    if (($dp = opendir($dir)) == false)
    {
        // The directory could not be opened
        return false;
    }

    // Loop through all directory entries, and construct two temporary arrays containing files and sub directories
    while($entry = readdir($dp))
    {
        if (is_dir($dir . $entry) && $entry !=  ".." && $entry !=  ".")
        {
            $_subdirs[] = $entry;
        }
        else if ($entry !=  ".." && $entry !=  ".")
        {
            $_files[] = $entry;
        }
    }

    // Close directory
    closedir($dp);

    // Delete all files in the curent directory return false and halt if a file cannot be removed
    for($i=0; $i<count($_files); $i++)
    {
        if (file_permission($_files[$i]) == true && is_sys_file($entry) == false)
        {
            $chmod = chmod_value($dir . $_files[$i]);
            chmod($dir . $_files[$i], 0777);

            if (unlink($dir . $_files[$i]) == false)
            {
                chmod($dir . $_files[$i], octdec((int) $chmod));
                return false;
            }
        }
        else
        {
            $myerr           = true;
            $debug["noperm"] = true;
        }
    }

    // Empty sub directories and then remove the directory
    for($i=0; $i<count($_subdirs); $i++)
    {
        if (is_sys_file($entry, true) == false)
        {
            $chmod = chmod_value($dir . $_subdirs[$i] . "/");
            chmod($dir . $_subdirs[$i] . "/", 0777);

            if (remove_dir($dir . $_subdirs[$i] . "/") == false)
            {
                chmod($dir . $_subdirs[$i] . "/", octdec((int) $chmod));
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    if ($myerr == false)
    {
        $chmod = chmod_value($dir);
        if (rmdir($dir) == false)
        {
            chmod($dir, octdec((int) $chmod));
            return false;
        }
    }
    else
    {
        return false;
    }
    return true;
}

// CHMOD an entire directory
function chmod_dir($dir, $chmod)
{
    global $debug;

    $debug["noperm"] = false;

    // Create arrays to store files and directories
    $_files      = array();
    $_subdirs    = array();

    $myerr = false; // This one needs to be reset every time the function is called

    if (($dp = opendir($dir)) == false)
    {
        // The directory could not be opened
        return false;
    }

    // Loop through all directory entries, and construct two temporary arrays containing files and sub directories
    while($entry = readdir($dp))
    {
        if (is_dir($dir . $entry) && $entry !=  ".." && $entry !=  ".")
        {
            $_subdirs[] = $entry;
        }
        else if ($entry !=  ".." && $entry !=  ".")
        {
            $_files[] = $entry;
        }
    }

    // Close directory
    closedir($dp);

    // Delete all files in the curent directory return false and halt if a file cannot be removed
    for($i=0; $i<count($_files); $i++)
    {
        if (file_permission($_files[$i]) == true && is_sys_file($entry) == false)
        {
            if (chmod($dir . $_files[$i], octdec((int) $chmod)) == false)
            {
                return false;
            }
        }
        else
        {
            $myerr           = true;
            $debug["noperm"] = true;
        }
    }

    // Empty sub directories and then remove the directory
    for($i=0; $i<count($_subdirs); $i++)
    {
        if (is_sys_file($entry, true) == false)
        {
            if (chmod_dir($dir . $_subdirs[$i] . "/", $chmod) == false)
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    if ($myerr == false)
    {
        if (chmod($dir, octdec((int) $chmod)) == false)
        {
            return false;
        }
    }
    else
    {
        return false;
    }
    return true;
}

// A function to transfer a file
function transfer_file($source, $destination, $type = "copy")
{
    return $type == "copy" ? copy($source, $destination) : rename($source, $destination);
}

// A function to transfer a directory
function transfer_dir($source, $destination, $type = "copy")
{
    global $debug, $settings;

    $debug["noperm"] = false;

    // Create arrays to store files and directories
    $_files      = array();
    $_subdirs    = array();

    $myerr = false; // This one needs to be reset every time the function is called

    if (($dp = opendir($source)) == false)
    {
        // The directory could not be opened
        return false;
    }

    // Loop through all directory entries, and construct two temporary arrays containing files and sub directories
    while($entry = readdir($dp))
    {
        if (is_dir($source . $entry) && $entry !=  ".." && $entry !=  ".")
        {
            $_subdirs[] = $entry;
        }
        else if ($entry !=  ".." && $entry !=  ".")
        {
            $_files[] = $entry;
        }
    }

    // Close directory
    closedir($dp);

    $chmod = chmod_value($source);

    // Next we need to create the destination directory
    if (mkdir($destination, octdec($chmod)) == false)
    {
        return false;
    }

    $type = $type == "copy" ? "copy" : "rename";

    // Transfer all files within the current directory
    for($i=0; $i<count($_files); $i++)
    {
        // Remove this for now  && is_sys_file($source . $_files[$i]) == false, do we really
        // not want the users notes, bookmarks and access files to not be transferred?

        if (file_permission($_files[$i]) == true)
        {
            $chmod = chmod_value($source . $_files[$i]);
            chmod($source . $_files[$i], 0777);

            if ($type($source . $_files[$i], $destination . $_files[$i]) == false)
            {
                chmod($source . $_files[$i], octdec((int) $chmod));
                return false;
            }
        }
        else
        {
            $myerr           = true;
            $debug["noperm"] = true;
        }
    }

    // Empty sub directories and then remove the directory
    for($i=0; $i<count($_subdirs); $i++)
    {
        if (is_sys_file($source . $_subdirs[$i], true) == false)
        {
            $chmod = chmod_value($source . $_subdirs[$i] . "/");
            chmod($source . $_subdirs[$i] . "/", 0777);

            if (transfer_dir($source . $_subdirs[$i] . "/", $destination . $_subdirs[$i] . "/", $type) == false)
            {
                chmod($source . $_subdirs[$i] . "/", octdec((int) $chmod));
                return false;
            }
        }
        else
        {
            $myerr           = true;
            $debug["noperm"] = true;
        }
    }

    if ($myerr == false && $type != "copy")
    {
        $chmod = chmod_value($source);
        if (remove_dir($source) == false)
        {
            chmod($source, octdec((int) $chmod));
            return false;
        }
    }
    else
    {
        return false;
    }

    return true;
}

// Generates the page links
function page_links($total, $current)
{
    global $page, $info, $input;

    if ($total < 2)
    {
        return "&nbsp;";
    }

    $html = "<span class=\"pagelink\"><a href=\"javascript:pageJump(" . $total . ", " . $current . ", 'index.php?act=list&amp;dir=" . relative_path("", $info["this"]) . "&amp;pg=');\">" . $total . " " . $page["pages"] . "</a> (<a href=\"index.php?act=list&amp;dir=" . relative_path("", $info["this"]) . "&amp;pg=all\">" . $page["view_all"] . "</a>)</span>&nbsp;&nbsp;";

    if ($current > 1)
    {
        $html .= "<span class=\"pagelink\">";
        if ($current > 2)
        {
            $html .= "<a href=\"index.php?act=list&amp;dir=" . relative_path("", $info["this"], 1) . "&amp;pg=1\">&laquo;</a>&nbsp;";
        }
        $html .= "<a href=\"index.php?act=list&amp;dir=" . relative_path("", $info["this"], 1) . "&amp;pg=" . ($current - 1) . "\">&lt;</a></span>&nbsp;";
    }

    for($i = 1; $i <= $total; ++$i )
    {
        if ($i == $current && $input["pg"] != "all")
        {
            $html .= "<span class=\"pagecurrent\">" . $i . "</span>&nbsp;";
        }
        else
        {
            // Below the start limit so continue
            if ($i < ($current - 2))
            {
                continue;
            }

            // We're above the end limit so break
            if ($i > ($current + 2))
            {
                break;
            }

            $html .= "<span class=\"pagelink\"><a href=\"index.php?act=list&amp;dir=" . relative_path("", $info["this"], 1) . "&amp;pg=" . $i . "\">" . $i . "</a></span>&nbsp;";
        }
    }


    if ($current < $total)
    {
        $html .= "<span class=\"pagelink\"><a href=\"index.php?act=list&amp;dir=" . relative_path("", $info["this"], 1) . "&amp;pg=" . ($current + 1) . "\">&gt;</a>";
        if ($current < $total - 1)
        {
            $html .= "&nbsp;<a href=\"index.php?act=list&amp;dir=" . relative_path("", $info["this"], 1) . "&amp;pg=" . $total . "\">&raquo;</a>";
        }
        $html .= "</span>";
    }

    return $html;
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */

// Build a list for the ASCII filter, no need to define a seperate filter, just use the ASCII files array
$settings["filters"]["txt"] = "";
foreach ($settings["ascii"] as $var)
{
     $settings["filters"]["txt"] .= $var . " ";
}

// Set a filter cookie if needed
if ($input["filter"] == 1)
{
    if (isset($_GET["predef"]) == true)
    {
        if ($input["predef"] == "fold")
        {
            $data = array("fold", "fold");
        }
        else if ($input["predef"] == "misc")
        {
            $data = array("misc", "misc");
        }
        else if ($input["predef"] != "all")
        {
            $data = array($input["predef"], $settings["filters"][$input["predef"]]);
            $input["custom"] = $settings["filters"][$input["predef"]];
        }
    }
    else
    {
        if ($input["predef"] == "fold")
        {
            $data = array("fold", "fold");
        }
        else if ($input["predef"] == "misc")
        {
            $data = array("misc", "misc");
        }
        else
        {
            $data = array($input["predef"], $input["custom"]);
        }
    }

    // Set the cookie that is needed
    if ($input["predef"] == "all")
    {
        setcookie($settings["filter_cookie"]); // Remove the cookie
    }
    else if ($input["once"] != 1)
    {
        setcookie($settings["filter_cookie"], serialize($data), (int) $settings["cookie_time"]);
    }
}
else
{
    $input["custom"] = unserialize(stripslashes($_COOKIE[$settings["filter_cookie"]]));

    if ($input["custom"][0] == "fold")
    {
        $input["predef"] = "fold";
        $input["custom"] = "";
    }
    else if ($input["custom"][0] == "misc")
    {
        $input["predef"] = "misc";
        $input["custom"] = "";
    }
    else if ($input["custom"][0] == "")
    {
        $input["predef"] = "all";
        $input["custom"] = "";
    }
    else
    {
        $input["predef"] = $input["custom"][0];
        $input["custom"] = $input["custom"][1];
    }
}

// Clean the path and then display the content of the directory
$dir            = clean_path($input["dir"]);
$jumpdir        = clean_path($input["jumpdir"]);
$bookmark       = clean_path($input["bookmark"]);

$debug["errs"]  = 0;
$debug["msg"]   = "";

if (empty($jumpdir) == false)
{
    $dir = $permissions["root"] . $jumpdir . "/";
}
else if (empty($bookmark) == false)
{
    $bookmark = $bookmark . "/";
    $dir      = $permissions["root"] . $bookmark;
}
else if (empty($dir) == true)
{
    $dir = $permissions["root"];
}
else
{
    $dir = $permissions["root"] . $dir . "/";
}

// Now we run any commands which have been called
switch ($input["op"])
{
    case "delmsg":
        /* ----------------------------
             Remove the admin message
           ---------------------------- */
        if ($permissions["adminmsg"] == 0)
        {
            $debug["msg"] = $page["no_msg_permission"];
        }
        else
        {
            if (file_exists($permissions["root"] . $settings["msg"]) == true)
            {
                chmod($permissions["root"] . $settings["msg"], 0777);

                if (filemtime($permissions["root"] . $settings["msg"]) - (time() - (int) $settings["msgtimeout"]) > 0)
                {
                    $debug["msg"] = $page["timeout_delete_adminmsg"];
                }
                else if (unlink($permissions["root"] . $settings["msg"]) == false)
                {
                    $debug["msg"] = $page["error_delete_adminmsg"];
                }
            }
            else
            {
                $debug["msg"] = $page["no_admin_msg"];
            }
        }
    break;
    case "bookmark":
        /* ----------------------------
              Bookmark the directory
           ---------------------------- */
        $input["bookname"] = trim(str_replace("|", "", $input["bookname"]));
        $input["dir"]      = trim(str_replace("|", "", clean_path($input["dir"])));
        $input["target"]   = trim(str_replace("|", "", clean_path($input["target"])));

        if (empty($input["target"]) == true || empty($input["bookname"]) == true)
        {
            $debug["msg"] = $page["bookmark_invalid"];
        }

        else if (($fp = fopen($permissions["root"] . $settings["bookmarks"], "a")) == true)
        {
            file_lock($fp);
            fwrite($fp, trim($input["bookname"]) . "|" . preg_replace("/^\//", "", $input["dir"] . "/" . $input["target"]) . "\n");
            file_lock($fp, LOCK_UN);
            fclose($fp);

            $debug["msg"] = $page["bookmark_created"];
        }
        else
        {
            $debug["msg"] = $page["bookmark_not_created"];
        }

    break;
    case "notes":
        /* ----------------------------
             Edit the directory notes
           ---------------------------- */
        if ($permissions["notes"] == 0)
        {
            $debug["msg"] = $page["no_notes_permission"];
        }
        else
        {
            if (($fp = fopen($dir . $settings["notes"], "w")) == true)
            {
                file_lock($fp);
                $notes = fwrite($fp, strip_tags(trim($input["notes"])));
                file_lock($fp, LOCK_UN);
                fclose($fp);

                if (filesize($dir . $settings["notes"]) == 0)
                {
                    unlink($dir . $settings["notes"]);
                }

                $debug["msg"] = $info["notes_updated"];
            }
            else
            {
                $debug["msg"] = $page["error_updating_notes"];
            }
        }
    break;
    case "get":
        /* ----------------------------
                 Download a file
           ---------------------------- */
        $input["target"] = clean_path($input["target"]);

        if ($permissions["download"] == 0)
        {
            $debug["msg"] = preg_replace("/<%FILE%>/i", basename($input["target"]), $page["no_permission_to_get"]);
        }
        else
        {
            if (file_exists($permissions["root"] . $input["target"]) == true)
            {
                if (file_permission($input["target"]) == false || is_sys_file($permissions["root"] . $input["target"]) == true)
                {
                    $debug["msg"] = preg_replace("/<%FILE%>/i", basename($input["target"]), $page["no_permission_to_get"]);
                }
                else
                {
                    $browser = web_browser();
                    $type    = ($browser == "ie" || $browser == "opera") ? "application/octetstream" : "application/octet-stream";

                    // Set the required HTTP headers
                    header("Content-Type: " . $type, true);
                    header("Content-Transfer-Encoding: binary");
                    header("Content-Length: " . filesize($permissions["root"] . $input["target"]));
                    header("Content-Disposition: attachment; filename=" . str_replace(" ", "_", basename($input["target"]))); // Doesn't like spaces in file names

                    readfile($permissions["root"] . $input["target"]);

                    exit; // Exit after printing out the file content
                }
            }
            else
            {
                $debug["msg"] = $page["no_file_to_get"];
            }
        }
    break;
    case "del":
        /* ----------------------------
               Delete files/folders
           ---------------------------- */
        if ($permissions["delete"] == 0)
        {
            $debug["msg"] = $page["no_delete_permission"];
        }
        else
        {
            // Firstly lets get the list of directories and files to delete
            $files = explode("|", $input["files"]);
            $dirs  = explode("|", $input["directories"]);

            unset($files[count($files) - 1]);
            unset($dirs[count($dirs) - 1]);

            // Lets try and delete the files
            foreach($files as $entry)
            {
                if (empty($entry) == false)
                {
                    $entry = clean_path($entry);

                    if (file_exists($dir . $entry) == false)
                    {
                        $nofile[] = $entry;
                        $debug["errs"]++;
                    }
                    else if (file_permission($entry) == false || is_sys_file($entry) == true)
                    {
                       $noext[] = $entry;
                       $debug["errs"]++;
                    }
                    else
                    {
                        $chmod = chmod_value($dir . $entry);
                        chmod($dir . $entry, 0777);
                        if (unlink($dir . $entry) == false)
                        {
                            $nodelfile[] = $entry;
                            $debug["errs"]++;

                            chmod($dir . $entry, octdec((int) $chmod));
                        }
                    }
                }
            }

            // Now lets try and delete the directory
            foreach($dirs as $entry)
            {
                if (empty($entry) == false)
                {
                    $entry = clean_path($entry);

                    if (file_exists($dir . $entry . "/") == false)
                    {
                        $nodir[] = $entry;
                        $debug["errs"]++;
                    }
                    else if (is_sys_file($entry, true) == true)
                    {
                        $nodeldir[] = $entry;
                        $debug["errs"]++;
                    }
                    else
                    {
                        $chmod = chmod_value($dir . $entry . "/");

                        if (check_link($dir . $entry . "/") == true)
                        {
                            // Delete the link, not the entire directory
                            if (unlink($dir . $entry) == false)
                            {
                                $nodeldir[] = $entry;
                                $debug["errs"]++;

                                chmod($dir . $entry . "/", octdec((int) $chmod));
                            }
                        }
                        else
                        {
                            if (remove_dir($dir . $entry . "/") == false)
                            {
                                $nodeldir[] = $entry;
                                $debug["errs"]++;

                                chmod($dir . $entry . "/", octdec((int) $chmod));
                            }
                        }
                    }
                }
            }

            // Populate the "no such file" message
            $debug["msg"] .= create_message($nofile, $page["no_such_file"], $page["no_such_files"]);

            // Populate the "no such directory" message
            $debug["msg"] .= create_message($nodir, $page["no_such_directory"], $page["no_such_directories"]);

            // Populate the "no permission" message
            $debug["msg"] .= create_message($noext, $page["delete_filetype_permission"], $page["delete_filetypes_permission"]);

            // Populate the delete file error message
            $debug["msg"] .= create_message($nodelfile, $page["error_delete_file"], $page["error_delete_files"]);

            if ($debug["noperm"] == true)
            {
                // Populate the delete directory error message when the user doesn't have file access
                $debug["msg"] .= create_message($nodeldir, $page["delete_filetype_permission_dir"], $page["delete_filetypes_permission_dir"]);
            }
            else
            {
                // Populate the delete directory error message
                $debug["msg"] .= create_message($nodeldir, $page["error_delete_directory"], $page["error_delete_directories"]);
            }

            if ($debug["errs"] == 0)
            {
                $debug["msg"] = $page["all_deleted"];
            }
            else if ($debug["errs"] < (count($files) + count($dirs)))
            {
                $debug["msg"] .= $page["part_deleted"];
            }
        }
    break;
    case "mkdir":
        /* ----------------------------
                  Create folder
           ---------------------------- */
        if ($permissions["newfolder"] == 0)
        {
            $debug["msg"] = $page["no_mkdir_permission"];
        }
        else
        {
            $input["newdir"] = clean_path($input["newdir"]);

            if (is_invalid_name($input["newdir"]) == true || empty($input["newdir"]) == true)
            {
                $debug["msg"] = $page["newdir_invalid"];
            }
            else if (file_exists($dir . $input["newdir"]) == true)
            {
                $debug["msg"] = $page["newdir_exists"];
            }
            else if (directory_size($permissions["root"]) >= $permissions["space"])
            {
                $debug["msg"] = $page["low_space"];
            }
            else if (mkdir($dir . $input["newdir"] . "/", octdec($settings["default_chmod"])) == false)
            {
                $debug["msg"] = $page["error_creating_newdir"];
            }
            else
            {
                $debug["msg"] = $page["newdir_created"];
            }
        }
    break;
    case "rename":
        /* ----------------------------
                     Rename
           ---------------------------- */
        if ($permissions["rename"] == 0)
        {
            $debug["msg"] = $page["no_rename_permission"];
        }
        else
        {
            $i = 0;

            foreach ($input["oldname"] as $key => $var)
            {
                $i++;

                $oldname = clean_path($input["oldname"][$key]);
                $newname = clean_path($input["newname"][$key]);

                if (is_dir($dir . $oldname) == true)
                {
                    $old = $dir . $oldname . "/";
                    $new = $dir . $newname . "/";
                }
                else
                {
                    $old = $dir . $oldname;
                    $new = $dir . $newname;
                }

                if ($new == $old)
                {
                    $i--;
                    continue;
                }
                else if (is_invalid_name($newname) == true || empty($newname) == true)
                {
                    $invalid[] = $oldname;
                    $debug["errs"]++;
                }
                else if (file_exists($old) == false)
                {
                    $nosuch[] = $oldname;
                    $debug["errs"]++;
                }
                else if (file_exists($new) == true)
                {
                    $exists[] = $oldname;
                    $debug["errs"]++;
                }
                else if (file_permission($old) == false || is_sys_file($old) == true)
                {
                    $old_filetypes[] = $oldname;
                    $debug["errs"]++;
                }
                else if (file_permission($new) == false || is_sys_file($new) == true)
                {
                    $new_filetypes[] = $oldname;
                    $debug["errs"]++;
                }
                else if (rename($old, $new) == false)
                {
                    $rename[] = $oldname;
                    $debug["errs"]++;
                }
                else
                {
                    $renamed[] = $oldname;
                }
            }

            // Populate the invalid name message
            $debug["msg"] .= create_message($invalid, $page["rename_invalid"], $page["renames_invalid"]);

            // Populate the missing source message
            $debug["msg"] .= create_message($nosuch, $page["rename_no_such_file"], $page["rename_no_such_files"]);

            // Populate the target exists message
            $debug["msg"] .= create_message($exists, $page["rename_target_exist"], $page["rename_targets_exist"]);

            // Populate the source filetype message
            $debug["msg"] .= create_message($old_filetypes, $page["rename_filetype_permission"], $page["rename_filetypes_permission"]);

            // Populate the target filetype message
            $debug["msg"] .= create_message($new_filetypes, $page["rename_to_filetype_permission"], $page["rename_to_filetypes_permission"]);

            // Populate the rename error message
            $debug["msg"] .= create_message($rename, $page["rename_error"], $page["rename_errors"]);

            if ($debug["errs"] == 0)
            {
                $debug["msg"] = $page["all_renamed"];
            }
            else if ($debug["errs"] < $i)
            {
                $debug["msg"] .= $page["partial_rename"];
            }

        }
    break;
    case "upload":
        /* ----------------------------
                   File Uploads
           ---------------------------- */
        if ($permissions["upload"] == 0)
        {
            $debug["msg"] = $page["no_upload_permission"];
        }
        else
        {
            $remote = $dir . clean_path($input["remote"]);

            if ($_FILES["local"]["error"] != UPLOAD_ERR_OK)
            {
                if ($_FILES["local"]["error"] == UPLOAD_ERR_INI_SIZE || $_FILES["local"]["error"] == UPLOAD_ERR_FORM_SIZE)
                {
                    $debug["msg"] = preg_replace("/{LIMIT}/i", get_cfg_var("upload_max_filesize"), preg_replace("/<%FILE%>/i", $_FILES["local"]["name"], $page["upload_too_large"]));
                }
                else if ($_FILES["local"]["error"] == UPLOAD_ERR_PARTIAL)
                {
                    // The file wasn't completed
                    $debug["msg"] = preg_replace("/<%FILE%>/i", $_FILES["local"]["name"], $page["upload_partial"]);
                }
                else if ($_FILES["local"]["error"] == UPLOAD_ERR_NO_FILE)
                {
                    // No file was selected
                    $debug["msg"] = $page["upload_none"];
                }
            }
            else if (is_invalid_name($input["remote"]) == true || empty($input["remote"]) == true)
            {
                $debug["msg"] = $page["name_invalid"];
            }
            else if (file_permission($remote) == false || is_sys_file($remote) == true)
            {
                $debug["msg"] = preg_replace("/<%FILE%>/i", basename($remote), $page["upload_filetype_permission"]);
            }
            else if (file_permission($_FILES["local"]["name"]) == false || is_sys_file($_FILES["local"]["name"]) == true)
            {
                $debug["msg"] = preg_replace("/<%FILE%>/i", $_FILES["local"]["name"], $page["upload_filetype_permission"]);
            }
            else if (file_exists($remote) == true && $permissions["overwrite"] == 0)
            {
                $debug["msg"] = $page["no_overwrite_permission"];
            }
            else if (directory_size($permissions["root"]) + $_FILES["local"]["size"] >= $permissions["space"])
            {
                $debug["msg"] = $page["low_space"];
            }
            else if (move_uploaded_file($_FILES["local"]["tmp_name"], $remote) == false)
            {
                // File can not be copied, show message
                $debug["msg"] = preg_replace("/<%FILE%>/i", basename($remote), $page["upload_error_copy"]);
            }
            else
            {
                chmod($remote, octdec($settings["default_chmod"]));

                // File has been uploaded
                $debug["msg"] = $page["uploaded"];
            }

        }
    break;
    case "uploads":
        /* ----------------------------
                 Multi file Uploads
           ---------------------------- */
        if ($permissions["upload"] == 0)
        {
            $debug["msg"] = $page["no_upload_permission"];
        }
        else
        {
            $total = $settings["max_uploads"];

            for ($i=0; $i<$settings["max_uploads"]; $i++)
            {
                $remote = $dir . $_FILES["local"]["name"][$i];

                if ($_FILES["local"]["error"][$i] != UPLOAD_ERR_OK)
                {
                    if ($_FILES["local"]["error"][$i] == UPLOAD_ERR_INI_SIZE || $_FILES["local"]["error"][$i] == UPLOAD_ERR_FORM_SIZE)
                    {
                        $tolarge[] = $_FILES["local"]["name"][$i];
                        $debug["errs"]++;
                    }
                    else if ($_FILES["local"]["error"][$i] == UPLOAD_ERR_PARTIAL)
                    {
                        // The file wasn't completed
                        $partial[] = $_FILES["local"]["name"][$i];
                        $debug["errs"]++;
                    }
                    elseif ($_FILES["local"]["error"][$i] != UPLOAD_ERR_NO_FILE)
                    {
                        // If there is no error increase the file counter
                        $total--;
                    }
                }
                else if (file_permission($_FILES["local"]["name"][$i]) == false || is_sys_file($_FILES["local"]["name"][$i]) == true)
                {
                    $filetypes[] = $_FILES["local"]["name"][$i];
                    $debug["errs"]++;
                }
                else if (file_exists($remote) == true && $permissions["overwrite"] == 0)
                {
                    $overwrite[] = $_FILES["local"]["name"][$i];
                    $debug["errs"]++;
                }
                else if (directory_size($permissions["root"]) + $_FILES["local"]["size"][$i] >= $permissions["space"])
                {
                    $lowspace[] = $_FILES["local"]["name"][$i];
                    $debug["errs"]++;
                }
                else if (move_uploaded_file($_FILES["local"]["tmp_name"][$i], $remote) == false)
                {
                    // File can not be copied, show message
                    $copy[] = $_FILES["local"]["name"][$i];
                    $debug["errs"]++;
                }
                else
                {
                    chmod($remote, octdec($settings["default_chmod"]));

                    // File has been uploaded
                    $uploaded[] = $_FILES["local"]["name"][$i];
                }
            }

            $page["upload_too_large"] = preg_replace("/{LIMIT}/i", get_cfg_var("upload_max_filesize"), $page["upload_too_large"]);
            $page["uploads_too_large"] = preg_replace("/{LIMIT}/i", get_cfg_var("upload_max_filesize"), $page["uploads_too_large"]);

            // Populate the "file to large" message
            $debug["msg"] .= create_message($tolarge, $page["upload_too_large"], $page["uploads_too_large"]);

            // Populate the "partial upload" message
            $debug["msg"] .= create_message($partial, $page["upload_partial"], $page["uploads_partial"]);

            // Populate the "filetype" message
            $debug["msg"] .= create_message($filetypes, $page["upload_filetype_permission"], $page["upload_filetypes_permission"]);

            // Populate the "overwrite" message
            $debug["msg"] .= create_message($overwrite, $page["upload_overwrite"], $page["uploads_overwrite"]);

            // Populate the "low space" message
            $debug["msg"] .= create_message($lowspace, $page["upload_low_space"], $page["uploads_low_space"]);

            // Populate the "copy" message
            $debug["msg"] .= create_message($copy, $page["upload_error_copy"], $page["upload_errors_copy"]);

            // No files were selected
            if ($total == 0)
            {
                $debug["msg"] = $page["upload_none"];
            }

            if ($debug["errs"] == 0)
            {
                $debug["msg"] .= $page["all_uploaded"];
            }
            else if ($debug["errs"] > 0 && count($uploaded) > 0)
            {
                $debug["msg"] .= $page["part_uploaded"];
            }
        }
    break;
    case "move":
        /* ----------------------------
                     Move/Copy
           ---------------------------- */
        $input["type"] == "move" ? "move" : "copy";

        if ($permissions[$input["type"]] == 0)
        {
            // Little bit more complex than the normal message
            $debug["msg"] = $page["no_{$input['type']}_permission"];
        }
        else
        {
            $i = 0;

            $input["files"]  = explode("|", $input["files"]);
            $input["dirs"]   = explode("|", $input["dirs"]);

            $input["target"] = clean_path($input["target"]);

            if (empty($input["target"]) == true)
            {
                $input["target"] = $permissions["root"];
            }
            else
            {
                $input["target"] = $permissions["root"] . $input["target"] . "/";
            }

            foreach ($input["files"] as $key => $var)
            {
                $source = $dir . trim($var);
                $target = $input["target"] . trim($var);

                if (empty($var) == false)
                {
                    $i++;

                    if (file_permission($var) == false || is_sys_file($source) == true || is_sys_file($target) == true)
                    {
                        $filetypes[] = $var;
                        $debug["errs"]++;
                    }
                    else if (file_exists($source) == false)
                    {
                        $nofile[] = $var;
                        $debug["errs"]++;
                    }
                    else if (file_exists($target) == true)
                    {
                        if ($permissions["overwrite"] == 0)
                        {
                            $overwrite[] = $var;
                            $debug["errs"]++;
                        }
                        else if (unlink($target) == false)
                        {
                            $transfer[] = $var;
                            $debug["errs"]++;
                        }
                    }
                    else
                    {
                        // Probably best to do all the checking outside our function and just have the function do the actual transfer?
                        if (transfer_file($source, $target, $input["type"]) == false)
                        {
                            $transfer[] = $var;
                            $debug["errs"]++;
                        }
                    }
                }
                else
                {
                    // Empty so remove it
                    unset($input["files"][$key]);
                }
            }

            // Populate the "filetype" message
            $debug["msg"] = create_message($filetypes, $page["{$input['type']}_filetype_permission"], $page["{$input['type']}_filetypes_permission"]);

            // Populate the "missing file" message
            $debug["msg"] .= create_message($nofile, $page["copy_no_such_file"], $page["copy_no_such_files"]);

            // Populate the "overwrite" message
            $debug["msg"] .= create_message($overwrite, $page["copy_overwrite"], $page["copy_overwrites"]);

            // Populate the transfer error message
            $debug["msg"] .= create_message($tranfer, $page["{$input['type']}_error"], $page["{$input['type']}_errors"]);

            foreach ($input["dirs"] as $key => $var)
            {
                $source = $dir . trim($var) . "/";
                $target = $input["target"] . trim($var) . "/";

                if (preg_match("/^" . str_replace("/", "\/", $source) . "/", $input["target"]) == true)
                {
                    $path[] = $var;
                    $debug["errs"]++;
                }
                else if (empty($var) == false)
                {
                    $i++;

                    if (file_exists($source) == false)
                    {
                        $sources[] = $var;
                        $debug["errs"]++;
                    }
                    else if (file_exists($target) == true)
                    {
                        $targets[] = $var;
                        $debug["errs"]++;
                    }
                    else
                    {
                        // Transfer the actual directory, this is a little bit more complex than transferring a file
                        if (transfer_dir($source, $target, $input["type"]) == false)
                        {
                            $transdir[] = $var;
                            $debug["errs"]++;
                        }
                    }
                }
            }

            $debug["msg"] = "";
            if (is_array($path) == true)
            {
                $debug["msg"] = $page["{$input['type']}_path_error"] . "<br />";
            }

            // Populate the missing source directory error
            $debug["msg"] .= create_message($sources, $page["source_dir_error"], $page["source_dir_errors"]);

            // Populate the existing target directory error
            $debug["msg"] .= create_message($targets, $page["target_dir_error"], $page["target_dir_errors"]);

            if ($debug["noperm"] == true)
            {
                // Populate the transfer directory error message when the user doesn't have file access
                $debug["msg"] .= create_message($transdir, $page["{$input['type']}_filetype_permission_dir"], $page["{$input['type']}_filetypes_permission_dir"]);
            }
            else
            {
                // Populate the tranfer directory error message
                $debug["msg"] .= create_message($transdir, $page["error_{$input['type']}_directory"], $page["error_{$input['type']}_directories"]);
            }

            if ($debug["errs"] == 0)
            {
                $debug["msg"] = $page["{$input['type']}_all_done"];
            }
            else if ($debug["errs"] < $i)
            {
                $debug["msg"] .= $page["partial_{$input['type']}"];
            }
        }
    break;
    case "chmod":
        /* ----------------------------
                    Mass CHMOD
           ---------------------------- */
        if ($permissions["chmod"] == 0)
        {
            // Check the user name permission
            $debug["msg"] = $page["no_chmod_permission"];
        }
        else
        {
            // Firstly lets get the list of directories and files to delete
            $files = explode("|", $input["files"]);
            $dirs  = explode("|", $input["dirs"]);

            unset($files[count($files) - 1]);
            unset($dirs[count($dirs) - 1]);

            // Lets try and CHMOD the files
            foreach($files as $entry)
            {
                if (empty($entry) == false)
                {
                    $entry = clean_path($entry);

                    if (file_exists($dir . $entry) == false)
                    {
                        $nofile[] = $entry;
                        $debug["errs"]++;
                    }
                    else if (file_permission($entry) == false || is_sys_file($entry) == true)
                    {
                       $noext[] = $entry;
                       $debug["errs"]++;
                    }
                    else if (chmod($dir . $entry, octdec((int) $input["decimal"])) == false)
                    {
                        $nochmodfile[] = $entry;
                        $debug["errs"]++;
                    }
                }
            }

            // Now lets try and CHMOD the directories
            foreach($dirs as $entry)
            {
                if (empty($entry) == false)
                {
                    $entry = clean_path($entry);

                    if (file_exists($dir . $entry . "/") == false)
                    {
                        $nodir[] = $entry;
                        $debug["errs"]++;
                    }
                    else if (is_sys_file($entry, true) == true)
                    {
                        $no_dir_ext[] = $entry;
                        $debug["errs"]++;
                    }
                    else
                    {
                        if (isset($input["recursive"]) == true)
                        {
                            if (chmod_dir($dir . $entry . "/", $input["decimal"]) == false)
                            {
                                $nochmoddir[] = $entry;
                                $debug["errs"]++;
                            }
                        }
                        else
                        {
                            if (chmod($dir . $entry . "/", octdec((int) $input["decimal"])) == false)
                            {
                                $nochmoddir[] = $entry;
                                $debug["errs"]++;
                            }
                        }
                    }
                }
            }

            // Populate the "no such file" message
            $debug["msg"] .= create_message($nofile, $page["no_such_file"], $page["no_such_files"]);

            // Populate the "no such directory" message
            $debug["msg"] .= create_message($nodir, $page["no_such_directory"], $page["no_such_directories"]);

            // Populate the "no permission" message
            $debug["msg"] .= create_message($noext, $page["chmod_filetype_permission"], $page["chmod_filetypes_permission"]);

            // Populate the "no folder permission" message
            $debug["msg"] .= create_message($no_dir_ext, $page["chmod_filetype_permission_dir"], $page["chmod_filetypes_permission_dir"]);

            // Populate the chmod file error message
            $debug["msg"] .= create_message($nochmodfile, $page["error_chmod_file"], $page["error_chmod_files"]);

            if ($debug["noperm"] == true)
            {
                // Populate the CHMOD directory error message when the user doesn't have file access
                $debug["msg"] .= create_message($nochmoddir, $page["chmod_filetype_permission_dir"], $page["chmod_filetypes_permission_dir"]);
            }
            else
            {
                // Populate the delete directory error message
                $debug["msg"] .= create_message($nochmoddir, $page["error_chmod_directory"], $page["error_chmod_directories"]);
            }


            if ($debug["errs"] == 0)
            {
                $debug["msg"] = $page["all_chmoded"];
            }
            else if ($debug["errs"] < (count($files) + count($dirs)))
            {
                $debug["msg"] .= $page["part_chmoded"];
            }
        }
    break;
    case "link":
        /* ----------------------------
                 Create a Symlink
           ---------------------------- */
        if ($permissions["symlink"] == 0)
        {
            $debug["msg"] = $page["no_symlink_permission"];
        }
        else
        {
            $link_name = clean_path($input["name"]);
            $target    = $dir . clean_path($input["link"]);

            if (empty($input["target"]) == true)
            {
                $link = $permissions["root"] . $link_name;
            }
            else
            {
                $link = $permissions["root"] . $input["target"] . "/" .  $link_name;
            }

            if (preg_match("/^" . str_replace("/", "\/", $target) . "/", $link) == true && $target != $link && is_dir($target) == true && dirname($target) != dirname($link))
            {
                $debug["msg"] = $page["link_path_error"];
            }
            else if (is_invalid_name($link_name) == true || empty($link_name) == true)
            {
                $debug["msg"] = $page["link_invalid"];
            }
            else if (file_exists($target) == false)
            {
                $debug["msg"] = str_replace("<%FILE%>", basename($target), $page["no_such_file"]);
            }
            else if (file_exists($link) == true)
            {
                $debug["msg"] = $page["link_exists"];
            }
            else if (directory_size($permissions["root"]) >= $permissions["space"])
            {
                $debug["msg"] = $page["low_space"];
            }
            else if (symlink($target, $link) == false)
            {
                $debug["msg"] = $page["link_error"];
            }
            else
            {
                $debug["msg"] = $page["link_created"];
            }
        }
    break;
    case "protect":
        /* ----------------------------
           Password protect a directory
           ---------------------------- */
        if ($permissions["htpasswd"] == 0)
        {
            $debug["msg"] = $page["no_htpasswd_permission"];
        }
        else
        {
            $target = clean_path($input["target"]);

            if (empty($target) == true)
            {
                $target = $permissions["root"];
            }
            else
            {
                $target = $permissions["root"] . $target . "/";
            }

            if (file_exists($target) == false)
            {
                $debug["msg"] = $page["protect_no_target"];
            }
            else if (file_exists($target . $settings["htaccess"]) == true && file_exists($target . $settings["htpasswd"]) == false)
            {
                $debug["msg"] = $page["protect_htaccess_exists"];
            }
            else
            {
                // Get the old user data
                if (file_exists($target . $settings["htpasswd"]) == true)
                {
                    // If a file already exists we need to get the data from there
                    $oldfile = file($target . $settings["htpasswd"]);

                    $olduser_data = implode("", $oldfile);

                    foreach ($oldfile as $var)
                    {
                        $tmp = explode(":", $var);
                        $oldusers[$tmp[0]] = $tmp[1];
                    }
                }

                // Get the old config data
                if (file_exists($target . $settings["htaccess"]) == true)
                {
                    $oldconfig_data = implode("", file($target . $settings["htaccess"]));

                    // Get the old authname
                    preg_match("/authname \"(.*)\"/i", $oldconfig_data, $data);
                    $config["auth"] = $data[1];

                    // Get the old path
                    preg_match("/authuserfile \"(.*)\"/i", $oldconfig_data, $data);
                    $config["path"] = $data[1];
                }

                $user_data = "";
                foreach ($input["username"] as $key => $var)
                {
                    if (empty($var) == false)
                    {
                        // The username does not already exist, and the password has not been set
                        if (isset($oldusers[$key]) == false && empty($input["password"][$key]) == true)
                        {
                            break;
                        }

                        // Rather messy way to check for an unchanged password, but its unlikely
                        // that ************ would be used normally
                        if ($input["password"][$key] == "************")
                        {
                            $password = trim($oldusers[$key]);
                        }
                        else
                        {
                            // Windows seems to prefer plan text passwords
                            if (is_win() == true)
                            {
                                $password = $input["password"][$key];
                            }
                            else
                            {
                                $password = crypt($input["password"][$key], CRYPT_STD_DES);
                            }
                        }

                        $user_data .= $var . ":" . $password . "\r\n";
                    }
                }

                if (trim($user_data) == trim($olduser_data) && $config["auth"] == $input["prompt"] && $config["path"] == $target . $settings["htpasswd"])
                {
                    $debug["msg"] = $page["no_password_changes"];
                }
                else
                {
                    if (empty($user_data) == true)
                    {
                        if (file_exists($target . $settings["htpasswd"]) == true)
                        {
                            if (unlink($target . $settings["htaccess"]) == false || unlink($target . $settings["htpasswd"]) == false)
                            {
                                $debug["msg"] = $page["password_remove_error"];
                            }
                            else
                            {
                                $debug["msg"] = $page["unprotected"];
                            }
                        }
                        else
                        {
                            $debug["msg"] = $page["no_password_changes"];
                        }
                    }
                    else
                    {
                        $config_data = "AuthType Basic\r\n";
                        $config_data .= "AuthName \"" . str_replace("\"", "''", $input["prompt"]) . "\"\r\n"; // Replace " with '' so not to break the file, can't escape it as htaccess doesn't allow for it
                        $config_data .= "AuthUserFile \"" . $target . $settings["htpasswd"] . "\"\r\n";
                        $config_data .= "Require valid-user";

                        // Write the password list
                        if (($fp = fopen($target . $settings["htpasswd"], "w")) == false)
                        {
                            $debug["msg"] = $page["error_saving_passwords"];
                        }
                        else
                        {
                            file_lock($fp);
                            fwrite($fp, $user_data);
                            file_lock($fp, LOCK_UN);
                            fclose($fp);

                            // Write the config data
                            if (($fp = fopen($target . $settings["htaccess"], "w")) == false)
                            {
                                $debug["msg"] = $page["error_saving_config"];
                            }
                            else
                            {
                                file_lock($fp, LOCK_EX);
                                fwrite($fp, $config_data);
                                file_lock($fp, LOCK_UN);
                                fclose($fp);

                                $debug["msg"] = $page["protected"];
                            }
                        }
                    }
                }
            }
        }
    break;
    case "archive":
        /* ----------------------------
                    Archiving
           ---------------------------- */
        // $types = array_keys(get_archives_types(1));
        $types = array_keys(get_archives_types());
        $input["filename"] = clean_path($input["filename"]);
        if ($permissions["archive"] == 0)
        {
            $debug["msg"] = $page["no_archive_permission"];
        }
        else if (in_array($input["arctype"], $types) == false)
        {
            $debug["msg"] = $page["no_compression_permission"];
        }
        else if (is_invalid_name($input["filename"]) == true || empty($input["filename"]) == true)
        {
            $debug["msg"] = $page["name_invalid"];
        }
        else
        {
            // Cleanup the input
            if ($permissions["arc_save"] < SAVE_AND_DOWNLOAD)
            {
                $input["option"] = $permissions["arc_save"];
            }

            // Set the filename
            if ($input["arctype"] == "gz")
            {
                $input["filename"] = $input["filename"] . ".tar.gz";
            }
            else if ($input["arctype"] == "tgz")
            {
                $input["filename"] = $input["filename"] . ".tgz";
            }
            else if ($input["arctype"] == "bz2")
            {
                $input["filename"] = $input["filename"] . ".tar.bz2";
            }
            else if ($input["arctype"] == "zip")
            {
                $input["filename"] = $input["filename"] . ".zip";
            }
            else
            {
                $input["filename"] = $input["filename"] . ".tar";
            }


            if (file_exists($dir . $input["filename"]) == true && $permissions["overwrite"] != 1 && $input["option"] != DOWNLOAD)
            {
                // If the file exists, the user is saving the file and they don't have permission to
                // overwrite a file, tell them so
                $debug["msg"] = $page["no_overwrite_permission"];
            }
            else if (($input["arctype"] == "tgz" || $input["arctype"] == "gz" || $input["arctype"] == "zip") && extension_loaded("zlib") == false)
            {
                $debug["msg"] = $page["no_zlib_module"];
            }
            else if ($input["arctype"] == "bz" && extension_loaded("bz2") == false)
            {
                $debug["msg"] = $page["no_bzip2_module"];
            }
            else
            {
                if ($input["arctype"] == "zip")
                {
                    include ($settings["script_dir"] . "modules/archive/zip.lib.php");
                }
                else
                {
                    include ($settings["script_dir"] . "modules/archive/tar.lib.php");
                }

                if (class_exists("archive") == false)
                {
                    $debug["msg"] = $page["error_loading_archive"];
                }
                else
                {
                    // Create the archive object
                    $archive = new archive($dir , $input["filename"]);

                    // Add the directories
                    $input["directories"] = explode("|", $input["directories"]);
                    foreach ($input["directories"] as $var)
                    {
                        if (empty($var) == false && check_link($dir) == false)
                        {
                            if (file_exists($dir . $var) == false || is_dir($dir . $var) == false)
                            {
                                $nodir[] = $var;
                                $debug["errs"]++;
                            }
                            else if (is_sys_file($dir . $var, true) == true)
                            {
                               $no_dir_ext[] = $var;
                               $debug["errs"]++;
                            }
                            else
                            {
                                $archive->add_directory($dir . $var);
                            }
                        }
                    }

                    // Add the files
                    $input["files"] = explode("|", $input["files"]);
                    foreach ($input["files"] as $var)
                    {
                        if (empty($var) == false)
                        {
                            if (file_exists($dir . $var) == false || is_file($dir . $var) == false)
                            {
                                $nofile[] = $var;
                                $debug["errs"]++;
                            }
                            else if (file_permission($dir . $var) == false || is_sys_file($dir . $var) == true)
                            {
                               $noext[] = $var;
                               $debug["errs"]++;
                            }
                            else
                            {
                                $workfiles[] = $dir . $var;
                            }
                        }
                    }
                    $archive->add_files($workfiles);

                    // Create the archive
                    $archive->create_archive();

                    // Compress is needed
                    if ($input["arctype"] == "gz" || $input["arctype"] == "tgz")
                    {
                        $archive->gzip_compress($input["arctype"]);
                    }
                    else if ($input["arctype"] == "bz")
                    {
                        $archive->bzip2_compress();
                    }

                    if (count($archive->data_in_mem) < 1)
                    {
                        $debug["msg"] = $page["archive_empty"];
                    }
                    else if ($input["option"] == DOWNLOAD)
                    {
                        // Download the archive
                        $archive->download_archive();
                    }
                    else
                    {
                        // Ensure there is enough space for the archive
                        if (directory_size($permissions["root"]) + strlen($archive->data) > $permissions["space"])
                        {
                            $debug["msg"] = $page["low_space"];
                        }
                        else
                        {
                            // Save the archive
                            $archive->save_archive();

                            if ($archive->no_save == true)
                            {
                                $debug["msg"] = $page["archive_save_error"];
                                continue;
                            }

                            // Populate the "no such file" message
                            $debug["msg"] .= create_message($nofile, $page["no_such_file"], $page["no_such_files"]);

                            // Populate the "no such directory" message
                            $debug["msg"] .= create_message($nodir, $page["no_such_directory"], $page["no_such_directories"]);

                            // Populate the "no permission" message
                            $debug["msg"] .= create_message($noext, $page["archive_filetype_permission"], $page["archive_filetypes_permission"]);

                            // Populate the "no folder permission" message
                            $debug["msg"] .= create_message($no_dir_ext, $page["archive_filetype_permission_dir"], $page["archive_filetypes_permission_dir"]);

                            if ($archive->noext == true)
                            {
                                $debug["msg"] .= $page["archive_nosub_file_permission"] . "<br />";
                            }

                            if ($archive->no_save == true)
                            {
                                $debug["msg"] = $page["archive_save_error"];
                            }
                            else if ($debug["errs"] > 0 || $archive->errors > 0)
                            {
                                $debug["msg"] .= $page["archive_created_errors"];
                            }
                            else
                            {
                                $debug["msg"] .= $page["archive_created"];
                            }

                            // Download the archive
                            if ($input["option"] == SAVE_AND_DOWNLOAD && $archive->no_save == false)
                            {
                                $file = empty($input["dir"]) == true ? $permissions["root_url"] . $archive->file_name : $permissions["root_url"] . $input["dir"] . "/" . $archive->file_name;
                                $debug["msg"] .= "<meta http-equiv=\"refresh\" content=\"1; url=" . $file . "\" />";
                            }
                        }
                    }
                }
            }
        }
    break;
    case "extract":
        /* ----------------------------
                 Archive extraction
           ---------------------------- */
        $input["archive"] = clean_path($input["archive"]);
        $input["target"]  = clean_path($input["target"]);

        $types            = array_keys(get_archives_types(1));
        $input["arctype"] = ext($input["archive"]);

        if ($permissions["view_arc"] == 0)
        {
            $debug["msg"] = $page["no_extraction_permission"];
        }
        else if (in_array($input["arctype"], $types) == false || file_permission($input["archive"]) == false)
        {
            $debug["msg"] = $page["no_compression_permission"] ;
        }
        else if (file_exists($dir . $input["archive"]) == false)
        {
            $debug["msg"] = str_replace("<%FILE%>", basename($input["archive"]), $page["no_such_file"]);
        }
        else if ($input["arctype"] == "zip" && extension_loaded("zip") == false)
        {
            $debug["msg"] = $page["no_zip_module"];
        }
        else if (($input["arctype"] == "tgz" || $input["arctype"] == "gz") && extension_loaded("zlib") == false)
        {
            $debug["msg"] = $page["no_zlib_module"];
        }
        else if ($input["arctype"] == "bz" && extension_loaded("bz2") == false)
        {
            $debug["msg"] = $page["no_bzip2_module"];
        }
        else
        {
            if ($input["arctype"] == "zip")
            {
                include ($settings["script_dir"] . "modules/archive/zip.lib.php");
            }
            else
            {
                include ($settings["script_dir"] . "modules/archive/tar.lib.php");
            }

            if (class_exists("archive") == false)
            {
                $debug["msg"] = $page["error_loading_archive"];
            }
            else
            {
                // Create the archive object
                $archive = new archive($dir, $input["archive"]);

                $output  = $permissions["root"] . $input["target"] . "/";

                // If the user selected the use archive name option we need to get the archive name
                if ($input["usename"] == 1)
                {
                    $name   = explode(".", $input["archive"]);
                    $output = $output . $name[0] . "/";
                }

                $archive->extract_files($output);

                if ($archive->open_error == true)
                {
                    $debug["msg"] = $page["error_open_archive"];
                }
                else
                {
                    if (count($archive->no_dir_open) > 0)
                    {
                        $debug["msg"] = $page["extract_dir_structure"];
                    }

                    if ($archive->noext == true)
                    {
                        $debug["msg"] .= $page["extract_no_permission"];
                    }

                    if (count($archive->file_exists) > 0)
                    {
                        $debug["msg"] .= $page["extract_no_overwrite_permission"];
                    }

                    if (count($archive->no_overwrite_file) > 0)
                    {
                        $debug["msg"] .= $page["extract_no_overwrite_selected"];
                    }

                    if (count($archive->nospace) > 0)
                    {
                        $debug["msg"] .= $page["extract_low_space"];
                    }

                    if (count($archive->openfile) > 0)
                    {
                        $debug["msg"] .= $page["extract_error"];
                    }

                    if ($archive->errors > 0)
                    {
                        if ($archive->errors == $archive->total_files)
                        {
                            $debug["msg"] = $page["extract_error_all"];
                        }
                        else
                        {
                            $debug["msg"] .= $page["extract_error_partial"];
                        }

                        if ($input["delete"] == 1 && $permissions["delete"] == 1)
                        {
                            $debug["msg"] .= $page["extract_no_delete"];
                        }
                    }
                    else
                    {
                        $debug["msg"] = $page["extracted_all"];

                        // Delete the archive if need be
                        if ($input["delete"] == 1 && $permissions["delete"] == 1)
                        {
                            if (unlink($archive->file_path_name) == true)
                            {
                                 $debug["msg"] .= $page["extracted_all_deleted"];
                            }
                            else
                            {
                                $debug["msg"] .= $page["extracted_all_not_deleted"];
                            }
                        }
                    }
                }
            }
        }
    break;
}

// Get the page numbers
$currentPage = is_numeric($input["pg"]) == true ? round($input["pg"]) : 1;
$firstItem = (($currentPage - 1) * $settings["perpage"]);
$firstItem = $firstItem < 1 ? 0 : $firstItem;

// Now we traverse the directory, the reason for doing this now is because if we do it before any command are run the output won't be "in sync"
$info    = traverse($dir, $firstItem);  // Get the directory listing
$dirinfo = unix_info($dir);             // Get the directory info

// Set the message if need be
if (empty($debug["msg"]) == false)
{
    $info["msg"] = $debug["msg"];
}
else if (empty($info["msg"]) == true && file_exists($permissions["root"] . $settings["msg"]) == true)
{
    // Read the admin messages but only if we haven't already set a message
    if ($fp = fopen($permissions["root"] . $settings["msg"], "r"))
    {
        $info["msg"] = fread($fp, filesize($permissions["root"] . $settings["msg"]));
        fclose($fp);

        if (filemtime($permissions["root"] . $settings["msg"]) - (time() - (int) $settings["msgtimeout"]) > 0 || $permissions["adminmsg"] == 0)
        {
            $info["msg"] = preg_replace("/<%ADMINMSG%>/i", nl2br($info["msg"]), $page["admin_message"]);
        }
        else
        {
            $info["msg"] = preg_replace("/<%ADMINMSG%>/i", nl2br($info["msg"]), $page["del_admin_message"]);
        }
    }
}
else if (isset($input["msg"]) == true && isset($page[$input["msg"]]) == true)
{
    $info["msg"] = $page[$input["msg"]];
}
else if (empty($info["msg"]) == true && $dir == $permissions["root"])
{
    $info["msg"] = get_history();
}

// Get the directory notes
$notes = $page["none"];
if (file_exists($dir . $settings["notes"]) == true)
{
    $fp = fopen($dir . $settings["notes"], "r");

    if ($fp == true)
    {
        $notes = fread($fp, filesize($dir . $settings["notes"]));
        fclose($fp);

        $notes = nl2br($notes);
    }
}
$notes = strlen($notes) == 0 ? $page["none"] : $notes;

$bookmarks = bookmark_form();

// Toggle the different template sections
$info["upperall"]  = $info["count"] > 1 ? 1 : 0;
$info["lowerall"]  = $info["count"] > 1 ? 1 : 0;
$info["bookmarks"] = $bookmarks["count"] > 0 ? 1 : 0;
$info["editnote"]  = $permissions["notes"] > 0 ? 1 : 0;

$page["filetools"] = generate_toolbar();
if (empty($page["filetools"]) == true)
{
    $info["upper_filetools"] = 0;
    $info["lower_filetools"] = 0;
}
else
{
    $info["upper_filetools"] = 1;
    $info["lower_filetools"] = $settings["show_second_toolbar"] == 1 ? 1 : "";
}

$info["total_size"] = $info["total_size"] > 0 ? $info["total_size"] :  "";
if ($info["total_size"] > 0 && $permissions["stats"] == 1 )
{
    $info["total_size"] = "<a href=\"index.php?act=cp&amp;op=report&amp;dir=<%THIS%>\">" . $info["total_size"] . "</a>";
}

// Get the blocks from the template file and then parse them
$blocks = array("msg", "up", "folders", "files", "total_size", "upperall", "lowerall", "bookmarks", "upper_filetools", "lower_filetools", "editnote");
$blocks = get_blocks($settings["script_dir"] . "skin/index.tpl", $blocks);
$blocks = parse_blocks($info, $blocks);

// There is a fatal error!
if (isset($info["fatal"]))
{
    // Finish up by outputting the completed page and then exit
    $page["title"]   = $page["fatal_error"];
    $page["msg"]     = $info["msg"];

    $page["content"] = static_template($settings["script_dir"] . "skin/msg.tpl", $page);
    exit(static_template($settings["script_dir"] . "skin/global.tpl", $page, 1));
}

// Build the dropdown filter list
$list = array("all" => $page["show_all"], "fold" => $page["fold_only"], "img" => $page["img_only"], "mm" => $page["mm_only"], "arc" => $page["archive_only"], "txt" => $page["ascii_only"], "db" => $page["db_only"], "bin" => $page["bin_only"], "off" => $page["off_only"], "misc" => $page["show_others"], "cus" => $page["custom_only"]);

// Remove any entries which aren't allowed if show_all is set to off
if ($permissions["show_all"] == 0)
{
    foreach ($list as $key => $var)
    {
        if ($key != "all" && $key != "fold" && $key != "misc" && $key != "cus")
        {
            $tmp = explode(" ", $settings["filters"][$key]);

            $allowed = false;
            foreach ($tmp as $var)
            {
                if (file_permission("tmp." . $var) == true)
                {
                    $allowed = true;
                    break;
                }
            }

            if ($allowed == false)
            {
                unset($list[$key]);
            }
        }
    }
}

$input["predef"]       = $input["predef"] == "" ? "all" : $input["predef"];
$page["filter"]        = dropdown("predef", $list, $input["predef"], true);

$page["js"]            = "list";
$page["content"]       = parse_html($page, dynamic_template($settings["script_dir"] . "skin/index.tpl", $blocks));
$page["nav"]           = $info["nav"];
$page["back_img"]      = $info["back_img"];
$page["dir"]           = str_replace("%23", "#", $info["dir"]);
$page["this"]          = $info["this"];
$page["is_read"]       = is_readable($dir) == 1 ? "Yes" : "No";
$page["is_write"]      = is_writable($dir) == 1 ? "Yes" : "No";
$page["owner"]         = $dirinfo["owner"];
$page["group"]         = $dirinfo["group"];
$page["notes"]         = $notes;
$page["total_folders"] = count($info["folders"]);
$page["total_files"]   = $info["count"] - count($info["folders"]);
$page["booklist"]      = $bookmarks["list"];
$page["custom_filter"] = $input["custom"];
$page["page_links"]    = page_links($info["pages"], $currentPage);

$page["this_folder_icon"] = "openfolder";
if (file_exists($dir . $settings["htaccess"] ) == true && file_exists($dir . $settings["htpasswd"]) == true && preg_match("/apache/i", $_SERVER["SERVER_SOFTWARE"]) == true)
{
    $page["this_folder_icon"] = "lockedopenfolder";
}

?>