<?php

/*
 * Invision Power File Manager
 * MP3 info viewer
 * Copyright (c) 2005 Invision Power Services
 * -------------------------------------
 * You may not remove the copyright or
 * redistribute the script in any form
 *
 * Authors: Stephen Ball
 * Version: 1.0.1
 * Last Modified: 16th August 2004
 *
 */

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */

// Clean the user input
$target     = clean_path($input["target"]);

// Set some parameters
$global_tpl = "skin/popup.tpl";
$file_name  = "/" . $target;

// If we are running on Windows we need to reverse the slashes
if (is_win() == true)
{
    $file_name = preg_replace("/[\/]/", "\\", $file_name);
}

if (file_exists($permissions["root"] . $target) == false)
{
    // The user didn't select any existing object
    $page["title"]   = $page["user_error"];
    $page["msg"]     = $page["no_exist"];
    $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
}
else if (is_file($permissions["root"] . $target) == false)
{
    // The user didn't select a file, but selected folder or none supported file type, eg socket or block device
    $page["title"]   = $page["user_error"];
    $page["msg"]     = $page["no_file"];
    $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
}
else if (file_permission($target) == false || $permissions["mp3"] == 0)
{
    // The user doesn't have filetype permissions
    $page["title"]   = $page["user_error"];
    $page["msg"]     = $page["no_perm"];
    $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
}
else if (filesize($permissions["root"] . $target) == 0)
{
    // The file is empty
    $page["title"]   = $page["blank"];
    $page["msg"]     = $page["empty"];
    $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
}
else if (preg_match("/\.mp3$/i", $target) == false)
{
    // The user didn't select a valid file
    $page["title"]   = $page["user_error"];
    $page["msg"]     = $page["not_mp3"];
    $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
}
else
{
    require($settings["script_dir"] . "modules/others/mp3info.lib.php");

    $mp3 = new MP3($permissions["root"] . $target);

    if (empty($mp3->fh) == true)
    {
        // The file could not be opened
        $page["title"]   = $page["system_error"];
        $page["msg"]     = $page["no_open"];
        $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
    }
    else
    {
        // Get the MP3 tag info
        $mp3->get_id3();

        if ($mp3->error == "no_id3")
        {
            // The user didn't select a valid file
            $page["title"]  = $page["user_error"];
            $page["msg"]    = $page["no_id3"];

           $page["content"] = static_template($settings["script_dir"] . "skin/cards/error.tpl", $page);
        }
        else
        {
            $page["artist"]   = empty($mp3->id3["author"])   ? $page["unknown"] : htmlspecialchars($mp3->id3["author"]);
            $page["track"]    = empty($mp3->id3["title"])    ? $page["unknown"] : htmlspecialchars($mp3->id3["title"]);
            $page["album"]    = empty($mp3->id3["album"])    ? $page["unknown"] : htmlspecialchars($mp3->id3["album"]);
            $page["year"]     = empty($mp3->id3["year"])     ? $page["unknown"] : htmlspecialchars($mp3->id3["year"]);
            $page["genre"]    = empty($mp3->id3["genre"])    ? $page["unknown"] : htmlspecialchars($mp3->id3["genre"]);
            $page["comments"] = empty($mp3->id3["comments"]) ? $page["none"]    : htmlspecialchars($mp3->id3["comments"]);

            // Now we get the file info
            $mp3->get_info();

            $page["bitrate"]  = empty($mp3->info["bitrate"])       ? $page["unknown"] : $mp3->info["bitrate"];
            $page["sample"]   = empty($mp3->info["sampling_rate"]) ? $page["unknown"] : $mp3->info["sampling_rate"] . " Hz (" . $mp3->info["sampling_rate"] / 1000 . " kHz)";
            $page["channels"] = empty($mp3->info["channel_mode"])  ? $page["unknown"] : $mp3->info["channel_mode"];
            $page["length"]   = empty($mp3->info["length"])        ? $page["unknown"] : $mp3->info["length"];
            $page["size"]     = format_bytes($mp3->info["filesize"]);

            $page["file"]     = $file_name;
            $page["icon"]     = get_icon(basename($target));

            $page["content"] = static_template($settings["script_dir"] . "skin/cards/mp3.tpl", $page);
        }

        $mp3->close();
    }
}

$page["js"]      = "";

?>