<?php

/*
* Invision Power File Manager
* User Control Panel
* Copyright (c) 2005 Invision Power Services
* -------------------------------------
* You may not remove the copyright or
* redistribute the script in any form
*
* Authors: Stephen Ball
* Version: 1.0.1
* Last Modified: 27th October 2004
*
*/

// Security check
if (defined("IPFM_RUNNING") == false)
{
    die("You can not access this module directly");
}

require $settings["script_dir"] . "settings/sys/zones.inc.php";

/* -----------------------------------------
                Define Constants
   ----------------------------------------- */
define("STATS", 1);
define("REPORT", 2);

/* -----------------------------------------
                Module Functions
   ----------------------------------------- */
// Traverses the users space generating a detailed listing
function get_stats($dir, $traverse = true)
{
    global $permissions, $settings;

    // Create arrays to store files and directories
    $_files      = array();
    $_subdirs    = array();
    $tmp         = array("found" => false, "system" => 0);

    if (($dp = opendir($dir)) == false)
    {
        // The directory could not be opened
        return false;
    }

    // Loop through all directory entries, and construct two temporary arrays containing files and sub directories
    while($entry = readdir($dp))
    {
        if (check_link($dir . $entry) == false && $entry !=  ".." && $entry !=  ".")
        {
            if (is_dir($dir . $entry) == true)
            {
                $_subdirs[] = $entry;
            }
            else
            {
                $_files[] = $entry;
            }
        }
    }

    // Close directory
    closedir($dp);

    // Now sort out the files
    for($i=0; $i<count($_files); $i++)
    {
        $size = filesize($dir . $_files[$i]);
        if ($size > $tmp["largesize"])
        {
            $tmp["largefile"] = str_replace($permissions["root"], "/", $dir . $_files[$i]);
            $tmp["largesize"] = $size;
        }

        if (is_sys_file($dir . $_files[$i]) == true)
        {
            $tmp["system"] = $tmp["system"] + $size;
        }

        // If the file isn't hidden show it
        if (($settings["hide_dot"] == true && preg_match("/^\./", $_files[$i]) == false) || $settings["hide_dot"] == false)
        {
            $ext = ext($_files[$i]);

            $tmp["types"][$ext]["count"]++;
            $tmp["types"][$ext]["size"] = $tmp["types"][$ext]["size"] + $size;
            $tmp["found"] = true;
        }
    }

    $tmp["dircount"] = count($_subdirs);

    // Traverse the directories if traverse if set
    if ($traverse == true)
    {
        for($i=0; $i<count($_subdirs); $i++)
        {
            // If the folder isn't hidden show it
            if (($settings["hide_dot"] == true && preg_match("/^\./", $_subdirs[$i]) == false) || $settings["hide_dot"] == false)
            {
                $returned = get_stats($dir . $_subdirs[$i] . "/");
            }

            if (is_array($returned["types"]) == true)
            {
                foreach ($returned["types"] as $key => $var)
                {
                    $tmp["types"][$key]["count"] = $tmp["types"][$key]["count"] + $returned["types"][$key]["count"];
                    $tmp["types"][$key]["size"]  = $tmp["types"][$key]["size"]  + $returned["types"][$key]["size"];
                    $tmp["found"] = true;
                }
            }

            if ($returned["largesize"] > $tmp["largesize"])
            {
                $tmp["largesize"] = $returned["largesize"];
                $tmp["largefile"] = $returned["largefile"];
            }

            $tmp["dircount"] = $tmp["dircount"] + $returned["dircount"];

            $tmp["system"] = $tmp["system"] + $returned["system"];
        }

    }

    // Remove any system files
    unset($tmp["types"][str_replace("." , "", $settings["bookmarks"])]);
    unset($tmp["types"][str_replace("." , "", $settings["msg"])]);
    unset($tmp["types"][str_replace("." , "", $settings["notes"])]);

    return $tmp;
}

/* -----------------------------------------
                Program Execution
   ----------------------------------------- */

if ($permissions["ucp"] == 0 && $permissions["use_bookmarks"] == 0 && $permissions["stats"] == 0)
{
    go_to("index.php?op=list&dir=" . $dir . "&msg=no_ucp_permission");
}
else
{
    switch ($input["op"])
    {
        case "img":
            /* ----------------------------
                  Generate the pie chart
               ---------------------------- */
            if ($permissions["stats"] == 0)
            {
                exit;
            }

            header("Content-type: image/png", true); // Firstly output the PNG header

            $id         = ImageCreate(180, 180);
            $hcenter    = ImageSX($id)/2;
            $vcenter    = ImageSY($id)/2;

            $diameter   = 160;

            // Define the colours
            $black          = ImageColorAllocate($id, 0, 0, 0);
            $white          = ImageColorAllocate($id, 255, 255, 255);

            $arc_dec[0]     = 0;
            $arc_rad[0]     = 0;

            // If the type is stats show the space pie chart
            if ($input["type"] == STATS)
            {
                $pie_color[1]   = ImageColorAllocate($id, $settings["free_chart"][0], $settings["free_chart"][1], $settings["free_chart"][2]); // Free Space
                $pie_color[2]   = ImageColorAllocate($id, $settings["used_chart"][0], $settings["used_chart"][1], $settings["used_chart"][2]);  // Used Space

                $total  = $permissions["space"];
                $n      = 2;

                // Read the arguments into different arrays
                $value[1] = $total - directory_size($permissions["root"]);
                $value[2] = directory_size($permissions["root"]);

                // If all the space is used, set free space to 0
                if ($value[2] > $total)
                {
                    $value[1] = 0;
                }

                $arc_rad[1] = $value[1] * 2 * pi();
                $arc_rad[2] = $total * 2 * pi();
                $arc_dec[1] = $value[1] * 360;
                $arc_dec[2] = $total * 360;
            }
            else
            {
                // Otherwise show the file types pie chart

                $pie_color[1]   = ImageColorAllocate($id, $settings["folder_chart"][0], $settings["folder_chart"][1] ,$settings["folder_chart"][2]);
                $pie_color[2]   = ImageColorAllocate($id, $settings["img_chart"][0], $settings["img_chart"][1], $settings["img_chart"][2]);
                $pie_color[3]   = ImageColorAllocate($id, $settings["mm_chart"][0], $settings["mm_chart"][1], $settings["mm_chart"][2]);
                $pie_color[4]   = ImageColorAllocate($id, $settings["db_chart"][0], $settings["db_chart"][1], $settings["db_chart"][2]);
                $pie_color[5]   = ImageColorAllocate($id, $settings["bin_chart"][0], $settings["bin_chart"][1], $settings["bin_chart"][2]);
                $pie_color[6]   = ImageColorAllocate($id, $settings["off_chart"][0], $settings["off_chart"][1], $settings["off_chart"][2]);
                $pie_color[7]   = ImageColorAllocate($id, $settings["arc_chart"][0], $settings["arc_chart"][1], $settings["arc_chart"][2]);
                $pie_color[8]   = ImageColorAllocate($id, $settings["txt_chart"][0], $settings["txt_chart"][1], $settings["txt_chart"][2]);
                $pie_color[9]   = ImageColorAllocate($id, $settings["misc_chart"][0], $settings["misc_chart"][1], $settings["misc_chart"][2]);

                $total = 0;
                $n     = 0;

                // Read the arguments into different arrays
                foreach ($input["part"] as $key => $var)
                {
                    $var = intval($var);

                    $n++;

                    $value[$n] = $var;
                    $total = $total + $var;

                    $arc_rad[$n] = $total * 2 * pi();
                    $arc_dec[$n] = $total * 360;
                }
            }

            // The white background will be transparent
            ImageFill($id, 0, 0, $white);
            ImageColorTransparent($id, $white);

            // Count the labels
            for($i=1; $i<=$n; $i++)
            {
                // Calculate the percents
                $perc[$i]    = $value[$i]/$total;

                // Calculate the arc and line positions
                $arc_rad[$i]    = $arc_rad[$i] / $total;
                $arc_dec[$i]    = $arc_dec[$i] / $total;
                $hpos           = round($hcenter + ($diameter/2) * sin($arc_rad[$i]));
                $vpos           = round($vcenter + ($diameter/2) * cos($arc_rad[$i]));
                ImageLine($id, $hcenter, $vcenter, $hpos, $vpos, $black);
                ImageArc($id, $hcenter, $vcenter, $diameter, $diameter, $arc_dec[$i-1], $arc_dec[$i], $black);
            }

            for($i=1; $i<=$n; $i++)
            {
                // If the arc is the same as the previous then ignore it
                if ($arc_rad[$i] != $arc_rad[$i-1])
                {
                    $arc_rad_label  = $arc_rad[$i-1] + 0.5 * $perc[$i] * 2 * pi();
                    $hpos           = $hcenter + 0.8 * ($diameter/2) * sin($arc_rad_label);
                    $vpos           = $vcenter + 0.8 * ($diameter/2) * cos($arc_rad_label);
                    ImageFillToBorder($id, $hpos, $vpos, $black, $pie_color[$i]);
                }
            }

            ImagePng($id);
            ImageDestroy($id);

            exit;
        break;
        case "stats":
            /* ----------------------------
                       Space usage
               ---------------------------- */
            $page["caption"] = $page["stats_caption"];

            if ($permissions["stats"] == 0)
            {
                $page["msg"]    = $page["no_stats_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                $blocks = get_blocks($settings["script_dir"] . "skin/usercp/stats.tpl", array("key", "nogd", "lowspace"));

                $page["img"] = "<img src=\"images/nogd.gif\" alt=\"" . $page["no_gd"] . "\" title=\"" . $page["no_gd"] . "\" />";

                // Check if GD is installed and if PNG is supported
                if (function_exists("imagetypes") == true && imagetypes() & IMG_PNG)
                {
                    $page["img"]    = "<img src=\"index.php?act=cp&amp;op=img&amp;type=1\" alt=\"" . $page["space_usage"] . "\" title=\"" . $page["space_usage"] . "\" />";
                    $blocks["nogd"] = "";
                }
                else
                {
                    $blocks["key"]  = "";
                    $blocks["nogd"] = parse_html($page, $blocks["nogd"]);
                }

                $used = directory_size($permissions["root"]);
                $free = $used > $permissions["space"] ? 0 : $permissions["space"] - $used;

                $page["used"]     = format_bytes($used);
                $page["total"]    = format_bytes($permissions["space"]);
                $page["free"]     = format_bytes($free);

                $page["used_percentage"] = round(($used/$permissions["space"])*100, 2) . "%";
                $page["free_percentage"] = round(($free/$permissions["space"])*100, 2) . "%";

                $page["free_key"] = implode(", ", $settings["free_chart"]);
                $page["used_key"] = implode(", ", $settings["used_chart"]);

                if ($free > $settings["lowspace"])
                {
                    $blocks["lowspace"] = "";
                }

                $page["usercp"]  = parse_html($page, dynamic_template($settings["script_dir"] . "skin/usercp/stats.tpl", $blocks));
            }
        break;
        case "report":
            /* ----------------------------
                       Usage report
               ---------------------------- */
            $page["caption"] = $page["report_caption"];

            if ($permissions["stats"] == 0)
            {
                $page["msg"]    = $page["no_stats_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                if (isset($input["dir"]) == true)
                {
                    $input["dir"] = clean_path($input["dir"]);
                    if (empty($input["dir"]) == true)
                    {
                        $data = get_stats($permissions["root"], false);
                    }
                    else
                    {
                        $data = get_stats($permissions["root"] . $input["dir"] . "/", false);
                    }
                    $page["once"] = 1;
                }
                else
                {
                    $data = get_stats($permissions["root"]);
                    $page["once"] = 0;
                }

                if ($data["found"] == true)
                {
                    $blocks = get_blocks($settings["script_dir"] . "skin/usercp/report.tpl", array("dir", "all", "folders", "files", "lowspace"));

                    // Build a list for the ASCII filter, no need to define a seperate filter, just use the ASCII files array
                    $settings["filters"]["txt"] = "";
                    foreach ($settings["ascii"] as $var)
                    {
                        $settings["filters"]["txt"] .= $var . " ";
                    }

                    $counts = array("total" => 0, "misc" => 0, "bin" => 0, "off" => 0, "img" => 0, "mm" => 0, "arc" => 0, "db" => 0, "off" => 0, "txt" => 0);
                    $sizes  = array("total" => 0, "misc" => 0, "bin" => 0, "off" => 0, "img" => 0, "mm" => 0, "arc" => 0, "db" => 0, "off" => 0, "txt" => 0);

                    // Filter the file types list
                    foreach ($data["types"] as $index => $value)
                    {
                        $found = 0;

                        foreach ($settings["filters"] as $key => $var)
                        {
                            $tmp = explode(" ", trim($var));

                            if (in_array($index, $tmp) == true)
                            {
                                $counts[$key] = $counts[$key] + $value["count"];
                                $sizes[$key]  = $sizes[$key]  + $value["size"];
                                $found = 1;
                                break;
                            }
                        }

                        if ($found == 0)
                        {
                            $counts["misc"] = $counts["misc"] + $value["count"];
                            $sizes["misc"]  = $sizes["misc"]  + $value["size"];
                        }

                        $counts["total"] = $counts["total"] + $value["count"];
                        $sizes["total"]  = $sizes["total"]  + $value["size"];
                    }
                    $counts["folders"] = $data["dircount"];

                    $page["largest_size"]  = format_bytes($data["largesize"]);
                    $page["dir"]           = substr(dirname($data["largefile"]), 1, strlen(dirname($data["largefile"])));
                    $page["folder_count"]  = $counts["folders"];
                    $page["folder_key"]    = implode(", ", $settings["folder_chart"]);

                    // Sort out the report
                    $report = array("img"  => array("COUNT"      => $counts["img"],
                                                    "SIZE"       => format_bytes($sizes["img"]),
                                                    "PERCENTAGE" => round(($sizes["img"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["img_chart"])
                                              ),
                                    "mm"   => array("COUNT"      => $counts["mm"],
                                                    "SIZE"       => format_bytes($sizes["mm"]),
                                                    "PERCENTAGE" => round(($sizes["mm"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["mm_chart"])
                                              ),
                                    "db"   => array("COUNT"      => $counts["db"],
                                                    "SIZE"       => format_bytes($sizes["db"]),
                                                    "PERCENTAGE" => round(($sizes["db"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["db_chart"])
                                              ),
                                    "bin"  => array("COUNT"      => $counts["bin"],
                                                    "SIZE"       => format_bytes($sizes["bin"]),
                                                    "PERCENTAGE" => round(($sizes["bin"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["bin_chart"])
                                              ),
                                    "off"  => array("COUNT"      => $counts["off"],
                                                    "SIZE"       => format_bytes($sizes["off"]),
                                                    "PERCENTAGE" => round(($sizes["off"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["off_chart"])
                                              ),
                                    "arc"  => array("COUNT"      => $counts["arc"],
                                                    "SIZE"       => format_bytes($sizes["arc"]),
                                                    "PERCENTAGE" => round(($sizes["arc"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["arc_chart"])
                                              ),
                                    "txt"  => array("COUNT"      => $counts["txt"],
                                                    "SIZE"       => format_bytes($sizes["txt"]),
                                                    "PERCENTAGE" => round(($sizes["txt"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["txt_chart"])
                                              ),
                                    "misc" => array("COUNT"      => $counts["misc"],
                                                    "SIZE"       => format_bytes($sizes["misc"]),
                                                    "PERCENTAGE" => round(($sizes["misc"]/$sizes["total"])*100, 2) . "%",
                                                    "KEY"        => implode(", ", $settings["misc_chart"])
                                              )
                              );

                    // Build the blocks
                    $blocks["tmp"] = "";
                    foreach ($report as $key => $var)
                    {
                        if ($var["COUNT"] > 0)
                        {
                            $var["FILTER"] = $key;
                            $var["LABEL"]  = $page[$key . "_label"];

                            $blocks["tmp"] .= parse_html($var, $blocks["files"]);
                        }
                    }
                    $blocks["files"] = $blocks["tmp"];

                    $page["total_count"]   = $counts["total"];
                    $page["total_size"]    = format_bytes($sizes["total"]);
                    $page["sysfiles"]      = format_bytes($data["system"]);

                    // Now select which header to show
                    if (isset($input["dir"]) == true)
                    {
                        $page["dir"]          = make_navigation($input["dir"]);

                        $page["largest_file"] = preg_replace("/<%FILE%>/i", basename($data["largefile"]), $page["largest_file_dir"]);

                        $blocks["dir"]        = parse_html($page, $blocks["dir"]);
                        $blocks["all"]        = "";

                        $blocks["lowspace"]   = "";
                    }
                    else
                    {
                        $page["largest_file"]  = preg_replace("/<%FILE%>/i", basename($data["largefile"]), $page["largest_file"]);

                        $blocks["dir"]         = "";
                        $blocks["all"]         = parse_html($page, $blocks["all"]);

                        // The percentage of used space is more than the "lowspace" warning level
                        if ($permissions["space"] - $sizes["total"] > $settings["lowspace"])
                        {
                            $blocks["lowspace"] = "";
                        }
                    }

                    $page["folder"] = $input["dir"];

                    $page["usercp"] = parse_html($page, dynamic_template($settings["script_dir"] . "skin/usercp/report.tpl", $blocks));

                    // If the user doesn't have GD we need to remove the GD stuff
                    if (function_exists("imagetypes") == true && imagetypes() & IMG_PNG)
                    {
                        $page["img1"] = "<img src=\"index.php?act=cp&amp;op=img&amp;type=2&amp;part[1]=" . $counts["folders"] . "&amp;part[2]=" . $counts["img"] . "&amp;part[3]=" . $counts["mm"] . "&amp;part[4]=" . $counts["db"] . "&amp;part[5]=" . $counts["bin"] . "&amp;part[6]=" . $counts["off"] . "&amp;part[7]=" . $counts["arc"] . "&amp;part[8]=" . $counts["txt"] . "&amp;part[9]=" . $counts["misc"] . "\" alt=\"" . $page["report_alt_types"] . "\" title=\"" . $page["report_alt_types"] . "\" />";
                        $page["img2"] = "<img src=\"index.php?act=cp&amp;op=img&amp;type=2&amp;part[1]=0&amp;part[2]=" . $sizes["img"] . "&amp;part[3]=" . $sizes["mm"] . "&amp;part[4]=" . $sizes["db"] . "&amp;part[5]=" . $sizes["bin"] . "&amp;part[6]=" . $sizes["off"] . "&amp;part[7]=" . $sizes["arc"] . "&amp;part[8]=" . $sizes["txt"] . "&amp;part[9]=" . $sizes["misc"] . "\" alt=\"" . $page["report_alt_space"] . "\" title=\"" . $page["report_alt_space"] . "\" />";
                    }
                    else
                    {
                        $page["usercp"] = preg_replace("/<!-- BEGIN DYNAMIC BLOCK: gd -->(.+?)<!-- END DYNAMIC BLOCK: gd -->/si", "&nbsp;", $page["usercp"]);
                    }
                }
                else
                {
                    $page["msg"]    = $page["no_report"];
                    $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                }
            }
        break;
        case "clean":
            /* ----------------------------
                      Bookmark cleanup
               ---------------------------- */
            $page["caption"] = $page["cleanup_caption"];
            if ($permissions["bookmark"] == 0)
            {
                $page["msg"]    = $page["no_bookmark_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                $data = array("total" => 0, "invalid" => 0, "duplicate_names" => 0, "duplicate_paths" => 0, "names" => array(), "paths" => array());

                if (file_exists($permissions["root"] . $settings["bookmarks"]) == false)
                {
                    $page["msg"]    = $page["no_bookmarks"];
                    $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                }
                else
                {
                    $bookmarks = file($permissions["root"] . $settings["bookmarks"]);

                    if (count($bookmarks) < 1)
                    {
                        $page["msg"]    = $page["no_bookmarks"];
                        $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                    }
                    else
                    {
                        while(list(,$value) = each($bookmarks))
                        {
                            list($name, $path) = split("[|]", $value);

                            $name = trim($name);
                            $path = trim($path);

                            $data["total"]++;

                            if (file_exists($permissions["root"] . $path . "/") == false)
                            {
                                // Check for invalid links
                                $data["invalid"]++;
                            }
                            else if (in_array($name, $data["names"]) == true)
                            {
                                // Now check for duplicate names
                                $data["duplicate_names"]++;
                            }
                            else if (in_array($path, $data["paths"]) == true)
                            {
                                // Finally check for duplicate paths
                                $data["duplicate_paths"]++;
                            }

                            $data["names"][] = $name;
                            $data["paths"][] = $path;
                        }

                        $page["total"]           = $data["total"];
                        $page["invalid"]         = $data["invalid"];
                        $page["duplicate_names"] = $data["duplicate_names"];
                        $page["duplicate_paths"] = $data["duplicate_paths"];

                        $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/cleanup.tpl", $page);
                    }
                }
            }
        break;
        case "do_cleanup":
            /* ----------------------------
                        Run cleanup
               ---------------------------- */
            $page["caption"] = $page["cleanup_caption"];
            if ($permissions["bookmark"] == 0)
            {
                $page["msg"]    = $page["no_bookmark_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                if (count($input["options"]) == 0)
                {
                    $page["msg"]    = $page["no_cleanup_options"];
                    $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                }
                else
                {
                    if ($input["options"]["all"] == true)
                    {
                        // The user selected delete all, so there is no point in run cleanup options, just remove the bookmarks
                        if (unlink($permissions["root"] . $settings["bookmarks"]) == false)
                        {
                            $page["msg"] = $page["all_cleanup_error"];
                        }
                        else
                        {
                            $page["msg"] = $page["all_cleanup_done"];
                        }

                        $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                    }
                    else
                    {
                        if (file_exists($permissions["root"] . $settings["bookmarks"]) == false)
                        {
                            $page["msg"]    = $page["no_bookmarks"];
                            $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                        }
                        else
                        {
                            $bookmarks = file($permissions["root"] . $settings["bookmarks"]);

                            $tmp = array("total" => 0, "invalid" => 0, "duplicate_names" => 0, "duplicate_paths" => 0, "names" => array(), "paths" => array());

                            if (count($bookmarks) < 1)
                            {
                                $page["msg"]    = $page["no_bookmarks"];
                                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                            }
                            else
                            {
                                $data = "";

                                while(list(,$value) = each($bookmarks))
                                {
                                    list($name, $path) = split("[|]", $value);

                                    $name = trim($name);
                                    $path = trim($path);

                                    if (file_exists($permissions["root"] . $path . "/") == false)
                                    {
                                        // If the file doesn't exist, but invalid was not checked leave the bookmarks
                                        if ($input["options"]["invalid"] == false)
                                        {
                                            $data .= $name . "|" . $path . "\n";
                                        }
                                    }
                                    else if (in_array($name, $tmp["names"]) == true)
                                    {
                                        // If the name already exists in the list, but duplicate name was not checked leave the bookmark
                                        if ($input["options"]["name"] == false)
                                        {
                                            $data .= $name . "|" . $path . "\n";
                                        }
                                    }
                                    else if (in_array($path, $tmp["paths"]) == true)
                                    {
                                        // Finally check the path
                                        if ($input["options"]["path"] == false)
                                        {
                                            $data .= $name . "|" . $path . "\n";
                                        }
                                    }
                                    else
                                    {
                                        $data .= $name . "|" . $path . "\n";
                                    }

                                    $tmp["names"][] = $name;
                                    $tmp["paths"][] = $path;
                                }

                                if (($fp = fopen($permissions["root"] . $settings["bookmarks"], "w")) == fakse)
                                {
                                    $page["msg"]     = $page["error_save_bookmark"];
                                    $page["usercp"]  = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                                }
                                else
                                {
                                    file_lock($fp);
                                    fwrite($fp, $data);
                                    file_lock($fp, LOCK_UN);
                                    fclose($fp);

                                    $page["msg"]    = $page["bookmark_saved"];
                                    $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                                }
                            }
                        }
                    }
                }
            }
        break;
        case "update":
            /* ----------------------------
                     Update bookmarks
               ---------------------------- */
            $page["caption"] = $page["edit_caption"];
            if ($permissions["bookmark"] == 0)
            {
                $page["msg"]    = $page["no_bookmark_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                // Define regex matching patterns
                $search  = array("/\\\/", "/\|/", "/^[\/]*(.+?)[\/]*$/");
                $replace = array("/", "", "\\1");

                $data = "";
                // Create the data file
                foreach ($input["name"] as $key => $var)
                {
                    // Clean the path input
                    $input["path"][$key] = preg_replace($search, $replace, $input["path"][$key]);
                    $input["path"][$key] = clean_path($input["path"][$key]);

                    $input["name"][$key] = str_replace("|", "", $input["name"][$key]);

                    // Both fields are set
                    if (empty($input["name"][$key]) == false && empty($input["path"][$key]) == false)
                    {
                        // Used to check that the path exists, this has now been moved to bookmark cleanup
                        $data .= $var . "|" . $input["path"][$key] . "\n";
                    }
                }

                if (($fp = fopen($permissions["root"] . $settings["bookmarks"], "w")) == fakse)
                {
                    $page["msg"]     = $page["error_save_bookmark"];
                    $page["usercp"]  = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                }
                else
                {
                    file_lock($fp);
                    fwrite($fp, $data);
                    file_lock($fp, LOCK_UN);
                    fclose($fp);

                    $page["msg"]    = $page["bookmark_saved"];
                    $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                }
            }
        break;
        case "man":
            /* ----------------------------
                      Manage bookmarks
               ---------------------------- */
            $page["caption"] = $page["edit_caption"];
            if ($permissions["bookmark"] == 0)
            {
                $page["msg"]    = $page["no_bookmark_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                if (file_exists($permissions["root"] . $settings["bookmarks"]) == true && filesize($permissions["root"] . $settings["bookmarks"]) > 0)
                {
                    $data = "";
                    $blocks = get_blocks($settings["script_dir"] . "skin/usercp/edit.tpl", array("bookmark"));
                    $bookmarks = file($permissions["root"] . $settings["bookmarks"]);

                    natcasesort($bookmarks);

                    while(list(,$value) = each($bookmarks))
                    {
                        list($page["name"], $page["path"]) = split("[|]", $value);

                        $page["view"] = rawurlencode(trim($page["path"]));
                        $page["path"] = "/" . trim($page["path"]) . "/";
                        if (is_win() == true)
                        {
                            $page["path"] = str_replace("/", "\\", $page["path"]);
                        }
                        $data .= parse_html($page, $blocks["bookmark"]);
                    }

                    $blocks["bookmark"] = $data;
                    $page["usercp"]     = parse_html($page, dynamic_template($settings["script_dir"] . "skin/usercp/edit.tpl", $blocks));
                }
                else
                {
                    $page["msg"]    = $page["no_bookmarks"];
                    $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
                }
            }
        break;
        case "time":
            /* ----------------------------
                      Change timezone
               ---------------------------- */
            $page["caption"] = $page["timezone_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                $page["current_user_time"] = gmdate($settings["format"], timestamp(time()));
                $page["current_user_time"] = preg_replace("/<%AT%>/i", $page["at"], $page["current_user_time"]);

                $page["timezone_list"]     = dropdown("timezone", $zones, $permissions["offset"]/3600);

                $page["isdst"] = "";
                if ($permissions["isdst"] == 1)
                {
                    $page["isdst"] = "checked=\"checked\"";
                }

                $page["usercp"]            = static_template($settings["script_dir"] . "skin/usercp/timezone.tpl", $page);
            }
        break;
        case "save_timezone":
            /* ----------------------------
                     Save the timezone
               ---------------------------- */
            $page["caption"] = $page["timezone_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                $valid_zones     = array_keys($zones);

                // If the user tries to enter an invalid timezone use the default zone
                if (in_array($input["timezone"], $valid_zones) == false)
                {
                    $input["timezone"] = $settings["default_offset"];
                }

                if ($input["isdst"] != 1)
                {
                    $input["isdst"] = 0;
                }

                build_query($queries["update_timezone"], array("OFFSET" => $input["timezone"] * 3600, "UID" => $permissions["uid"], "DST" => $input["isdst"]));
                $result = db_query($queries["update_timezone"], $link_id);

                // Check if the query ran
                if (empty($result) == true)
                {
                    $page["msg"] = $page["error_save_timezone"];
                }
                else
                {
                    $page["msg"] = $page["timezone_saved"];
                }

                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
        break;
        case "email":
            /* ----------------------------
                      Change email
               ---------------------------- */
            $page["caption"] = $page["email_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/email.tpl", $page);
            }
        break;
        case "update_email";
            /* ----------------------------
                      Save email
               ---------------------------- */
            $page["caption"] = $page["email_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                if ($input["new_email"] != $input["confirm_email"])
                {
                    $page["msg"] = $page["new_email_mismatch"];
                }
                else if (strlen(trim($input["new_email"])) < 6 || preg_match("/^[^@ ]+@[^@ ]+[\.][^@ \.]+$/", $input["new_email"]) == false)
                {
                    $page["msg"] = $page["email_invalid"];
                }
                else if (strlen($input["new_email"]) > 75)
                {
                    // Email address too long
                    $page["msg"] = $page["email_too_long"];
                }
                else
                {
                    build_query($queries["update_email"], array("EMAIL" => trim($input["new_email"]), "UID" => $permissions["uid"]));
                    $result = db_query($queries["update_email"], $link_id);

                    if (empty($result) == true)
                    {
                        $page["msg"] = $page["error_save_email"];
                    }
                    else
                    {
                        $page["msg"] = $page["email_saved"];
                    }
                }

                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
        break;
        case "pass":
            /* ----------------------------
                      Change password
               ---------------------------- */
            $page["caption"] = $page["password_caption"];
            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                $page["password_tip"] = str_replace("<%MIN_PASS_LEN%>", $settings["min_pass_length"], $page["password_tip"]);

                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/password.tpl", $page);
            }
        break;
        case "update_pass":
            /* ----------------------------
                      Save password
               ---------------------------- */
            $page["caption"] = $page["password_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                if (md5($input["current_pass"]) != $permissions["password"])
                {
                    $page["msg"] = $page["current_pass_wrong"];
                }
                else if ($input["new_pass"] != $input["confirm_pass"])
                {
                    $page["msg"] = $page["new_pass_mismatch"];
                }
                else if (empty($input["new_pass"]) == true || strlen($input["new_pass"]) < $settings["min_pass_length"])
                {
                    $page["msg"] = preg_replace("/<%PASSWORD_LENGTH%>/i", $settings["min_pass_length"], $page["new_pass_short"]);
                }
                else
                {
                    $newpass = md5($input["new_pass"]);

                    build_query($queries["update_password"], array("PASSWORD" => $newpass, "UID" => $permissions["uid"]));
                    $result = db_query($queries["update_password"], $link_id);

                    if (empty($result) == true)
                    {
                        $page["msg"] = $page["error_save_password"];
                    }
                    else
                    {
                        setcookie($settings["pass_cookie"], $newpass, (int) $settings["cookie_time"]);
                        $page["msg"] = $page["password_saved"];
                    }
                }

                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
        break;
        case "lang":
            /* ----------------------------
                    Change language
               ---------------------------- */
            $page["caption"] = $page["language_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {
                // Build the language list
                $lang_name[0] = $page["default_lang"] . " - " . $lang_name[$settings["default_lang"]];

                $page["language_list"] = dropdown("language", $lang_name, $permissions["lang"]);

                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/languages.tpl", $page);
            }
        break;
        case "update_lang":
            /* ----------------------------
                    Save the language
               ---------------------------- */
            $page["caption"] = $page["language_caption"];

            if (is_guest() == true || $permissions["ucp"] == 0)
            {
                $page["msg"]    = $page["no_tools_permission"];
                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
            else
            {

                // If the user tries to enter an invalid language use the default value
                if (empty($lang_name[$input["language"]]) == true)
                {
                    $input["language"] = "0";
                }

                build_query($queries["update_language"], array("LANGUAGE" => $input["language"], "UID" => $permissions["uid"]));
                $result = db_query($queries["update_language"], $link_id);

                // Check if the query ran
                if (empty($result) == true)
                {
                    $page["msg"] = $page["error_save_language"];
                }
                else
                {
                    $page["msg"] = $page["language_saved"];
                }

                $page["usercp"] = static_template($settings["script_dir"] . "skin/usercp/msg.tpl", $page);
            }
        break;
        default:
            $blocks = get_blocks($settings["script_dir"] . "skin/usercp/main.tpl", array("permission", "filetypes", "editable"));

            // Construct the array of permissions to check (PROBABLY NEED TO UPDATE THIS!!! extract etc)
            $to_check = array("upload"    => $page["can_upload"],
                              "download"  => $page["can_download"],
                              "htpasswd"  => preg_match("/apache/i", $_SERVER["SERVER_SOFTWARE"]) == true ? $page["can_protect"] : "",
                              "chmod"     => is_win() == false ? $page["can_chmod"] : "",
                              "bookmark"  => $page["can_bookmark"],
                              "rename"    => $page["can_rename"],
                              "edit"      => $page["can_edit"],
                              "source"    => $page["can_view"],
                              "delete"    => $page["can_delete"],
                              "copy"      => $page["can_copy"],
                              "move"      => $page["can_move"],
                              "newfolder" => $page["can_mkdir"],
                              "newfile"   => $page["can_mkfile"],
                              "overwrite" => $page["can_overwrite"],
                              "archive"   => $page["can_archive"],
                              "sql"       => $page["can_sql"],
                              "admin"     => $page["access_acp"],
                              "offline"   => $page["access_offline"]
                         );

            // Parse the dynamic block
            $data = "";
            foreach ($to_check as $key => $var)
            {
                if ($permissions[$key] == 1 && empty($var) == false)
                {
                    $data .= parse_html(array("permission" => $var), $blocks["permission"]);
                }
            }
            $blocks["permission"] = $data;

            if (empty($permissions["files"]) == true)
            {
                $blocks["filetypes"] = "";
            }

            // Build the list of ASCII file types, we could use the ASCII array but we need to take away the files the user may not access
            $tmp             = $settings["ascii"];
            $file_permission = explode(" ", $permissions["files"]);

            foreach ($tmp as $var)
            {
                if (file_permission("tmp." . $var) == true)
                {
                    $can_edit[] = $settings["filext.com"] == true ? "<a href=\"http://www.filext.com/detaillist.php?extdetail=" . $var . "\" target=\"_blank\" title=\"" . $page["ext_info"] . "\">" . $var . "</a>" : $var;
                }
            }

            // If the user can't edit, or there are no editable file types then there is no need to show that section
            if (($permissions["edit"] == 0 && $permissions["source"] == 0 && $permissions["newfile"] == 0) || isset($can_edit) == false)
            {
                $blocks["editable"] = "";
            }

            // If the admin has selected to show filext links we need to parse the list
            if ($settings["filext.com"] == true)
            {
                foreach ($file_permission as $key => $var)
                {
                    $file_permission[$key] = "<a href=\"http://www.filext.com/detaillist.php?extdetail=" . $var . "\" target=\"_blank\" title=\"" . $page["ext_info"] . "\">" . $var . "</a>";
                }
            }

            $page["email"]      = $permissions["email"];
            $page["allow"]      = $permissions["access"] == DISALLOW ? $page["not_allowed_filetypes"] : $page["allowed_filetypes"];
            $page["allow_list"] = build_list($file_permission);
            $page["edit_list"]  = empty($blocks["editable"]) == false ? build_list($can_edit) : "";
            $page["caption"]    = $page["main_caption"];
            $page["usercp"]     = parse_html($page, dynamic_template($settings["script_dir"] . "skin/usercp/main.tpl", $blocks));
    }

    // Construct the page
    $blocks              = get_blocks($settings["script_dir"] . "skin/usercp/index.tpl", array("bookmarks", "stats", "tools"));

    $blocks["bookmarks"] = $permissions["bookmark"] == 0 ? "" : $blocks["bookmarks"];
    $blocks["stats"]     = $permissions["stats"]    == 0 ? "" : $blocks["stats"];
    $blocks["tools"]     = $permissions["ucp"]      == 0 ? "" : $blocks["tools"];

    $page["content"]     = parse_html($page, dynamic_template($settings["script_dir"] . "skin/usercp/index.tpl", $blocks));
}

$page["js"]      = "cp";

?>